@@ -1,8 +1,8 @@
 // -*-c++-*-
 
 /*!
-	\file feditor_canvas.cpp
-	\brief formation editor canvas class Source File.
+  \file feditor_canvas.cpp
+  \brief formation editor canvas class Source File.
 */
 
 /*
@@ -12,16 +12,16 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this code; see the file COPYING.	If not, write to
+ along with this code; see the file COPYING.  If not, write to
  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
  *EndCopyright:
@@ -46,10 +46,16 @@
 
 #include <rcsc/geom/triangle_2d.h>
 
+#include <rcsc/timer.h>
+
 #include <algorithm>
 #include <iostream>
+#include <limits>
 #include <cassert>
 
+using namespace rcsc;
+using namespace formation;
+
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -63,6 +69,8 @@ FEditCanvas::FEditCanvas( MainWindow * parent )
     , M_enlarged( true )
     , M_show_index( true )
     , M_show_circumcircle( false )
+    , M_show_background_data( true )
+      // foreground graphic context
     , M_field_brush( QColor( 31, 160, 31 ), Qt::SolidPattern )
     , M_field_dark_brush( QColor( 15, 143, 15 ), Qt::SolidPattern )
     , M_line_pen( QColor( 255, 255, 255 ), 1, Qt::SolidLine )
@@ -77,8 +85,19 @@ FEditCanvas::FEditCanvas( MainWindow * parent )
     , M_select_pen( Qt::white, 2, Qt::SolidLine )
     , M_left_team_brush( QColor( 255, 215, 0 ), Qt::SolidPattern )
     , M_right_team_brush( QColor( 240, 20, 20 ), Qt::SolidPattern )
-    , M_synmetry_brush( QColor( 0, 255, 95 ), Qt::SolidPattern )
+    , M_symmetry_brush( QColor( 0, 255, 95 ), Qt::SolidPattern )
     , M_player_font( "Sans Serif", 10 )
+      // background graphic context
+    , M_background_contained_area_brush( QColor( 31, 143, 31 ), Qt::SolidPattern )
+    , M_background_triangle_pen( QColor( 0, 127, 255 ), 1, Qt::SolidLine )
+      //    , M_background_player_pen( QColor( 127, 127, 127 ), 1, Qt::SolidLine )
+    , M_background_player_pen( QColor( 255, 255, 255 ), 1, Qt::SolidLine )
+      //    , M_background_left_team_brush( QColor( 192, 251, 0 ), Qt::SolidPattern )
+    , M_background_left_team_brush( QColor( 255, 0, 0 ), Qt::SolidPattern )
+    , M_background_right_team_brush( QColor( 127, 20, 20 ), Qt::SolidPattern )
+      //    , M_background_symmetry_brush( QColor( 0, 192, 31 ), Qt::SolidPattern )
+    , M_background_symmetry_brush( QColor( 127, 31, 192 ), Qt::SolidPattern )
+    , M_background_font_pen( QColor( 0, 63, 127 ), 1, Qt::SolidLine )
 {
     assert( parent );
 
@@ -148,8 +167,16 @@ FEditCanvas::paintEvent( QPaintEvent * event )
     painter.begin( &M_canvas_pixmap, this );
 
     drawField( painter );
-    drawTrainingData( painter );
+    if ( M_show_background_data )
+    {
+        drawBackgroundData( painter );
+    }
+    drawData( painter );
     drawPlayers( painter );
+    if ( M_show_background_data )
+    {
+        drawBackgroundPlayers( painter );
+    }
     drawBall( painter );
 
     painter.end();
@@ -170,6 +197,10 @@ FEditCanvas::drawField( QPainter & painter )
     painter.fillRect( painter.window(), M_field_brush );
 
     //--------------------------------------
+    if ( M_show_background_data )
+    {
+        drawBackgroundContainedArea( painter );
+    }
     drawContainedArea( painter );
 
     // set screen coordinates of field
@@ -259,13 +290,13 @@ FEditCanvas::drawContainedArea( QPainter & painter )
         return ;
     }
 
-    boost::shared_ptr< const rcsc::Formation > f = ptr->formation();
+    boost::shared_ptr< const Formation > f = ptr->formation();
     if ( ! f )
     {
         return;
     }
 
-    if ( f->methodName() == rcsc::FormationDT::name() )
+    if ( f->methodName() == FormationDT::name() )
     {
         if ( ptr->triangulation().vertices().size() < 3 )
         {
@@ -276,8 +307,12 @@ FEditCanvas::drawContainedArea( QPainter & painter )
         //          << std::endl;
 
         // draw polygon
-        const rcsc::DelaunayTriangulation::TrianglePtr tri
+//         MSecTimer timer;
+        const DelaunayTriangulation::Triangle * tri
             = ptr->triangulation().findTriangleContains( ptr->ball() );
+//         std::cerr << "triangulation.findTriangle elapsed "
+//                   << timer.elapsedReal() << " [ms]."
+//                   << std::endl;
         if ( tri )
         {
             painter.setPen( Qt::NoPen );
@@ -297,8 +332,8 @@ FEditCanvas::drawContainedArea( QPainter & painter )
             painter.drawConvexPolygon( vertices );
 
             // draw center point
-            rcsc::Vector2D center
-                = rcsc::Triangle2D::centroid( tri->vertex( 0 )->pos(),
+            Vector2D center
+                = Triangle2D::centroid( tri->vertex( 0 )->pos(),
                                               tri->vertex( 1 )->pos(),
                                               tri->vertex( 2 )->pos() );
 
@@ -309,7 +344,7 @@ FEditCanvas::drawContainedArea( QPainter & painter )
                               screenY( center.y ) - 1,
                               3, 3 );
 #if 0
-            center = rcsc::Triangle2D::incenter( tri->vertex( 0 )->pos(),
+            center = Triangle2D::incenter( tri->vertex( 0 )->pos(),
                                                  tri->vertex( 1 )->pos(),
                                                  tri->vertex( 2 )->pos() );
             painter.setPen( Qt::red );
@@ -344,22 +379,23 @@ FEditCanvas::drawContainedArea( QPainter & painter )
 
 */
 void
-FEditCanvas::drawTrainingData( QPainter & painter )
+FEditCanvas::drawData( QPainter & painter )
 {
     boost::shared_ptr< FEditData > ptr = M_data_ptr.lock();
-    if ( ! ptr )
+    if ( ! ptr
+         || ! ptr->samples() )
     {
         return;
     }
 
-    boost::shared_ptr< const rcsc::Formation > f = ptr->formation();
+    Formation::ConstPtr f = ptr->formation();
     if ( ! f )
     {
         return;
     }
 
-    if ( f->methodName() == rcsc::FormationDT::name()
-         || f->methodName() == rcsc::FormationKNN::name() )
+    if ( f->methodName() == FormationDT::name()
+         || f->methodName() == FormationKNN::name() )
     {
         // draw delaunay triangulation
 
@@ -373,11 +409,9 @@ FEditCanvas::drawTrainingData( QPainter & painter )
         painter.setPen( M_triangle_pen );
         painter.setBrush( Qt::NoBrush );
 
-        const std::map< int, rcsc::DelaunayTriangulation::EdgePtr >::const_iterator edge_end
-            = ptr->triangulation().edgeMap().end();
-        for ( std::map< int, rcsc::DelaunayTriangulation::EdgePtr >::const_iterator it
-                  = ptr->triangulation().edgeMap().begin();
-              it != edge_end;
+        const DelaunayTriangulation::EdgeCont::const_iterator e_end = ptr->triangulation().edges().end();
+        for ( DelaunayTriangulation::EdgeCont::const_iterator it = ptr->triangulation().edges().begin();
+              it != e_end;
               ++it )
         {
             painter.drawLine( screenX( it->second->vertex( 0 )->pos().x ),
@@ -393,17 +427,15 @@ FEditCanvas::drawTrainingData( QPainter & painter )
 
         const int radius = scale( 0.7 );
         int count = 0;
-        const std::list< rcsc::Formation::Snapshot >::const_iterator data_end
-            = ptr->trainingData().end();
-        for ( std::list< rcsc::Formation::Snapshot >::const_iterator it
-                  = ptr->trainingData().begin();
-              it != data_end;
+        const SampleDataSet::DataCont::const_iterator d_end = ptr->samples()->dataCont().end();
+        for ( SampleDataSet::DataCont::const_iterator it = ptr->samples()->dataCont().begin();
+              it != d_end;
               ++it )
         {
             int ix = screenX( it->ball_.x );
             int iy = screenY( it->ball_.y );
             //char buf[4];
-            //std::snprintf( buf, 4, "%2d", ++count );
+            //snprintf( buf, 4, "%2d", ++count );
             if ( M_show_index )
             {
                 painter.drawText( ix + radius, iy - radius,
@@ -424,11 +456,9 @@ FEditCanvas::drawTrainingData( QPainter & painter )
         const int r = scale( 0.5 );
 
         int count = 0;
-        const std::list< rcsc::Formation::Snapshot >::const_iterator data_end
-            = ptr->trainingData().end();
-        for ( std::list< rcsc::Formation::Snapshot >::const_iterator it
-                  = ptr->trainingData().begin();
-              it != data_end;
+        const SampleDataSet::DataCont::const_iterator d_end = ptr->samples()->dataCont().end();
+        for ( SampleDataSet::DataCont::const_iterator it = ptr->samples()->dataCont().begin();
+              it != d_end;
               ++it )
         {
             int x = screenX( it->ball_.x );
@@ -439,7 +469,7 @@ FEditCanvas::drawTrainingData( QPainter & painter )
             painter.drawLine( x - r, y + r,
                               x + r, y - r );
             //char buf[4];
-            //std::snprintf( buf, 4, "%2d", ++count );
+            //snprintf( buf, 4, "%2d", ++count );
             if ( M_show_index )
             {
                 painter.drawText( x + r, y - r,
@@ -449,17 +479,15 @@ FEditCanvas::drawTrainingData( QPainter & painter )
 
     }
 
-    if ( f->methodName() == rcsc::FormationKNN::name() )
+    if ( f->methodName() == FormationKNN::name() )
     {
-        const int k = std::min( size_t( 3 ), ptr->trainingData().size() );
+        const int k = std::min( size_t( 3 ), ptr->samples()->dataCont().size() );
 
-        std::vector< rcsc::Vector2D > points( k );
+        std::vector< Vector2D > points( k );
         std::vector< double > dists( k, std::numeric_limits< double >::max() );
 
-        const std::list< rcsc::Formation::Snapshot >::const_iterator end
-            = ptr->trainingData().end();
-        for ( std::list< rcsc::Formation::Snapshot >::const_iterator it
-                  = ptr->trainingData().begin();
+        const SampleDataSet::DataCont::const_iterator end = ptr->samples()->dataCont().end();
+        for ( SampleDataSet::DataCont::const_iterator it = ptr->samples()->dataCont().begin();
               it != end;
               ++it )
         {
@@ -512,21 +540,22 @@ FEditCanvas::drawPlayers( QPainter & painter )
         return;
     }
 
-    boost::shared_ptr< const rcsc::Formation > f = ptr->formation();
+    boost::shared_ptr< const Formation > f = ptr->formation();
     if ( ! f )
     {
         return;
     }
 
     const int radius = ( M_enlarged
-                         ? scale( 1.1 )
+                         ? scale( 1.085 )
                          : scale( 0.3 ) );
     const int diameter = radius * 2;;
 
-    const int kickable_radius = scale( 1.1 );
+    const int kickable_radius = scale( 1.085 );
     const int kickable_diameter = kickable_radius * 2;
 
-    const int catchable_radius = scale( 2.0 );
+    //const int catchable_radius = scale( 2.0 );
+    const int catchable_radius = scale( 1.3 );
 
     const int selected_radius = radius + 3;
 
@@ -539,13 +568,13 @@ FEditCanvas::drawPlayers( QPainter & painter )
     {
         painter.setBrush( M_right_team_brush );
 
-        const std::vector< rcsc::Vector2D >::const_iterator
+        const std::vector< Vector2D >::const_iterator
             selection = ( ptr->getSelectType() == FEditData::SELECT_OPP
                           ? ( ptr->getOppPlayersPos().begin()
                               + ptr->getSelectIndex() )
                           : ptr->getOppPlayersPos().end() );
         int unum = 1;
-        for ( std::vector< rcsc::Vector2D >::const_iterator
+        for ( std::vector< Vector2D >::const_iterator
                   it = ptr->getOppPlayersPos().begin();
               it != ptr->getOppPlayersPos().end();
               ++it, ++unum )
@@ -569,7 +598,7 @@ FEditCanvas::drawPlayers( QPainter & painter )
                                      diameter );
             }
             //char buf[4];
-            //std::snprintf( buf, 4, "%2d", unum );
+            //snprintf( buf, 4, "%2d", unum );
             painter.DrawText( ix + radius, iy - radius,
                               QString().sprintf( "%2d", unum ) );
             //QString::fromAscii( buf ) );
@@ -580,13 +609,13 @@ FEditCanvas::drawPlayers( QPainter & painter )
     ////////////////////////////////////////////////////////////
     // draw left players
     {
-        const std::vector< rcsc::Vector2D >::const_iterator
+        const std::vector< Vector2D >::const_iterator
             selection = ( ptr->getSelectType() == FEditData::SELECT_OUR
                           ? ( ptr->ourPlayers().begin()
                               + ptr->getSelectIndex() )
                           : ptr->ourPlayers().end() );
         int unum = 1;
-        for ( std::vector< rcsc::Vector2D >::const_iterator
+        for ( std::vector< Vector2D >::const_iterator
                   it = ptr->ourPlayers().begin();
               it != ptr->ourPlayers().end();
               ++it, ++unum )
@@ -596,18 +625,18 @@ FEditCanvas::drawPlayers( QPainter & painter )
             if ( selection == it )
             {
                 painter.setPen( M_select_pen );
-                painter.setBrush( f->isSynmetryType( unum )
-                                  ? M_synmetry_brush
+                painter.setBrush( f->isSymmetryType( unum )
+                                  ? M_symmetry_brush
                                   : M_left_team_brush );
                 painter.drawEllipse( ix - selected_radius,
                                      iy - selected_radius,
                                      selected_radius * 2,
                                      selected_radius * 2 );
             }
-            else if ( f->isSynmetryType( unum ) )
+            else if ( f->isSymmetryType( unum ) )
             {
                 painter.setPen( M_player_pen );
-                painter.setBrush( M_synmetry_brush );
+                painter.setBrush( M_symmetry_brush );
                 painter.drawEllipse( ix - radius,
                                      iy - radius,
                                      diameter,
@@ -711,6 +740,215 @@ FEditCanvas::drawBall( QPainter & painter )
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+FEditCanvas::drawBackgroundContainedArea( QPainter & painter )
+{
+    boost::shared_ptr< FEditData > ptr = M_data_ptr.lock();
+    if ( ! ptr )
+    {
+        return ;
+    }
+
+    if ( ptr->backgroundTriangulation().vertices().size() < 3 )
+    {
+        return;
+    }
+
+    const DelaunayTriangulation::Triangle * tri
+            = ptr->backgroundTriangulation().findTriangleContains( ptr->ball() );
+
+    if ( ! tri )
+    {
+        return;
+    }
+
+    painter.setPen( Qt::NoPen );
+    painter.setBrush( M_background_contained_area_brush );
+
+    QPointArray vertices( 3 );
+    vertices.setPoint( 0,
+                       screenX( tri->vertex( 0 )->pos().x ),
+                       screenY( tri->vertex( 0 )->pos().y ) );
+    vertices.setPoint( 1,
+                       screenX( tri->vertex( 1 )->pos().x ),
+                       screenY( tri->vertex( 1 )->pos().y ) );
+    vertices.setPoint( 2,
+                       screenX( tri->vertex( 2 )->pos().x ),
+                       screenY( tri->vertex( 2 )->pos().y ) );
+
+    painter.drawConvexPolygon( vertices );
+#if 0
+    // draw center point
+    Vector2D center
+        = Triangle2D::centroid( tri->vertex( 0 )->pos(),
+                                      tri->vertex( 1 )->pos(),
+                                      tri->vertex( 2 )->pos() );
+
+
+    painter.setPen( Qt::red );
+    painter.setBrush( Qt::red );
+    painter.drawRect( screenX( center.x ) - 1,
+                      screenY( center.y ) - 1,
+                      3, 3 );
+#endif
+#if 0
+    center = Triangle2D::incenter( tri->vertex( 0 )->pos(),
+                                         tri->vertex( 1 )->pos(),
+                                         tri->vertex( 2 )->pos() );
+    painter.setPen( Qt::red );
+    painter.setBrush( Qt::red );
+    painter.drawRect( screenX( center.x ) - 1,
+                      screenY( center.y ) - 1,
+                      3, 3 );
+#endif
+#if 0
+    // draw circum circle of this triangle
+    if ( M_show_circumcircle )
+    {
+        int x = screenX( tri->circumcenter().x );
+        int y = screenY( tri->circumcenter().y );
+        int r = scale( tri->circumradius() );
+
+        painter.setPen( Qt::cyan );
+        painter.setBrush( Qt::cyan );
+        painter.drawRect( x - 1, y - 1,
+                          3, 3 );
+        painter.setBrush( Qt::NoBrush );
+        painter.drawEllipse( x - r, y - r,
+                             r * 2, r * 2 );
+    }
+#endif
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+FEditCanvas::drawBackgroundData( QPainter & painter )
+{
+    boost::shared_ptr< FEditData > ptr = M_data_ptr.lock();
+    if ( ! ptr )
+    {
+        return;
+    }
+
+    if ( ptr->backgroundTriangulation().vertices().size() < 3 )
+    {
+        // too few kernel points
+        // no valid triangulation
+        return;
+    }
+
+    painter.setPen( M_background_triangle_pen );
+    painter.setBrush( Qt::NoBrush );
+
+    const DelaunayTriangulation::EdgeCont::const_iterator e_end = ptr->backgroundTriangulation().edges().end();
+    for ( DelaunayTriangulation::EdgeCont::const_iterator it = ptr->backgroundTriangulation().edges().begin();
+          it != e_end;
+          ++it )
+    {
+        painter.drawLine( screenX( it->second->vertex( 0 )->pos().x ),
+                          screenY( it->second->vertex( 0 )->pos().y ),
+                          screenX( it->second->vertex( 1 )->pos().x ),
+                          screenY( it->second->vertex( 1 )->pos().y ) );
+    }
+
+    painter.setFont( M_player_font );
+    painter.setPen( M_background_font_pen );
+    //painter.setPen( Qt::black );
+    painter.setBackgroundMode( Qt::TransparentMode );
+
+    const int radius = scale( 0.35 );
+    int count = 0;
+    const SampleDataSet::DataCont::const_iterator d_end = ptr->backgroundFormation()->samples()->dataCont().end();
+    for ( SampleDataSet::DataCont::const_iterator it = ptr->backgroundFormation()->samples()->dataCont().begin();
+          it != d_end;
+          ++it )
+    {
+        int ix = screenX( it->ball_.x );
+        int iy = screenY( it->ball_.y );
+        //char buf[4];
+            //snprintf( buf, 4, "%2d", ++count );
+        if ( M_show_index )
+        {
+            painter.drawText( ix + radius, iy - radius,
+                              QString().sprintf( "%2d", ++count ) ); //QString::fromAscii( buf ) );
+        }
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+FEditCanvas::drawBackgroundPlayers( QPainter & painter )
+{
+    boost::shared_ptr< FEditData > ptr = M_data_ptr.lock();
+    if ( ! ptr )
+    {
+        return;
+    }
+
+    Formation::ConstPtr f = ptr->backgroundFormation();
+    if ( ! f )
+    {
+        return;
+    }
+
+    const int radius = ( M_enlarged
+                         ? scale( 1.085 * 0.5 )
+                         : scale( 0.3 * 0.5 ) );
+    const int diameter = radius * 2;;
+
+    ////////////////////////////////////////////////////////////
+    painter.setFont( M_player_font );
+    painter.setBackgroundMode( Qt::TransparentMode );
+    ////////////////////////////////////////////////////////////
+    // draw left players
+    {
+        std::vector< Vector2D > players;
+        players.reserve( 11 );
+        f->getPositions( ptr->ball(), players );
+
+        int unum = 1;
+        for ( std::vector< Vector2D >::const_iterator it = players.begin();
+              it != players.end();
+              ++it, ++unum )
+        {
+            int ix = screenX( it->x );
+            int iy = screenY( it->y );
+            if ( f->isSymmetryType( unum ) )
+            {
+                painter.setPen( M_background_player_pen );
+                painter.setBrush( M_background_symmetry_brush );
+                painter.drawEllipse( ix - radius,
+                                     iy - radius,
+                                     diameter,
+                                     diameter );
+            }
+            else
+            {
+                painter.setPen( M_background_player_pen );
+                painter.setBrush( M_background_left_team_brush );
+                painter.drawEllipse( ix - radius,
+                                     iy - radius,
+                                     diameter,
+                                     diameter );
+            }
+
+            painter.setPen( Qt::gray );
+            painter.drawText( ix + radius, iy - radius,
+                              QString().sprintf( "%2d", unum ) );
+        }
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 FEditCanvas::setFocusPoint( const QPoint & pos )
@@ -805,6 +1043,18 @@ FEditCanvas::toggleShowCircumcircle( bool on )
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+FEditCanvas::toggleShowBackgroundData( bool on )
+{
+    M_show_background_data = on;
+
+    this->update();
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 FEditCanvas::mouseDoubleClickEvent( QMouseEvent * event )
@@ -902,7 +1152,7 @@ FEditCanvas::mouseMoveEvent( QMouseEvent * event )
     double field_x = fieldX( point.x() );
     double field_y = fieldY( point.y() );
     //char buf[32];
-    //std::snprintf( buf, 32, "(%.2f %.2f)", field_x, field_y );
+    //snprintf( buf, 32, "(%.2f %.2f)", field_x, field_y );
 
     if ( M_mouse_state[0].isDragged() )
     {
