@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -108,6 +108,7 @@ FEditDialog::createWidgets()
             M_ball_pos_x = new QLineEdit( tr( "0" ), box );
             M_ball_pos_x->setMinimumSize( 48, 24 );
             M_ball_pos_x->setMaximumSize( 64, 24 );
+            M_ball_pos_x->setValidator( new QDoubleValidator( -54.5, 54.5, 2, M_ball_pos_x ) );
         }
         {
             QLabel * label = new QLabel( tr( " Y:" ), box );
@@ -118,6 +119,7 @@ FEditDialog::createWidgets()
             M_ball_pos_y = new QLineEdit( tr( "0" ), box );
             M_ball_pos_y->setMinimumSize( 48, 24 );
             M_ball_pos_y->setMaximumSize( 64, 24 );
+            M_ball_pos_y->setValidator( new QDoubleValidator( -34.0, 34.0, 2, M_ball_pos_y ) );
         }
 //         {
 //             QRect rect = this->fontMetrics().boundingRect( tr( "ReverseY" ) );
@@ -131,7 +133,7 @@ FEditDialog::createWidgets()
     }
 
     const int unum_col_width = this->fontMetrics().width( tr( "Unum" ) ) + 4;
-    const int synmetry_col_width = this->fontMetrics().width( tr( "Symmetry" ) ) + 4;
+    const int symmetry_col_width = this->fontMetrics().width( tr( "Symmetry" ) ) + 4;
 
     // label info
     {
@@ -145,8 +147,8 @@ FEditDialog::createWidgets()
         {
             QLabel * label = new QLabel( tr( "Symmetry" ), box );
             label->setAlignment( Qt::AlignCenter );
-            label->setMinimumSize( synmetry_col_width, 24 );
-            label->setMaximumSize( synmetry_col_width, 24 );
+            label->setMinimumSize( symmetry_col_width, 24 );
+            label->setMaximumSize( symmetry_col_width, 24 );
         }
         {
             QLabel * label = new QLabel( tr( "Role" ), box );
@@ -181,9 +183,9 @@ FEditDialog::createWidgets()
             label->setMaximumSize( unum_col_width + 8, 24 );
         }
         {
-            M_synmetry_unum[i] = new QLineEdit( tr( "0" ), box );
-            M_synmetry_unum[i]->setMaximumSize( synmetry_col_width, 24 );
-            M_synmetry_unum[i]->setMinimumSize( synmetry_col_width, 24 );
+            M_symmetry_unum[i] = new QLineEdit( tr( "0" ), box );
+            M_symmetry_unum[i]->setMaximumSize( symmetry_col_width, 24 );
+            M_symmetry_unum[i]->setMinimumSize( symmetry_col_width, 24 );
         }
         {
             M_role_name[i] = new QLineEdit( tr( "Role" ), box );
@@ -195,11 +197,13 @@ FEditDialog::createWidgets()
             M_pos_x[i] = new QLineEdit( tr( "0.0" ), box );
             M_pos_x[i]->setMaximumSize( this->fontMetrics().width( tr( "-00.0000" ) ) + 4,
                                         24 );
+            M_pos_x[i]->setValidator( new QDoubleValidator( -54.5, 54.5, 2, M_pos_x[i] ) );
         }
         {
             M_pos_y[i] = new QLineEdit( tr( "0.0" ), box );
             M_pos_y[i]->setMaximumSize( this->fontMetrics().width( tr( "-00.0000" ) ) + 4,
                                         24 );
+            M_pos_y[i]->setValidator( new QDoubleValidator( -36.0, 36.0, 2, M_pos_y[i] ) );
         }
 
         top_vbox->addWidget( box );
@@ -226,7 +230,7 @@ FEditDialog::setDefaultValues()
 {
     for ( int i = 0; i < 11; ++i )
     {
-        M_synmetry_unum[i]->setText( tr( "0" ) );
+        M_symmetry_unum[i]->setText( tr( "0" ) );
         M_role_name[i]->setText( tr( "Role" ) );
         M_pos_x[i]->setText( tr( "0.0" ) );
         M_pos_y[i]->setText( tr( "0.0" ) );
@@ -270,10 +274,10 @@ FEditDialog::updateData()
     char buf[32];
 
     // ball info
-    std::snprintf( buf, 32, "%.2f", data_ptr->ball().x );
+    snprintf( buf, 32, "%.2f", data_ptr->ball().x );
     M_ball_pos_x->setText( QString::fromAscii( buf ) );
 
-    std::snprintf( buf, 32, "%.2f", data_ptr->ball().y );
+    snprintf( buf, 32, "%.2f", data_ptr->ball().y );
     M_ball_pos_y->setText( QString::fromAscii( buf ) );
 
     // player info
@@ -281,24 +285,24 @@ FEditDialog::updateData()
     const int MAX = positions.size();
     for ( int unum = 1; unum <= MAX; ++unum )
     {
-        const bool synmetry = f->isSynmetryType( unum );
+        //const bool symmetry = f->isSymmetryType( unum );
 
-        std::snprintf( buf, 32, "%2d", f->getSynmetryNumber( unum ) );
-        M_synmetry_unum[unum - 1]->setText( QString::fromAscii( buf ) );
+        snprintf( buf, 32, "%2d", f->getSymmetryNumber( unum ) );
+        M_symmetry_unum[unum - 1]->setText( QString::fromAscii( buf ) );
 
         M_role_name[unum - 1]->setText( QString::fromAscii( f->getRoleName( unum ).c_str() ) );
 
-        std::snprintf( buf, 32, "%.2f", positions[unum - 1].x );
+        snprintf( buf, 32, "%.2f", positions[unum - 1].x );
         M_pos_x[unum - 1]->setText( QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 32, "%.2f", positions[unum - 1].y );
+        snprintf( buf, 32, "%.2f", positions[unum - 1].y );
         M_pos_y[unum - 1]->setText( QString::fromAscii( buf ) );
 
-        const Qt::BackgroundMode mode = ( synmetry
-                                          ? Qt::PaletteDark
-                                          : Qt::PaletteBackground );
-        M_role_name[unum - 1]->setReadOnly( synmetry );
-        M_role_name[unum - 1]->setBackgroundMode( mode );
+        //const Qt::BackgroundMode mode = ( symmetry
+        //                                  ? Qt::PaletteDark
+        //                                  : Qt::PaletteBackground );
+        //M_role_name[unum - 1]->setReadOnly( symmetry );
+        //M_role_name[unum - 1]->setBackgroundMode( mode );
         //M_pos_x[unum - 1]->setBackgroundMode( mode );
         //M_pos_y[unum - 1]->setBackgroundMode( mode );
     }
@@ -314,7 +318,7 @@ FEditDialog::reverseY()
     boost::shared_ptr< FEditData > data_ptr = M_data_ptr.lock();
     if ( ! data_ptr )
     {
-        std::cerr << "FEditDialog::handleSynmetryBall  no data ptr" << std::endl;
+        std::cerr << "FEditDialog::handleSymmetryBall  no data ptr" << std::endl;
         return;
     }
 
@@ -355,11 +359,11 @@ FEditDialog::applyToField()
     for ( int unum = 1; unum <= 11; ++unum )
     {
         bool ok = false;
-        int synmetry_unum = M_synmetry_unum[unum-1]->text().toInt( &ok );
+        int symmetry_unum = M_symmetry_unum[unum-1]->text().toInt( &ok );
         if ( ! ok )
         {
             std::cerr << __FILE__ << ':' << __LINE__
-                      << " *** ERROR *** Invalid synmetry number."
+                      << " *** ERROR *** Invalid symmetry number."
                       << std::endl;
             continue;
         }
@@ -374,7 +378,7 @@ FEditDialog::applyToField()
             continue;
         }
 
-        data_ptr->updateRoleData( unum, synmetry_unum, role_name );
+        data_ptr->updateRoleData( unum, symmetry_unum, role_name );
 
         bool ok_x = false;
         bool ok_y = false;
