@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -77,7 +77,7 @@
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 MainWindow::MainWindow()
 {
     //    QPixmapCache::setCacheLimit( 1024 * 24 );
@@ -134,7 +134,7 @@ MainWindow::MainWindow()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 MainWindow::~MainWindow()
 {
     //std::cerr << "delete MainWindow" << std::endl;
@@ -146,38 +146,39 @@ MainWindow::~MainWindow()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::init()
 {
-//     M_editor_dialog->show();
+    //     M_editor_dialog->show();
 
-//     QSize size = M_editor_dialog->size();
-//     M_editor_dialog->setMinimumSize( size );
-//     M_editor_dialog->setMaximumSize( size );
+    //     QSize size = M_editor_dialog->size();
+    //     M_editor_dialog->setMinimumSize( size );
+    //     M_editor_dialog->setMaximumSize( size );
 
     if ( ! FEditConfig::instance().confFile().empty() )
     {
-        if ( ! FEditConfig::instance().dataFile().empty() )
-        {
-            openConfFile( FEditConfig::instance().confFile() );
-        }
-        else
-        {
-            open( FEditConfig::instance().confFile() );
-        }
+        openConfFile( FEditConfig::instance().confFile() );
     }
 
     if ( ! FEditConfig::instance().dataFile().empty() )
     {
         openDataFile( FEditConfig::instance().dataFile() );
     }
+
+    //
+    // open the background formation & data.
+    //
+    if ( ! FEditConfig::instance().backgroundConfFile().empty() )
+    {
+        openBackgroundConfFile( FEditConfig::instance().backgroundConfFile() );
+    }
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::loadSettings()
 {
@@ -202,7 +203,7 @@ MainWindow::loadSettings()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::saveSettings()
 {
@@ -225,7 +226,7 @@ MainWindow::saveSettings()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::createActions()
 {
@@ -239,7 +240,7 @@ MainWindow::createActions()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::createActionsFile()
 {
@@ -251,56 +252,55 @@ MainWindow::createActionsFile()
     M_new_file_act->setStatusTip( tr( "Create new formation data" ) );
     connect( M_new_file_act, SIGNAL( activated() ), this, SLOT( newFile() ) );
     //
-    M_open_act = new QAction( QIconSet( QPixmap( open_xpm ) ),
-                              tr( "&Open formation" ),
-                              QKeySequence(),
-                              this );
-    M_open_act->setAccel( tr( "Ctrl+O" ) );
-    M_open_act->setStatusTip( tr( "Open formation file" ) );
-    connect( M_open_act, SIGNAL( activated() ), this, SLOT( open() ) );
+    M_open_conf_act = new QAction( QIconSet( QPixmap( open_xpm ) ),
+                                   tr( "&Open formation" ),
+                                   QKeySequence(),
+                                   this );
+    M_open_conf_act->setAccel( tr( "Ctrl+O" ) );
+    M_open_conf_act->setStatusTip( tr( "Open formation file" ) );
+    connect( M_open_conf_act, SIGNAL( activated() ), this, SLOT( openConf() ) );
     //
-    M_open_train_act = new QAction( QIconSet( QPixmap( open_xpm ) ),
-                                    tr( "&Open training data." ),
-                                    QKeySequence(),
-                                    this );
-    M_open_train_act->setAccel( tr( "Ctrl+Alt+O" ) );
-    M_open_train_act->setStatusTip( tr( "Open training data file" ) );
-    connect( M_open_train_act, SIGNAL( activated() ),
+    M_open_data_act = new QAction( QIconSet( QPixmap( open_xpm ) ),
+                                   tr( "&Open training data." ),
+                                   QKeySequence(),
+                                   this );
+    M_open_data_act->setAccel( tr( "Ctrl+Alt+O" ) );
+    M_open_data_act->setStatusTip( tr( "Open training data file" ) );
+    connect( M_open_data_act, SIGNAL( activated() ),
              this, SLOT( openData() ) );
     //
-    M_save_all_act = new QAction( QIconSet( QPixmap( save_xpm ) ),
-                                  tr( "&Save all" ),
-                                  QKeySequence(),
-                                  this );
-    M_save_all_act->setAccel( tr( "Ctrl+S" ) );
-    M_save_all_act->setStatusTip( tr( "Save all data" ) );
-    connect( M_save_all_act, SIGNAL( activated() ), this, SLOT( saveAll() ) );
+    M_open_background_conf_act = new QAction( QIconSet( QPixmap( open_xpm ) ),
+                                              tr( "&Open background formation" ),
+                                              QKeySequence(),
+                                              this );
+    M_open_background_conf_act->setStatusTip( tr( "Open background formation file" ) );
+    connect( M_open_background_conf_act, SIGNAL( activated() ),
+             this, SLOT( openBackgroundConf() ) );
+
     //
-    M_save_act = new QAction( tr( "&Save formation" ),
+    M_save_act = new QAction( QIconSet( QPixmap( save_xpm ) ),
+                              tr( "&Save formation" ),
                               QKeySequence(),
                               this );
-    M_save_act->setStatusTip( tr( "Save formation data" ) );
-    connect( M_save_act, SIGNAL( activated() ), this, SLOT( save() ) );
+    M_save_act->setAccel( tr( "Ctrl+S" ) );
+    M_save_act->setStatusTip( tr( "Save to the current file." ) );
+    connect( M_save_act, SIGNAL( activated() ), this, SLOT( saveConf() ) );
+
     //
     M_save_as_act = new QAction( tr( "&Save formation as..." ),
-                              QKeySequence(),
-                              this );
-    M_save_as_act->setStatusTip( tr( "Save formation data to the new file" ) );
-    connect( M_save_as_act, SIGNAL( activated() ), this, SLOT( saveAs() ) );
+                                 QKeySequence(),
+                                 this );
+    M_save_as_act->setStatusTip( tr( "Save to the new file." ) );
+    connect( M_save_as_act, SIGNAL( activated() ), this, SLOT( saveConfAs() ) );
+
     //
-    M_save_train_act = new QAction( tr( "&Save training data" ),
-                              QKeySequence(),
-                              this );
-    M_save_train_act->setStatusTip( tr( "Save training data" ) );
-    connect( M_save_train_act, SIGNAL( activated() ),
-             this, SLOT( saveTrainingData() ) );
-    //
-    M_save_train_as_act = new QAction( tr( "&Save training data as..." ),
-                              QKeySequence(),
-                              this );
-    M_save_train_as_act->setStatusTip( tr( "Save training data to the new file" ) );
-    connect( M_save_train_as_act, SIGNAL( activated() ),
-             this, SLOT( saveTrainingDataAs() ) );
+    M_save_data_as_act = new QAction( QIconSet( QPixmap( save_xpm ) ),
+                                      tr( "&Save training data" ),
+                                      QKeySequence(),
+                                      this );
+    M_save_data_as_act->setStatusTip( tr( "Save training data only." ) );
+    connect( M_save_data_as_act, SIGNAL( activated() ), this, SLOT( saveDataAs() ) );
+
     //
     M_exit_act = new QAction( tr( "&Quit" ),
                               QKeySequence(),
@@ -313,7 +313,7 @@ MainWindow::createActionsFile()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::createActionsMode()
 {
@@ -350,16 +350,16 @@ MainWindow::createActionsMode()
     connect( M_toggle_data_auto_select_act, SIGNAL( toggled( bool ) ),
              this, SLOT( toggleDataAutoSelect( bool ) ) );
     //
-    M_toggle_symmetric_mode_act = new QAction( QIconSet( QPixmap( symmetry_xpm ) ),
-                                               tr( "Symmetry" ),
-                                               QKeySequence(),
-                                               this );
-    M_toggle_symmetric_mode_act->setAccel( Qt::Key_S );
-    M_toggle_symmetric_mode_act->setStatusTip( tr( "Toggle symmetrical mode." ) );
-    M_toggle_symmetric_mode_act->setToggleAction( true );
-    M_toggle_symmetric_mode_act->setOn( true );
-    connect( M_toggle_symmetric_mode_act, SIGNAL( toggled( bool ) ),
-             this, SLOT( toggleSymmetricMode( bool ) ) );
+    M_toggle_symmetry_mode_act = new QAction( QIconSet( QPixmap( symmetry_xpm ) ),
+                                              tr( "Symmetry" ),
+                                              QKeySequence(),
+                                              this );
+    M_toggle_symmetry_mode_act->setAccel( Qt::Key_S );
+    M_toggle_symmetry_mode_act->setStatusTip( tr( "Toggle symmetry mode." ) );
+    M_toggle_symmetry_mode_act->setToggleAction( true );
+    M_toggle_symmetry_mode_act->setOn( true );
+    connect( M_toggle_symmetry_mode_act, SIGNAL( toggled( bool ) ),
+             this, SLOT( toggleSymmetryMode( bool ) ) );
     //
     M_show_edit_dialog_act = new QAction( tr( "Show Edit Dialog" ),
                                           QKeySequence(),
@@ -375,14 +375,14 @@ MainWindow::createActionsMode()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::createActionsData()
 {
     M_insert_data_act = new QAction( QIconSet( QPixmap( insert_xpm ) ),
-                                      tr( "Insert" ),
-                                      QKeySequence(),
-                                      this );
+                                     tr( "Insert" ),
+                                     QKeySequence(),
+                                     this );
     M_insert_data_act->setStatusTip( tr( "Insert the screen data after the current index" ) );
     connect( M_insert_data_act, SIGNAL( activated() ),
              this, SLOT( insertData() ) );
@@ -423,9 +423,9 @@ MainWindow::createActionsData()
              this, SLOT( train() ) );
     //
     M_record_data_act = new QAction( QIconSet( QPixmap( record_xpm ) ),
-                               tr( "Record" ),
-                               QKeySequence(),
-                               this );
+                                     tr( "Record" ),
+                                     QKeySequence(),
+                                     this );
     M_record_data_act->setAccel( tr( "Ctrl+R" ) );
     M_record_data_act->setStatusTip( tr( "Record current field status as a training data" ) );
     connect( M_record_data_act, SIGNAL( activated() ),
@@ -435,7 +435,7 @@ MainWindow::createActionsData()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::createActionsView()
 {
@@ -446,7 +446,7 @@ MainWindow::createActionsView()
     M_zoom_in_act->setStatusTip( tr( "Zoom In" ) );
     connect( M_zoom_in_act, SIGNAL( activated() ),
              M_editor_canvas, SLOT( zoomIn() ) );
-
+    //
     M_zoom_out_act = new QAction( tr( "Zoom Out" ),
                                   QKeySequence(),
                                   this );
@@ -454,15 +454,15 @@ MainWindow::createActionsView()
     M_zoom_out_act->setStatusTip( tr( "Zoom Out" ) );
     connect( M_zoom_out_act, SIGNAL( activated() ),
              M_editor_canvas, SLOT( zoomOut() ) );
-
+    //
     M_fit_act = new QAction( tr( "Fit Size" ),
-                                  QKeySequence(),
-                                  this );
+                             QKeySequence(),
+                             this );
     M_fit_act->setAccel( Qt::Key_I );
     M_fit_act->setStatusTip( tr( "Fit Size" ) );
     connect( M_fit_act, SIGNAL( activated() ),
              M_editor_canvas, SLOT( fitSize() ) );
-
+    //
     M_toggle_enlarge_act = new QAction( tr( "Toggle Enlarge" ),
                                         QKeySequence(),
                                         this );
@@ -472,7 +472,7 @@ MainWindow::createActionsView()
     connect( M_toggle_enlarge_act, SIGNAL( toggled( bool ) ),
              M_editor_canvas, SLOT( toggleEnlarge( bool ) ) );
     M_toggle_enlarge_act->setOn( true );
-
+    //
     M_toggle_show_index_act = new QAction( tr( "Show Index" ),
                                            QKeySequence(),
                                            this );
@@ -481,7 +481,7 @@ MainWindow::createActionsView()
     connect( M_toggle_show_index_act, SIGNAL( toggled( bool ) ),
              M_editor_canvas, SLOT( toggleShowIndex( bool ) ) );
     M_toggle_show_index_act->setOn( true );
-
+    //
     M_toggle_show_circumcircle_act = new QAction( tr( "Show Circumcircle" ),
                                                   QKeySequence(),
                                                   this );
@@ -490,13 +490,21 @@ MainWindow::createActionsView()
     connect( M_toggle_show_circumcircle_act, SIGNAL( toggled( bool ) ),
              M_editor_canvas, SLOT( toggleShowCircumcircle( bool ) ) );
     M_toggle_show_circumcircle_act->setOn( false );
-
+    //
+    M_toggle_show_background_data_act = new QAction( tr( "Show Background Data" ),
+                                                     QKeySequence(),
+                                                     this );
+    M_toggle_show_background_data_act->setStatusTip( tr( "Show/Hide Background Data" ) );
+    M_toggle_show_background_data_act->setToggleAction( true );
+    connect( M_toggle_show_background_data_act, SIGNAL( toggled( bool ) ),
+             M_editor_canvas, SLOT( toggleShowBackgroundData( bool ) ) );
+    M_toggle_show_background_data_act->setOn( true );
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::createActionsHelp()
 {
@@ -511,7 +519,7 @@ MainWindow::createActionsHelp()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::createMenus()
 {
@@ -525,7 +533,7 @@ MainWindow::createMenus()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::createMenuFile()
 {
@@ -533,19 +541,22 @@ MainWindow::createMenuFile()
     menu->insertTearOffHandle();
 
     M_new_file_act->addTo( menu );
+
     menu->insertSeparator();
 
-    M_open_act->addTo( menu );
-    M_open_train_act->addTo( menu );
-    menu->insertSeparator();
-
-    M_save_all_act->addTo( menu );
+    M_open_conf_act->addTo( menu );
     M_save_act->addTo( menu );
-    M_save_train_act->addTo( menu );
+    M_save_as_act->addTo( menu );
+
     menu->insertSeparator();
 
-    M_save_as_act->addTo( menu );
-    M_save_train_as_act->addTo( menu );
+    M_open_background_conf_act->addTo( menu );
+
+    menu->insertSeparator();
+
+    M_open_data_act->addTo( menu );
+    M_save_data_as_act->addTo( menu );
+
     menu->insertSeparator();
 
     M_exit_act->addTo( menu );
@@ -556,7 +567,7 @@ MainWindow::createMenuFile()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::createMenuEdit()
 {
@@ -583,7 +594,7 @@ MainWindow::createMenuEdit()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::createMenuMode()
 {
@@ -592,7 +603,7 @@ MainWindow::createMenuMode()
     M_toggle_ball_drag_act->addTo( menu );
     M_toggle_player_auto_move_act->addTo( menu );
     M_toggle_data_auto_select_act->addTo( menu );
-    M_toggle_symmetric_mode_act->addTo( menu );
+    M_toggle_symmetry_mode_act->addTo( menu );
 
     menu->insertSeparator();
 
@@ -604,7 +615,7 @@ MainWindow::createMenuMode()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::createMenuView()
 {
@@ -619,6 +630,7 @@ MainWindow::createMenuView()
     M_toggle_enlarge_act->addTo( menu );
     M_toggle_show_index_act->addTo( menu );
     M_toggle_show_circumcircle_act->addTo( menu );
+    M_toggle_show_background_data_act->addTo( menu );
 
     menuBar()->insertItem( tr( "&View" ), menu );
 }
@@ -626,7 +638,7 @@ MainWindow::createMenuView()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::createMenuHelp()
 {
@@ -639,16 +651,16 @@ MainWindow::createMenuHelp()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::createToolBars()
 {
     M_tool_bar = new QToolBar( this );
 
-    M_save_all_act->addTo( M_tool_bar );
+    M_save_act->addTo( M_tool_bar );
     //M_toggle_ball_drag_act->addTo( M_tool_bar );
     M_toggle_player_auto_move_act->addTo( M_tool_bar );
-    M_toggle_symmetric_mode_act->addTo( M_tool_bar );
+    M_toggle_symmetry_mode_act->addTo( M_tool_bar );
 
     M_tool_bar->addSeparator();
 
@@ -674,7 +686,7 @@ MainWindow::createToolBars()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::createStatusBar()
 {
@@ -695,7 +707,7 @@ MainWindow::createStatusBar()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::createFieldCanvas()
 {
@@ -714,7 +726,7 @@ MainWindow::createFieldCanvas()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::createEditDialog()
 {
@@ -725,7 +737,7 @@ MainWindow::createEditDialog()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::closeEvent( QCloseEvent * event )
 {
@@ -742,9 +754,9 @@ MainWindow::closeEvent( QCloseEvent * event )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
-MainWindow::resizeEvent( QResizeEvent * event )
+MainWindow::resizeEvent( QResizeEvent * )
 {
     //event->accept(); qt4
 
@@ -753,7 +765,7 @@ MainWindow::resizeEvent( QResizeEvent * event )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::wheelEvent( QWheelEvent * event )
 {
@@ -793,7 +805,7 @@ MainWindow::wheelEvent( QWheelEvent * event )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::about()
 {
@@ -827,7 +839,7 @@ MainWindow::about()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::toggleToolBar()
 {
@@ -844,7 +856,7 @@ MainWindow::toggleToolBar()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::toggleStatusBar()
 {
@@ -861,7 +873,7 @@ MainWindow::toggleStatusBar()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::toggleFullScreen()
 {
@@ -878,7 +890,7 @@ MainWindow::toggleFullScreen()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::changeStyle( const QString & style )
 {
@@ -888,7 +900,7 @@ MainWindow::changeStyle( const QString & style )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::resizeCanvas( const QSize & size )
 {
@@ -930,7 +942,7 @@ MainWindow::resizeCanvas( const QSize & size )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::updatePositionLabel( const QString & str )
 {
@@ -947,7 +959,7 @@ MainWindow::updatePositionLabel( const QString & str )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::newFile()
 {
@@ -993,7 +1005,7 @@ MainWindow::newFile()
     M_editor_data = boost::shared_ptr< FEditData >( new FEditData() );
 
     M_editor_data->setTrainingTypeName( type_name );
-    M_editor_data->createDefaultParam();
+    M_editor_data->createDefaultData();
 
     if ( ! M_editor_data->formation() )
     {
@@ -1015,9 +1027,9 @@ MainWindow::newFile()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
-MainWindow::open()
+MainWindow::openConf()
 {
     if ( ! saveChanged() )
     {
@@ -1032,16 +1044,21 @@ MainWindow::open()
                                                      this,
                                                      tr( "Open Formation" ),
                                                      tr( "Choose a file name to open" ) );
+    if ( filepath.isEmpty() )
+    {
+        return;
+    }
 
-    open( filepath );
+    //openConfAndData( filepath );
+    openConfFile( filepath );
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
-MainWindow::open( const QString & filepath )
+MainWindow::openConfAndData( const QString & filepath )
 {
     if ( ! openConfFile( filepath ) )
     {
@@ -1064,7 +1081,7 @@ MainWindow::open( const QString & filepath )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 MainWindow::openConfFile( const QString & filepath )
 {
@@ -1092,7 +1109,7 @@ MainWindow::openConfFile( const QString & filepath )
     }
 
     M_editor_data = boost::shared_ptr< FEditData >( new FEditData() );
-    if ( ! M_editor_data->open( filepath.ascii() ) )
+    if ( ! M_editor_data->openConf( filepath.ascii() ) )
     {
         M_editor_data.reset();
         QMessageBox::warning( this,
@@ -1125,13 +1142,14 @@ MainWindow::openConfFile( const QString & filepath )
     M_editor_data->moveBallTo( 0.0, 0.0 );
     M_editor_dialog->updateData();
 
+    setDataSize();
     return true;
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::openData()
 {
@@ -1145,20 +1163,25 @@ MainWindow::openData()
         return;
     }
 
-    QString filter( tr( "Formation file (*.dat);;"
+    QString filter( tr( "Formation data file (*.dat);;"
                         "All files (*)" ) );
     QString filepath = QFileDialog::getOpenFileName( QString::null,
                                                      filter,
                                                      this,
                                                      tr( "Open Training Data" ),
                                                      tr( "Choose a file name to open" ) );
+    if ( filepath.isEmpty() )
+    {
+        return;
+    }
+
     openDataFile( filepath );
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 MainWindow::openDataFile( const QString & filepath )
 {
@@ -1195,7 +1218,7 @@ MainWindow::openDataFile( const QString & filepath )
         return false;
     }
 
-    if ( ! M_editor_data->openTrainingData( filepath.ascii() ) )
+    if ( ! M_editor_data->openData( filepath.ascii() ) )
     {
         QMessageBox::warning( this,
                               tr( "Error" ),
@@ -1206,7 +1229,7 @@ MainWindow::openDataFile( const QString & filepath )
 
     }
 
-    setTrainingDataSize();
+    setDataSize();
 
     emit viewUpdated();
 
@@ -1216,7 +1239,90 @@ MainWindow::openDataFile( const QString & filepath )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
+void
+MainWindow::openBackgroundConf()
+{
+    QString filter( tr( "Background formation file (*.conf);;"
+                        "All files (*)" ) );
+    QString filepath = QFileDialog::getOpenFileName( QString::null,
+                                                     filter,
+                                                     this,
+                                                     tr( "Open Background Formation" ),
+                                                     tr( "Choose a file name to open" ) );
+
+    //openBackgroundConfAndData( filepath );
+    openBackgroundConfFile( filepath );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+bool
+MainWindow::openBackgroundConfFile( const QString & filepath )
+{
+    if ( ! M_editor_data )
+    {
+        QMessageBox::warning( this,
+                              tr( "Error" ),
+                              tr( "Tried to open a background formation. But no foreground data yet." ),
+                              QMessageBox::Ok,
+                              QMessageBox::NoButton );
+        return false;
+    }
+
+    if ( ! M_editor_data->formation() )
+    {
+        QMessageBox::warning( this,
+                              tr( "Error" ),
+                              tr( "No formation for foreground data." ),
+                              QMessageBox::Ok,
+                              QMessageBox::NoButton );
+        return false;
+    }
+
+    if ( filepath.isEmpty() )
+    {
+        QMessageBox::warning( this,
+                              tr( "Warning" ),
+                              tr( "Empty file path." ),
+                              QMessageBox::Ok,
+                              QMessageBox::NoButton );
+        return false;
+    }
+
+    QFileInfo fileinfo( filepath );
+
+    if ( ! fileinfo.exists()
+         || ! fileinfo.isReadable() )
+    {
+        QMessageBox::warning( this,
+                              tr( "Warning" ),
+                              tr( "No such a file or not readable. " ) + filepath,
+                              QMessageBox::Ok,
+                              QMessageBox::NoButton );
+        return false;
+    }
+
+
+    if ( ! M_editor_data->openBackgroundConf( filepath.ascii() ) )
+    {
+        QMessageBox::warning( this,
+                              tr( "Error" ),
+                              tr( "Failed to open a " ) + filepath,
+                              QMessageBox::Ok,
+                              QMessageBox::NoButton );
+        return false;
+    }
+
+    return true;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
 bool
 MainWindow::saveChanged()
 {
@@ -1226,14 +1332,16 @@ MainWindow::saveChanged()
     }
 
     if ( ! M_editor_data->isModified()
-         && ! M_editor_data->isTrainingDataChanged() )
+         //&& ( ! M_editor_data->formationData()
+         //|| ! M_editor_data->formationData()->isChanged() )
+         )
     {
         return true;
     }
 
     int result = QMessageBox::question( this,
                                         tr( "Save Notify" ),
-                                        tr( "Data is changed.\nSave?" ),
+                                        tr( "Something changed.\nSave?" ),
                                         QMessageBox::Cancel,
                                         QMessageBox::Ok,
                                         QMessageBox::No );
@@ -1246,12 +1354,8 @@ MainWindow::saveChanged()
     {
         if ( M_editor_data->isModified() )
         {
-            save();
-        }
-
-        if ( M_editor_data->isTrainingDataChanged() )
-        {
-            saveTrainingData();
+            std::cerr << "saveChanged saveConf()" << std::endl;
+            saveConf();
         }
     }
 
@@ -1261,25 +1365,9 @@ MainWindow::saveChanged()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
-MainWindow::saveAll()
-{
-    if ( ! M_editor_data )
-    {
-        return;
-    }
-
-    save();
-    saveTrainingData();
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-void
-MainWindow::save()
+MainWindow::saveConf()
 {
     if ( ! M_editor_data
          || ! M_editor_data->formation() )
@@ -1289,9 +1377,9 @@ MainWindow::save()
 
     std::cerr << "save" << std::endl;
 
-    if ( ! M_editor_data->save() )
+    if ( ! M_editor_data->saveConf() )
     {
-        saveAs();
+        saveConfAs();
     }
 
 }
@@ -1299,9 +1387,9 @@ MainWindow::save()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
-MainWindow::saveAs()
+MainWindow::saveConfAs()
 {
     if ( ! M_editor_data
          || ! M_editor_data->formation() )
@@ -1316,6 +1404,10 @@ MainWindow::saveAs()
                                                      this,
                                                      tr( "Save Formation" ),
                                                      tr( "Choose a file name to save under" ) );
+    if ( filepath.isEmpty() )
+    {
+        return;
+    }
 
     if ( filepath.length() <= 5
          || filepath.right( 5 ) != tr( ".conf" ) )
@@ -1323,7 +1415,7 @@ MainWindow::saveAs()
         filepath += ".conf";
     }
 
-    if ( M_editor_data->save( filepath.ascii() ) )
+    if ( M_editor_data->saveConfAs( filepath.ascii() ) )
     {
         QFileInfo fileinfo( filepath );
         this->setCaption( tr( "FormationEditor - " )
@@ -1342,45 +1434,31 @@ MainWindow::saveAs()
     }
 }
 
-/*-------------------------------------------------------------------*/
-/*!
-
-*/
-void
-MainWindow::saveTrainingData()
-{
-    if ( ! M_editor_data
-         || M_editor_data->trainingData().empty() )
-    {
-        return;
-    }
-
-    if ( ! M_editor_data->saveTrainingData() )
-    {
-        saveTrainingDataAs();
-    }
-}
 
 /*-------------------------------------------------------------------*/
 /*!
 
 */
 void
-MainWindow::saveTrainingDataAs()
+MainWindow::saveDataAs()
 {
     if ( ! M_editor_data
-         || M_editor_data->trainingData().empty() )
+         || ! M_editor_data->samples() )
     {
         return;
     }
 
-    QString filter( tr( "Formation file (*.dat);;"
+    QString filter( tr( "Training Data file (*.dat);;"
                         "All files (*)" ) );
     QString filepath = QFileDialog::getSaveFileName( QString::null,
                                                      filter,
                                                      this,
                                                      tr( "Save Training Data" ),
                                                      tr( "Choose a file name to save under" ) );
+    if ( filepath.isEmpty() )
+    {
+        return;
+    }
 
     if ( filepath.length() <= 4
          || filepath.right( 4 ) != tr( ".dat" ) )
@@ -1388,14 +1466,13 @@ MainWindow::saveTrainingDataAs()
         filepath += ".dat";
     }
 
-    if ( ! M_editor_data->saveTrainingData( filepath.ascii() ) )
+    if ( ! M_editor_data->samples()->save( filepath.ascii() ) )
     {
         QMessageBox::critical( this,
                                tr( "Error" ),
                                tr( "Failed to save the file " ) + filepath,
                                QMessageBox::Ok,
                                QMessageBox::NoButton );
-        std::cerr << "Error. Failed to open file output steramr" << std::endl;
     }
 
 }
@@ -1403,7 +1480,7 @@ MainWindow::saveTrainingDataAs()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::toggleBallDrag( bool on )
 {
@@ -1418,7 +1495,7 @@ MainWindow::toggleBallDrag( bool on )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::togglePlayerAutoMove( bool on )
 {
@@ -1433,7 +1510,7 @@ MainWindow::togglePlayerAutoMove( bool on )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::toggleDataAutoSelect( bool on )
 {
@@ -1448,22 +1525,22 @@ MainWindow::toggleDataAutoSelect( bool on )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
-MainWindow::toggleSymmetricMode( bool on )
+MainWindow::toggleSymmetryMode( bool on )
 {
-    //std::cerr << "toggle symmetric mode " << std::endl;
+    //std::cerr << "toggle symmetry mode " << std::endl;
 
     if ( M_editor_data )
     {
-        M_editor_data->setSymmetricMode( on );
+        M_editor_data->setSymmetryMode( on );
     }
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::showEditDialog( bool on )
 {
@@ -1484,18 +1561,19 @@ MainWindow::showEditDialog( bool on )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::recordData()
 {
     std::cerr << "recordData" << std::endl;
 
-    if ( ! M_editor_data )
+    if ( ! M_editor_data
+         || ! M_editor_data->samples() )
     {
         return;
     }
 
-    std::string msg = M_editor_data->recordTrainingData();
+    std::string msg = M_editor_data->recordData();
 
     if ( ! msg.empty() )
     {
@@ -1507,8 +1585,8 @@ MainWindow::recordData()
         return;
     }
 
-    setTrainingDataSize();
-    M_index_spin_box->setValue( M_editor_data->trainingData().size() );
+    setDataSize();
+    M_index_spin_box->setValue( M_editor_data->samples()->dataCont().size() + 1 );
 
     emit viewUpdated();
 }
@@ -1516,7 +1594,7 @@ MainWindow::recordData()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::insertData()
 {
@@ -1527,7 +1605,7 @@ MainWindow::insertData()
 
     int cur = M_index_spin_box->value();
 
-    std::string msg = M_editor_data->insertTrainingData( cur );
+    std::string msg = M_editor_data->insertData( cur - 1 );
 
     if ( ! msg.empty() )
     {
@@ -1539,7 +1617,7 @@ MainWindow::insertData()
         return;
     }
 
-    setTrainingDataSize();
+    setDataSize();
     M_index_spin_box->setValue( cur + 1 );
 
     emit viewUpdated();
@@ -1548,7 +1626,7 @@ MainWindow::insertData()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::replaceData()
 {
@@ -1557,16 +1635,24 @@ MainWindow::replaceData()
         return;
     }
 
-    if ( M_editor_data->replaceTrainingData( M_index_spin_box->value() ) )
+    int val = M_index_spin_box->value();
+    if ( ! M_editor_data->replaceData( val - 1 ) )
     {
-        emit viewUpdated();
+        QMessageBox::warning( this,
+                              tr( "Warning" ),
+                              tr( "Failed to replace the data [%1]" ).arg( val ),
+                              QMessageBox::Ok,
+                              QMessageBox::NoButton );
+        return;
     }
+
+    emit viewUpdated();
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::deleteData()
 {
@@ -1575,18 +1661,26 @@ MainWindow::deleteData()
         return;
     }
 
-    if ( M_editor_data->deleteTrainingData( M_index_spin_box->value() ) )
+    int val = M_index_spin_box->value();
+    if ( ! M_editor_data->deleteData( val - 1 ) )
     {
-        M_index_spin_box->setValue( 0 );
-        setTrainingDataSize();
-        emit viewUpdated();
+        QMessageBox::warning( this,
+                              tr( "Warning" ),
+                              tr( "Failed to delete the data [%1]" ).arg( val ),
+                              QMessageBox::Ok,
+                              QMessageBox::NoButton );
+        return;
     }
+
+    M_index_spin_box->setValue( 0 );
+    setDataSize();
+    emit viewUpdated();
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::reverseY()
 {
@@ -1601,7 +1695,7 @@ MainWindow::reverseY()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::train()
 {
@@ -1615,33 +1709,35 @@ MainWindow::train()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
-MainWindow::setTrainingDataSize()
+MainWindow::setDataSize()
 {
     if ( M_editor_data
-         && ! M_editor_data->trainingData().empty() )
+         && M_editor_data->samples()
+         && ! M_editor_data->samples()->dataCont().empty() )
     {
-        M_index_spin_box->setRange( 0, M_editor_data->trainingData().size() );
+        M_index_spin_box->setRange( 0, M_editor_data->samples()->dataCont().size() );
     }
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
-MainWindow::changeDataIndex( int idx )
+MainWindow::changeDataIndex( int value )
 {
-    if ( M_index_spin_box->value() != idx
-         && 0 < idx
-         && idx <= M_index_spin_box->maxValue() )
+    if ( M_index_spin_box->value() != value
+         && 0 <= value
+         && value <= M_index_spin_box->maxValue() )
     {
-        M_index_spin_box->setValue( idx );
+        M_index_spin_box->setValue( value );
     }
 
-    if ( M_editor_data
-         && M_editor_data->focusTrainingData( idx ) )
+    if ( 0 < value
+         && M_editor_data
+         && M_editor_data->focusData( value - 1 ) )
     {
         M_editor_dialog->updateData();
         emit viewUpdated();
@@ -1651,16 +1747,16 @@ MainWindow::changeDataIndex( int idx )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 MainWindow::updateDataIndex()
 {
     if ( M_editor_data
          && M_editor_data->isPlayerAutoMove() )
     {
-        int idx = M_editor_data->currentIndex();
+        int idx = M_editor_data->currentIndex() + 1;
         if ( M_index_spin_box->value() != idx
-             && 0 < idx
+             && 0 <= idx
              && idx <= M_index_spin_box->maxValue() )
         {
             M_index_spin_box->setValue( idx );
