@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -37,6 +37,7 @@
 
 #include <iostream>
 #include <cstdio>
+#include <cstring>
 
 /*-------------------------------------------------------------------*/
 /*!
@@ -68,21 +69,33 @@ DebugLogData::parse( const char * buf )
     buf += n_read;
 
     switch ( type ) {
-    case 'T':
+    case 'M':
         M_text_cont.push_back( TextT( level ) );
         M_text_cont.back().msg_ = buf;
         M_text_cont.back().msg_ += '\n';
         return true;
-    case 'p':
-        return addPoint( level, buf );
     case 'l':
         return addLine( level, buf );
+    case 'a':
+        return addArc( level, buf );
     case 'c':
-        return addCircle( level, buf );
-    case 't':
-        return addTriangle( level, buf );
+        return addCircle( level, buf, false );
+    case 'C':
+        return addCircle( level, buf, true );
     case 'r':
-        return addRect( level, buf );
+        return addRect( level, buf, false );
+    case 'R':
+        return addRect( level, buf, true );
+    case 's':
+        return addSector( level, buf, false );
+    case 'S':
+        return addSector( level, buf, true );
+    case 't':
+        return addTriangle( level, buf, false );
+    case 'T':
+        return addTriangle( level, buf, true );
+    case 'p':
+        return addPoint( level, buf );
     case 'm':
         return addMessage( level, buf );
     default:
@@ -192,10 +205,45 @@ DebugLogData::addLine( const boost::int32_t level,
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+bool
+DebugLogData::addArc( const boost::int32_t level,
+                       const char * buf )
+{
+    // <x> <y> <r> <star_angle> <span_angle>
+    ArcT arc;
+    int n_read = 0;
+    if ( std::sscanf( buf, " %lf %lf %lf %lf %lf %n",
+                      &arc.x_, &arc.y_,
+                      &arc.r_,
+                      &arc.start_angle_, &arc.span_angle_,
+                      &n_read ) != 5 )
+    {
+        std::cerr << "DebugLogData. Illegal Arc" << std::endl;
+        return false;
+    }
+
+    buf += n_read;
+
+    if ( std::strlen( buf ) > 0 )
+    {
+        arc.color_ = buf;
+    }
+
+    arc.level_ = level;
+
+    M_arc_cont.push_back( arc );
+    return true;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 bool
 DebugLogData::addCircle( const boost::int32_t level,
-                         const char * buf )
+                         const char * buf,
+                         const bool fill )
 {
     // <x> <y> <r>
     CircleT c;
@@ -217,7 +265,14 @@ DebugLogData::addCircle( const boost::int32_t level,
 
     c.level_ = level;
 
-    M_circle_cont.push_back( c );
+    if ( fill )
+    {
+        M_filled_circle_cont.push_back( c );
+    }
+    else
+    {
+        M_circle_cont.push_back( c );
+    }
     return true;
 }
 
@@ -227,7 +282,8 @@ DebugLogData::addCircle( const boost::int32_t level,
 */
 bool
 DebugLogData::addTriangle( const boost::int32_t level,
-                           const char * buf )
+                           const char * buf,
+                           const bool fill )
 {
     // <x1> <y1> <x2> <y2> <x3> <y3>
     TriangleT t;
@@ -251,7 +307,14 @@ DebugLogData::addTriangle( const boost::int32_t level,
 
     t.level_ = level;
 
-    M_triangle_cont.push_back( t );
+    if ( fill )
+    {
+        M_filled_triangle_cont.push_back( t );
+    }
+    else
+    {
+        M_triangle_cont.push_back( t );
+    }
     return true;
 }
 
@@ -261,7 +324,8 @@ DebugLogData::addTriangle( const boost::int32_t level,
 */
 bool
 DebugLogData::addRect( const boost::int32_t level,
-                       const char * buf )
+                       const char * buf,
+                       const bool fill )
 {
     // <left> <top> <width> <height> <color name>
     RectT r;
@@ -285,7 +349,55 @@ DebugLogData::addRect( const boost::int32_t level,
 
     r.level_ = level;
 
-    M_rect_cont.push_back( r );
+    if ( fill )
+    {
+        M_filled_rect_cont.push_back( r );
+    }
+    else
+    {
+        M_rect_cont.push_back( r );
+    }
+    return true;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+bool
+DebugLogData::addSector( const boost::int32_t level,
+                         const char * buf,
+                         const bool fill )
+{
+    // <x> <y> <min_r> <max_r> <star_angle> <span_angle>
+    SectorT sector;
+    int n_read = 0;
+    if ( std::sscanf( buf, " %lf %lf %lf %lf %lf %lf %n",
+                      &sector.x_, &sector.y_,
+                      &sector.min_r_, &sector.max_r_,
+                      &sector.start_angle_, &sector.span_angle_,
+                      &n_read ) != 6 )
+    {
+        std::cerr << "DebugLogData. Illegal Sector" << std::endl;
+        return false;
+    }
+    buf += n_read;
+
+    if ( std::strlen( buf ) > 0 )
+    {
+        sector.color_ = buf;
+    }
+
+    sector.level_ = level;
+
+    if ( fill )
+    {
+        M_filled_sector_cont.push_back( sector );
+    }
+    else
+    {
+        M_sector_cont.push_back( sector );
+    }
     return true;
 }
 
