@@ -40,7 +40,9 @@
 #include "view_holder.h"
 
 #include <rcsc/rcg.h>
+#ifndef NO_TIMER
 #include <rcsc/timer.h>
+#endif
 
 #ifdef HAVE_LIBRCSC_GZ
 #include <rcsc/gz/gzfstream.h>
@@ -112,7 +114,9 @@ MainData::openRCG( const std::string & file_path )
     rcsc::rcg::Parser::Ptr parser = rcsc::rcg::Parser::create( fin );
 
     // do parse & handle data
-    rcsc::MSecTimer timer;
+#ifndef NO_TIMER
+    rcsc::Timer timer;
+#endif
     if ( ! parser
          || ! parser->parse( fin, reader ) )
     {
@@ -120,7 +124,9 @@ MainData::openRCG( const std::string & file_path )
         fin.close();
         return false;
     }
+#ifndef NO_TIMER
     std::cerr << "paser elapsed " << timer.elapsedReal() << "[ms]" << std::endl;
+#endif
 
     M_view_holder.pushBackLatestViewData();
     fin.close();
