@@ -111,6 +111,7 @@ Options::Options()
     , M_canvas_width( -1 )
     , M_canvas_height( -1 )
     , M_field_center( 0, 0 )
+    , M_field_center_real( 0.0, 0.0 )
     , M_field_scale( 8.0 )
     , M_score_board_font_size( 11 )
     , M_score_board_height( 0 )
@@ -130,7 +131,7 @@ Options::Options()
     , M_reverse_side( false )
     , M_player_reverse_draw( false )
     , M_show_score_board( true )
-    , M_show_team_logo( true )
+    , M_show_team_graphic( true )
     , M_show_flags( false )
     , M_show_grid_coord( true )
     , M_grid_step( 0 )
@@ -148,6 +149,7 @@ Options::Options()
     , M_show_tackle_area( false )
     , M_show_kick_accel_area( false )
     , M_show_offside_line( false )
+    , M_show_card( true )
     , M_enlarge_mode( true )
     , M_ball_size( 0.35 )
     , M_player_size( 0.0 )
@@ -186,7 +188,9 @@ Options::Options()
     , M_image_name_prefix( "image-" )
     , M_image_save_format( "PNG" )
       // other options
-    , M_monitor_client_mode( false )
+    , M_monitor_client_mode( false ),
+      M_mouse_measure_mode( MEASURE_BALL_MOVE ),
+      M_mouse_measure_first_length( -1.0 )
 {
 
 }
@@ -339,12 +343,12 @@ Options::parseCmdLine( int argc,
         ( "hide-score-board", "",
           rcsc::NegateSwitch( &M_show_score_board ),
           "hide score board." )
-        ( "show-team-logo", "",
-          rcsc::BoolSwitch( &M_show_team_logo ),
-          "show team logo." )
-        ( "hide-team-logo", "",
-          rcsc::NegateSwitch( &M_show_team_logo ),
-          "hide team logo." )
+        ( "show-team-graphic", "",
+          rcsc::BoolSwitch( &M_show_team_graphic ),
+          "show team graphic." )
+        ( "hide-team-grahip", "",
+          rcsc::NegateSwitch( &M_show_team_graphic ),
+          "hide team graphic." )
         ( "anti-aliasing", "",
           &M_anti_aliasing,
           "set an anti-aliasing draw mode (only for Qt4 version)." )
@@ -399,6 +403,12 @@ Options::parseCmdLine( int argc,
         ( "hide-stamina-capacity", "",
           rcsc::NegateSwitch( &M_show_stamina_capacity ),
           "hide player\'s stamina capacity." )
+        ( "show-card", "",
+          rcsc::BoolSwitch( &M_show_card ),
+          "show player\'s card status." )
+        ( "hide-card", "",
+          rcsc::NegateSwitch( &M_show_card ),
+          "hide player\'s card status." )
         ( "enlarge-mode", "",
           rcsc::BoolSwitch( &M_enlarge_mode ),
           "show enlarged objects." )
@@ -748,12 +758,17 @@ Options::updateFieldCenter( const int canvas_width,
 {
     int field_height = canvas_height - scoreBoardHeight();
 
-    M_field_center.x = canvas_width/2 - scale( focusPoint().x );
-
+    M_field_center.x = canvas_width/2 - scaleInt( focusPoint().x );
     M_field_center.y
         = scoreBoardHeight()
         + field_height/2
-        - scale( focusPoint().y );
+        - scaleInt( focusPoint().y );
+
+    M_field_center_real.x = canvas_width*0.5 - scaleF( focusPoint().x );
+    M_field_center_real.y
+        = scoreBoardHeight()
+        + field_height*0.5
+        - scaleF( focusPoint().y );
 }
 
 /*-------------------------------------------------------------------*/
@@ -922,7 +937,7 @@ Options::zoomOut()
 
  */
 void
-Options::unzoom()
+Options::fitToScreen()
 {
     if ( M_zoomed
          || focusType() != Options::FOCUS_POINT
