@@ -72,6 +72,12 @@ public:
         FOCUS_POINT, // include center
     };
 
+    enum MouseMeasureMode {
+        MEASURE_NO_MODE,
+        MEASURE_BALL_MOVE,
+        MEASURE_PLAYER_MOVE,
+    };
+
     static const double MIN_FIELD_SCALE;
     static const double MAX_FIELD_SCALE;
 
@@ -130,6 +136,7 @@ private:
     int M_canvas_width;
     int M_canvas_height;
     Point M_field_center; //!< the screen point of field center
+    PointF M_field_center_real; //!< the screen point of field center
     double M_field_scale; //!< field scale rate
     int M_score_board_font_size;
     int M_score_board_height; //!< screen height of score board
@@ -158,7 +165,7 @@ private:
     bool M_player_reverse_draw;
 
     bool M_show_score_board;
-    bool M_show_team_logo;
+    bool M_show_team_graphic;
     bool M_show_flags; // no cmd line option
     bool M_show_grid_coord; // no cmd line option
     int M_grid_step; // no cmd line option
@@ -177,6 +184,7 @@ private:
     bool M_show_tackle_area; // no cmd line option
     bool M_show_kick_accel_area; // no cmd line option
     bool M_show_offside_line; // no cmd line option
+    bool M_show_card;
 
     bool M_enlarge_mode;
     double M_ball_size;
@@ -237,6 +245,12 @@ private:
     bool M_monitor_client_mode;
 
 
+    //
+    // mouse measure
+    //
+    MouseMeasureMode M_mouse_measure_mode;
+    double M_mouse_measure_first_length;
+
 
     //! private access for singleton
     Options();
@@ -328,13 +342,14 @@ public:
 
     void zoomIn();
     void zoomOut();
-    void unzoom();
+    void fitToScreen();
     bool zoomed() const { return M_zoomed; }
 
     void setFieldScale( const double & scale );
     const double & fieldScale() const { return M_field_scale; }
 
-    const Point & fieldCenter() const { return M_field_center; }
+    const Point & fieldCenterInt() const { return M_field_center; }
+    const PointF & fieldCenterF() const { return M_field_center_real; }
 
     void setFocusPoint( const int screen_x,
                         const int screen_y );
@@ -398,8 +413,8 @@ public:
     void toggleShowScoreBoard() { M_show_score_board = ! M_show_score_board; }
     bool showScoreBoard() const { return M_show_score_board; }
 
-    void toggleShowTeamLogo() { M_show_team_logo = ! M_show_team_logo; }
-    bool showTeamLogo() const { return M_show_team_logo; }
+    void toggleShowTeamGraphic() { M_show_team_graphic = ! M_show_team_graphic; }
+    bool showTeamGraphic() const { return M_show_team_graphic; }
 
     void toggleShowFlags() { M_show_flags = ! M_show_flags; }
     bool showFlags() const { return M_show_flags; }
@@ -437,6 +452,9 @@ public:
     void toggleShowStaminaCapacity() { M_show_stamina_capacity = ! M_show_stamina_capacity; }
     bool showStaminaCapacity() const { return M_show_stamina_capacity; }
 
+    void toggleShowCard() { M_show_card = ! M_show_card; }
+    bool showCard() const { return M_show_card; }
+
     void toggleShowBodyShadow() { M_show_body_shadow = ! M_show_body_shadow; }
     bool showBodyShadow() const { return M_show_body_shadow; }
 
@@ -561,7 +579,15 @@ public:
     const std::string & imageSaveFormat() const { return M_image_save_format; }
     bool autoImageSave() const { return M_auto_image_save; }
 
+    //
+    // mouse measure
+    //
 
+    MouseMeasureMode mouseMeasureMode() const { return M_mouse_measure_mode; }
+    void setMouseMeasureMode( const MouseMeasureMode mode ) { M_mouse_measure_mode = mode; }
+
+    const double & mouseMeasureFirstLength() const { return M_mouse_measure_first_length; }
+    void setMouseMeasureFirstLength( const double & len ) { M_mouse_measure_first_length = len; }
 
     /*!
       \brief update parameters with canvas size
@@ -592,7 +618,7 @@ public:
       \param len real length
       \return screen pixel length
     */
-    int scale( const double & len ) const
+    int scaleInt( const double & len ) const
       {
           return static_cast< int >( rint( len * fieldScale() ) );
           //double dlen = len * M_field_scale;
@@ -601,25 +627,41 @@ public:
           //return static_cast< int >( dlen );
       }
 
+    double scaleF( const double & len ) const
+      {
+          return len * fieldScale();
+      }
+
     /*!
       \brief convert 'x' to the screen coordinate X.
       \param abs_x real point
       \return screen point
     */
-    int absScreenX( const double & abs_x ) const
+    int absScreenXInt( const double & abs_x ) const
       {
-          return fieldCenter().x + scale( abs_x );
+          return fieldCenterInt().x + scaleInt( abs_x );
       }
     /*!
       \brief convert 'x' to the screen coordinate X with reverse mode.
       \param x real point (may be reversed)
       \return screen point
     */
-    int screenX( const double & x ) const
+    int screenXInt( const double & x ) const
       {
           return ( M_reverse_side
-                   ? fieldCenter().x + scale( -x )
-                   : fieldCenter().x + scale( x ) );
+                   ? fieldCenterInt().x + scaleInt( -x )
+                   : fieldCenterInt().x + scaleInt( x ) );
+      }
+
+    double absScreenXF( const double & abs_x ) const
+      {
+          return fieldCenterF().x + scaleF( abs_x );
+      }
+    double screenXF( const double & x ) const
+      {
+          return ( M_reverse_side
+                   ? fieldCenterF().x + scaleF( -x )
+                   : fieldCenterF().x + scaleF( x ) );
       }
 
     /*!
@@ -627,20 +669,31 @@ public:
       \param y real point
       \return screen point
     */
-    int absScreenY( const double & abs_y ) const
+    int absScreenYInt( const double & abs_y ) const
       {
-          return fieldCenter().y + scale( abs_y );
+          return fieldCenterInt().y + scaleInt( abs_y );
       }
     /*!
       \brief convert 'y' to the screen coordinate X.
       \param y real point (may be reversed)
       \return screen point
     */
-    int screenY( const double & y ) const
+    int screenYInt( const double & y ) const
       {
           return ( M_reverse_side
-                   ? fieldCenter().y + scale( -y )
-                   : fieldCenter().y + scale( y ) );
+                   ? fieldCenterInt().y + scaleInt( -y )
+                   : fieldCenterInt().y + scaleInt( y ) );
+      }
+
+    double absScreenYF( const double & abs_y ) const
+      {
+          return fieldCenterF().y + scaleF( abs_y );
+      }
+    double screenYF( const double & y ) const
+      {
+          return ( M_reverse_side
+                   ? fieldCenterF().y + scaleF( -y )
+                   : fieldCenterF().y + scaleF( y ) );
       }
 
     /*!
@@ -650,7 +703,7 @@ public:
      */
     double fieldX( const int x ) const
       {
-          return ( x - fieldCenter().x ) / fieldScale();
+          return ( x - fieldCenterInt().x ) / fieldScale();
       }
 
     /*!
@@ -660,7 +713,7 @@ public:
      */
     double fieldY( const int y ) const
       {
-          return ( y - fieldCenter().y ) / fieldScale();
+          return ( y - fieldCenterInt().y ) / fieldScale();
       }
 
     /*!
