@@ -66,10 +66,9 @@ const std::string DEFAULT_DEBUG_VIEW_EXTENSION = ".dcl";
 */
 ViewHolder::ViewHolder()
     : M_last_playmode( rcsc::PM_Null )
-    , M_default_type( rcsc::ServerParam::i() )
+    , M_default_type()
 {
-    M_player_types.insert( std::pair< int, rcsc::PlayerType >
-                           ( 0, M_default_type ) );
+    M_player_types.insert( std::pair< int, rcsc::PlayerType >( 0, M_default_type ) );
 
     M_monitor_view_cont.reserve( 1024 * 8 );
 }
@@ -106,10 +105,9 @@ ViewHolder::clear()
     M_left_debug_view.clear();
     M_right_debug_view.clear();
 
-    M_default_type = rcsc::PlayerType( rcsc::ServerParam::i() );
+    M_default_type = rcsc::PlayerType();
     M_player_types.clear();
-    M_player_types.insert( std::pair< int, rcsc::PlayerType >
-                           ( 0, M_default_type ) );
+    M_player_types.insert( std::pair< int, rcsc::PlayerType >( 0, M_default_type ) );
 
     M_team_graphic_left.clear();
     M_team_graphic_right.clear();
@@ -589,13 +587,12 @@ ViewHolder::addPlayerType( const rcsc::rcg::player_type_t & param )
 
     if ( M_player_types.find( id ) != M_player_types.end() )
     {
-        M_player_types[id] = rcsc::PlayerType( rcsc::ServerParam::i(), param );
+        M_player_types[id] = rcsc::PlayerType( param );
     }
     else
     {
         M_player_types.insert( std::pair< int, rcsc::PlayerType >
-                               ( id,
-                                 rcsc::PlayerType( rcsc::ServerParam::i(), param ) ) );
+                               ( id, rcsc::PlayerType( param ) ) );
     }
 
     return true;
@@ -610,7 +607,7 @@ ViewHolder::addServerParam( const rcsc::rcg::server_params_t & param )
 {
     //std::cerr << "ViewHolder::addServerParam" << std::endl;
     rcsc::ServerParam::instance().convertFrom( param );
-    M_default_type = rcsc::PlayerType( rcsc::ServerParam::i() );
+    M_default_type = rcsc::PlayerType();
     return true;
 }
 
@@ -737,7 +734,7 @@ ViewHolder::addServerParam( const std::string & msg )
 {
     //std::cerr << "ViewHolder::addServerParam" << std::endl;
     rcsc::ServerParam::instance().parse( msg.c_str(), 999.0 );
-    M_default_type = rcsc::PlayerType( rcsc::ServerParam::i() );
+    M_default_type = rcsc::PlayerType();
     return true;
 }
 
@@ -760,9 +757,7 @@ ViewHolder::addPlayerParam( const std::string & msg )
 bool
 ViewHolder::addPlayerType( const std::string & msg )
 {
-    rcsc::PlayerType player_type( rcsc::ServerParam::i(),
-                                  msg.c_str(),
-                                  999.0 );
+    rcsc::PlayerType player_type( msg.c_str(), 999.0 );
 
     //std::cerr << "ViewHolder::addPlayerType " << player_type.id() << std::endl;
     if ( player_type.id() < 0 )
