@@ -74,12 +74,12 @@ BallPainter::draw( QPainter & painter )
     const int ball_radius
         = std::max( 1,
                     ( opt.enlargeMode()
-                      ? opt.scale( opt.ballSize() )
-                      : opt.scale( rcsc::ServerParam::i().ballSize() ) )
+                      ? opt.scaleInt( opt.ballSize() )
+                      : opt.scaleInt( rcsc::ServerParam::i().ballSize() ) )
                     );
     const int ball_diameter = ball_radius * 2;
-    const int ix = opt.screenX( view->ball().x() );
-    const int iy = opt.screenY( view->ball().y() );
+    const int ix = opt.screenXInt( view->ball().x() );
+    const int iy = opt.screenYInt( view->ball().y() );
 
 
     // set GDI objects
@@ -120,7 +120,7 @@ BallPainter::draw( QPainter & painter )
         painter.setBrush( dconf.transparentBrush() );
 
         int kickable_radius
-            = opt.scale( rcsc::ServerParam::i().defaultKickableMargin() );
+            = opt.scaleInt( rcsc::ServerParam::i().defaultKickableMargin() );
         painter.drawEllipse( ix - kickable_radius,
                              iy - kickable_radius,
                              kickable_radius * 2,
@@ -156,8 +156,8 @@ BallPainter::drawFutureState( QPainter & painter ) const
     bpos *= opt.reverseValue();
     bvel *= opt.reverseValue();
 
-    QPoint first_point( opt.absScreenX( bpos.x ),
-                        opt.absScreenY( bpos.y ) );
+    QPoint first_point( opt.absScreenXInt( bpos.x ),
+                        opt.absScreenYInt( bpos.y ) );
 
     painter.save();
     painter.translate( first_point.x(), first_point.y() );
@@ -175,7 +175,7 @@ BallPainter::drawFutureState( QPainter & painter ) const
         travel += speed;
         speed *= ball_decay;
 
-        int x = opt.scale( travel );
+        int x = opt.scaleInt( travel );
         if ( std::abs( last_x - x ) < 1 )
         {
             break;
