@@ -117,8 +117,8 @@ BallTracePainter::draw( QPainter & painter )
     black_dot_pen.setStyle( Qt::DotLine );
 
     std::size_t i = first;
-    int prev_x = opt.screenX( vc[i]->ball().x() );
-    int prev_y = opt.screenY( vc[i]->ball().y() );
+    int prev_x = opt.screenXInt( vc[i]->ball().x() );
+    int prev_y = opt.screenYInt( vc[i]->ball().y() );
     ++i;
     for ( ; i <= last; ++i )
     {
@@ -127,8 +127,8 @@ BallTracePainter::draw( QPainter & painter )
         case rcsc::PM_TimeOver:
         case rcsc::PM_KickOff_Left:
         case rcsc::PM_KickOff_Right:
-            prev_x = opt.screenX( 0.0 );
-            prev_y = opt.screenY( 0.0 );
+            prev_x = opt.screenXInt( 0.0 );
+            prev_y = opt.screenYInt( 0.0 );
             continue;
         case rcsc::PM_PlayOn:
         case rcsc::PM_AfterGoal_Left:
@@ -140,8 +140,8 @@ BallTracePainter::draw( QPainter & painter )
             break;
         }
 
-        int ix = opt.screenX( vc[i]->ball().x() );
-        int iy = opt.screenY( vc[i]->ball().y() );
+        int ix = opt.screenXInt( vc[i]->ball().x() );
+        int iy = opt.screenYInt( vc[i]->ball().y() );
 
         if ( line_trace )
         {
