@@ -34,24 +34,25 @@
 
 #include <qdialog.h>
 
+#include <boost/function.hpp>
+
 #include <vector>
 
 class QListBox;
-class QPen;
-class QBrush;
+class QListBoxItem;
 
 struct ColorEntry {
-    QColor default_color_;
-    QColor color_;
-    QPen * pen_;
-    QBrush * brush_;
+    typedef boost::function< void ( const QColor & ) > Func;
 
-    ColorEntry( QPen * pen,
-                QBrush * brush );
+    QColor old_color_;
+    QColor color_;
+    Func func_;
+
+    ColorEntry( const QColor & old_color,
+                Func func );
 
     bool setColor( const QColor & color );
     bool revert();
-    bool checkColor();
 };
 
 /*-------------------------------------------------------------------*/
@@ -82,19 +83,24 @@ private:
     void createControls();
 
     void addEntry( const QString & name,
-                   QPen * pen,
-                   QBrush * brush );
+                   const QColor & old_color,
+                   ColorEntry::Func func );
 
-    QLayout * createColorButtons();
+    QLayout * createColorList();
     QLayout * createAnswerButtons();
 
+    void createColorItems();
+
     void revert();
 
+    void selectEntry( int index );
+
 private slots:
 
     void setDefaultColors();
 
     void selectEntry();
+    void selectEntry( QListBoxItem * item );
 
 public slots:
 
