@@ -149,8 +149,8 @@ DebugLogPainter::drawPoints( QPainter & painter,
                 painter.setPen( dconf.debugShapePen() );
             }
 
-            painter.drawPoint( opt.screenX( it->x_ * reverse ),
-                               opt.screenY( it->y_ * reverse ) );
+            painter.drawPoint( opt.screenXInt( it->x_ * reverse ),
+                               opt.screenYInt( it->y_ * reverse ) );
         }
     }
 
@@ -193,10 +193,10 @@ DebugLogPainter::drawLines( QPainter & painter,
                 painter.setPen( dconf.debugShapePen() );
             }
 
-            painter.drawLine( opt.screenX( it->x1_ * reverse ),
-                              opt.screenY( it->y1_ * reverse ),
-                              opt.screenX( it->x2_ * reverse ),
-                              opt.screenY( it->y2_ * reverse ) );
+            painter.drawLine( opt.screenXInt( it->x1_ * reverse ),
+                              opt.screenYInt( it->y1_ * reverse ),
+                              opt.screenXInt( it->x2_ * reverse ),
+                              opt.screenYInt( it->y2_ * reverse ) );
         }
     }
 }
@@ -262,8 +262,8 @@ DebugLogPainter::drawArcs( QPainter & painter,
 
             if ( loop == 1 )
             {
-                painter.drawPoint( opt.screenX( ( it->x_ + rpos.x ) * reverse ),
-                                   opt.screenY( ( it->y_ + rpos.y ) * reverse ) );
+                painter.drawPoint( opt.screenXInt( ( it->x_ + rpos.x ) * reverse ),
+                                   opt.screenYInt( ( it->y_ + rpos.y ) * reverse ) );
                 continue;
             }
 
@@ -271,8 +271,8 @@ DebugLogPainter::drawArcs( QPainter & painter,
             uint array_index = 0;
 
             points.setPoint( array_index,
-                             opt.screenX( ( it->x_ + rpos.x ) * reverse ),
-                             opt.screenY( ( it->y_ + rpos.y ) * reverse ) );
+                             opt.screenXInt( ( it->x_ + rpos.x ) * reverse ),
+                             opt.screenYInt( ( it->y_ + rpos.y ) * reverse ) );
             ++array_index;
 
             angle += angle_step;
@@ -280,8 +280,8 @@ DebugLogPainter::drawArcs( QPainter & painter,
             {
                 rpos = rcsc::Vector2D::polar2vector( it->r_, angle );
                 points.setPoint( array_index,
-                                 opt.screenX( ( it->x_ + rpos.x ) * reverse ),
-                                 opt.screenY( ( it->y_ + rpos.y ) * reverse ) );
+                                 opt.screenXInt( ( it->x_ + rpos.x ) * reverse ),
+                                 opt.screenYInt( ( it->y_ + rpos.y ) * reverse ) );
                 ++array_index;
             }
 
@@ -327,9 +327,9 @@ DebugLogPainter::drawCircles( QPainter & painter,
                 painter.setPen( dconf.debugShapePen() );
             }
 
-            int r = opt.scale( it->r_ );
-            painter.drawEllipse( opt.screenX( it->x_ * reverse ) - r,
-                                 opt.screenY( it->y_ * reverse ) - r,
+            int r = opt.scaleInt( it->r_ );
+            painter.drawEllipse( opt.screenXInt( it->x_ * reverse ) - r,
+                                 opt.screenYInt( it->y_ * reverse ) - r,
                                  r * 2, r * 2 );
         }
     }
@@ -353,9 +353,9 @@ DebugLogPainter::drawCircles( QPainter & painter,
                 painter.setBrush( dconf.debugShapePen().color() );
             }
 
-            int r = opt.scale( it->r_ );
-            painter.drawEllipse( opt.screenX( it->x_ * reverse ) - r,
-                                 opt.screenY( it->y_ * reverse ) - r,
+            int r = opt.scaleInt( it->r_ );
+            painter.drawEllipse( opt.screenXInt( it->x_ * reverse ) - r,
+                                 opt.screenYInt( it->y_ * reverse ) - r,
                                  r * 2, r * 2 );
         }
     }
@@ -400,14 +400,14 @@ DebugLogPainter::drawTriangles( QPainter & painter,
 
             QPointArray points( 3 );
             points.setPoint( 0,
-                             opt.screenX( it->x1_ * reverse ),
-                             opt.screenY( it->y1_ * reverse ) );
+                             opt.screenXInt( it->x1_ * reverse ),
+                             opt.screenYInt( it->y1_ * reverse ) );
             points.setPoint( 1,
-                             opt.screenX( it->x2_ * reverse ),
-                             opt.screenY( it->y2_ * reverse ) );
+                             opt.screenXInt( it->x2_ * reverse ),
+                             opt.screenYInt( it->y2_ * reverse ) );
             points.setPoint( 2,
-                             opt.screenX( it->x3_ * reverse ),
-                             opt.screenY( it->y3_ * reverse ) );
+                             opt.screenXInt( it->x3_ * reverse ),
+                             opt.screenYInt( it->y3_ * reverse ) );
 
             painter.drawPolygon( points );
         }
@@ -434,14 +434,14 @@ DebugLogPainter::drawTriangles( QPainter & painter,
 
             QPointArray points( 3 );
             points.setPoint( 0,
-                             opt.screenX( it->x1_ * reverse ),
-                             opt.screenY( it->y1_ * reverse ) );
+                             opt.screenXInt( it->x1_ * reverse ),
+                             opt.screenYInt( it->y1_ * reverse ) );
             points.setPoint( 1,
-                             opt.screenX( it->x2_ * reverse ),
-                             opt.screenY( it->y2_ * reverse ) );
+                             opt.screenXInt( it->x2_ * reverse ),
+                             opt.screenYInt( it->y2_ * reverse ) );
             points.setPoint( 2,
-                             opt.screenX( it->x3_ * reverse ),
-                             opt.screenY( it->y3_ * reverse ) );
+                             opt.screenXInt( it->x3_ * reverse ),
+                             opt.screenYInt( it->y3_ * reverse ) );
 
             painter.drawPolygon( points );
         }
@@ -485,10 +485,10 @@ DebugLogPainter::drawRectangles( QPainter & painter,
                 painter.setPen( dconf.debugShapePen() );
             }
 
-            painter.drawRect( opt.screenX( it->left_ * reverse ),
-                              opt.screenY( it->top_ * reverse ),
-                              opt.scale( it->width_ ),
-                              opt.scale( it->height_ ) );
+            painter.drawRect( opt.screenXInt( it->left_ * reverse ),
+                              opt.screenYInt( it->top_ * reverse ),
+                              opt.scaleInt( it->width_ ),
+                              opt.scaleInt( it->height_ ) );
         }
     }
 
@@ -511,10 +511,10 @@ DebugLogPainter::drawRectangles( QPainter & painter,
                 painter.setBrush( dconf.debugShapePen().color() );
             }
 
-            painter.drawRect( opt.screenX( it->left_ * reverse ),
-                              opt.screenY( it->top_ * reverse ),
-                              opt.scale( it->width_ ),
-                              opt.scale( it->height_ ) );
+            painter.drawRect( opt.screenXInt( it->left_ * reverse ),
+                              opt.screenYInt( it->top_ * reverse ),
+                              opt.scaleInt( it->width_ ),
+                              opt.scaleInt( it->height_ ) );
         }
     }
 
@@ -568,8 +568,8 @@ draw_sector( QPainter & painter,
     rcsc::AngleDeg angle = sector.start_angle_;
 
     rcsc::Vector2D rpos = rcsc::Vector2D::polar2vector( sector.max_r_, angle );
-    const int first_x = opt.screenX( ( sector.x_ + rpos.x ) * reverse );
-    const int first_y = opt.screenY( ( sector.y_ + rpos.y ) * reverse );
+    const int first_x = opt.screenXInt( ( sector.x_ + rpos.x ) * reverse );
+    const int first_y = opt.screenYInt( ( sector.y_ + rpos.y ) * reverse );
 
     points.setPoint( array_index, first_x, first_y );
     ++array_index;
@@ -579,8 +579,8 @@ draw_sector( QPainter & painter,
     {
         rpos = rcsc::Vector2D::polar2vector( sector.max_r_, angle );
         points.setPoint( array_index,
-                         opt.screenX( ( sector.x_ + rpos.x  ) * reverse ),
-                         opt.screenY( ( sector.y_ + rpos.y ) * reverse ) );
+                         opt.screenXInt( ( sector.x_ + rpos.x  ) * reverse ),
+                         opt.screenYInt( ( sector.y_ + rpos.y ) * reverse ) );
         ++array_index;
     }
 
@@ -594,8 +594,8 @@ draw_sector( QPainter & painter,
     }
 
     points.setPoint( array_index,
-                     opt.screenX( ( sector.x_ + rpos.x  ) * reverse ),
-                     opt.screenY( ( sector.y_ + rpos.y ) * reverse ) );
+                     opt.screenXInt( ( sector.x_ + rpos.x  ) * reverse ),
+                     opt.screenYInt( ( sector.y_ + rpos.y ) * reverse ) );
     ++array_index;
 
     angle -= min_angle_step;
@@ -603,8 +603,8 @@ draw_sector( QPainter & painter,
     {
         rpos = rcsc::Vector2D::polar2vector( sector.min_r_, angle );
         points.setPoint( array_index,
-                         opt.screenX( ( sector.x_ + rpos.x  ) * reverse ),
-                         opt.screenY( ( sector.y_ + rpos.y ) * reverse ) );
+                         opt.screenXInt( ( sector.x_ + rpos.x  ) * reverse ),
+                         opt.screenYInt( ( sector.y_ + rpos.y ) * reverse ) );
         ++array_index;
     }
 
@@ -721,8 +721,8 @@ DebugLogPainter::drawMessages( QPainter & painter,
                 painter.setPen( dconf.debugLogMessageFontPen() );
             }
 
-            painter.drawText( opt.screenX( it->x_ * reverse ),
-                              opt.screenY( it->y_ * reverse ),
+            painter.drawText( opt.screenXInt( it->x_ * reverse ),
+                              opt.screenYInt( it->y_ * reverse ),
                               it->message_ );
         }
     }
