@@ -130,7 +130,7 @@ DebugMessageWindow::DebugMessageWindow( QWidget * parent,
     M_debug_text[++i] = tr( "Role" );
     M_debug_text[++i] = tr( "Team" );
     M_debug_text[++i] = tr( "Analyzer" );
-    M_debug_text[++i] = tr( "Recursive" );
+    M_debug_text[++i] = tr( "Chain" );
 
     createActions();
     createWindows();
