@@ -68,8 +68,8 @@ draw_communication( QPainter & painter,
                              ? -1.0
                              : 1.0 );
 
-    const int say_r = std::max( 3, opt.scale( 3.5 ) );
-    const int hear_r = std::max( 2, opt.scale( 3.0 ) );
+    const int say_r = std::max( 3, opt.scaleInt( 3.5 ) );
+    const int hear_r = std::max( 2, opt.scaleInt( 3.0 ) );
 
     //
     // draw say message sender marks
@@ -87,8 +87,8 @@ draw_communication( QPainter & painter,
         const boost::shared_ptr< DebugViewData::SelfT > self = v->self();
         if ( ! self ) continue;
 
-        const int ix = opt.screenX( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
-        const int iy = opt.screenY( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+        const int ix = opt.screenXInt( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+        const int iy = opt.screenYInt( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
 
         painter.drawEllipse( ix - say_r, iy - say_r, say_r * 2, say_r * 2 );
     }
@@ -104,8 +104,8 @@ draw_communication( QPainter & painter,
             return;
         }
 
-        const int self_x = opt.screenX( self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
-        const int self_y = opt.screenY( self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+        const int self_x = opt.screenXInt( self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+        const int self_y = opt.screenYInt( self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
 
         const std::vector< Player > & players = monitor_view.players();
         const std::vector< Player >::const_iterator p_end = players.end();
@@ -125,8 +125,8 @@ draw_communication( QPainter & painter,
                 if ( p->side() == self_side
                      && p->unum() == it->first )
                 {
-                    int px = opt.screenX( p->x() );
-                    int py = opt.screenY( p->y() );
+                    int px = opt.screenXInt( p->x() );
+                    int py = opt.screenYInt( p->y() );
 
                     painter.drawEllipse( px - hear_r, py - hear_r,
                                          hear_r * 2, hear_r * 2 );
@@ -195,9 +195,9 @@ DebugPainter::draw( QPainter & painter )
         if ( ! self ) continue;
 
         const double reverse = ( self_side == rcsc::RIGHT ? -1.0 : 1.0 );
-        const int ix = opt.screenX( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
-        const int iy = opt.screenY( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
-        const int r = std::max( 3, opt.scale( 3.5 ) );
+        const int ix = opt.screenXInt( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+        const int iy = opt.screenYInt( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+        const int r = std::max( 3, opt.scaleInt( 3.5 ) );
 
         painter.setPen( DrawConfig::instance().debugCommentFontPen() );
         painter.setBrush( DrawConfig::instance().transparentBrush() );
@@ -319,19 +319,19 @@ DebugPainter::drawSelf( QPainter & painter,
                              ? 1.0
                              : -1.0 );
 
-    const int ix = opt.screenX( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
-    const int iy = opt.screenY( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+    const int ix = opt.screenXInt( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+    const int iy = opt.screenYInt( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
     const int r = std::max( 2,
                             ( opt.enlargeMode()
-                              ? opt.scale( 1.0 )
-                              : opt.scale( 0.3 ) ) );
+                              ? opt.scaleInt( 1.0 )
+                              : opt.scaleInt( 0.3 ) ) );
 
     // draw targets
     if ( opt.showDebugViewTarget()
          &&  view.targetPoint() )
     {
-        int tx = opt.screenX( view.targetPoint()->x * reverse );
-        int ty = opt.screenY( view.targetPoint()->y * reverse );
+        int tx = opt.screenXInt( view.targetPoint()->x * reverse );
+        int ty = opt.screenYInt( view.targetPoint()->y * reverse );
 
         painter.setPen( dconf.debugTargetPen() );
         painter.setBrush( dconf.transparentBrush() );
@@ -410,12 +410,12 @@ DebugPainter::drawBall( QPainter & painter,
                          (double)ball->vy_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
 
     const double ball_decay = rcsc::ServerParam::i().ballDecay();
-    const int ix = opt.screenX( bpos.x );
-    const int iy = opt.screenY( bpos.y );
+    const int ix = opt.screenXInt( bpos.x );
+    const int iy = opt.screenYInt( bpos.y );
     const int r = std::max( 2,
                             ( opt.enlargeMode()
-                              ? opt.scale( 0.25 )
-                              : opt.scale( 0.07 ) ) );
+                              ? opt.scaleInt( 0.25 )
+                              : opt.scaleInt( 0.07 ) ) );
 
     painter.setPen( dconf.transparentPen() );
     painter.setBrush( dconf.debugBallBrush() );
@@ -435,8 +435,8 @@ DebugPainter::drawBall( QPainter & painter,
         {
             bpos += bvel;
             bvel *= ball_decay;
-            bx = opt.screenX( bpos.x );
-            by = opt.screenY( bpos.y );
+            bx = opt.screenXInt( bpos.x );
+            by = opt.screenYInt( bpos.y );
             painter.drawEllipse( bx - 2, by - 2,
                                  4, 4 );
         }
@@ -480,16 +480,16 @@ DebugPainter::drawPlayers( QPainter & painter,
     const bool comment = opt.showDebugViewComment();
     const int r = std::max( 2,
                             ( opt.enlargeMode()
-                              ? opt.scale( 1.0 )
-                              : opt.scale( 0.3 ) ) );
+                              ? opt.scaleInt( 1.0 )
+                              : opt.scaleInt( 0.3 ) ) );
 
     const DebugViewData::PlayerCont::const_iterator end = players.end();
     for ( DebugViewData::PlayerCont::const_iterator it = players.begin();
           it != end;
           ++it )
     {
-        const int ix = opt.screenX( (double)(*it)->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
-        const int iy = opt.screenY( (double)(*it)->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+        const int ix = opt.screenXInt( (double)(*it)->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+        const int iy = opt.screenYInt( (double)(*it)->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
 
         if ( (*it)->body_ != -360 )
         {
@@ -534,8 +534,8 @@ DebugPainter::drawPlayers( QPainter & painter,
             const boost::shared_ptr< DebugViewData::SelfT > self = view.self();
             if ( self )
             {
-                const int sx = opt.screenX( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
-                const int sy = opt.screenY( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+                const int sx = opt.screenXInt( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+                const int sy = opt.screenYInt( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
                 painter.drawLine( sx, sy, ix, iy );
             }
 
@@ -587,10 +587,10 @@ DebugPainter::drawLines( QPainter & painter,
           it != end;
           ++it )
     {
-        painter.drawLine( opt.screenX( it->x1_ * reverse ),
-                          opt.screenY( it->y1_ * reverse ),
-                          opt.screenX( it->x2_ * reverse ),
-                          opt.screenY( it->y2_ * reverse ) );
+        painter.drawLine( opt.screenXInt( it->x1_ * reverse ),
+                          opt.screenYInt( it->y1_ * reverse ),
+                          opt.screenXInt( it->x2_ * reverse ),
+                          opt.screenYInt( it->y2_ * reverse ) );
     }
 }
 
@@ -623,12 +623,12 @@ DebugPainter::drawTriangles( QPainter & painter,
           it != end;
           ++it )
     {
-        int x1 = opt.screenX( it->x1_ * reverse );
-        int y1 = opt.screenY( it->y1_ * reverse );
-        int x2 = opt.screenX( it->x2_ * reverse );
-        int y2 = opt.screenY( it->y2_ * reverse );
-        int x3 = opt.screenX( it->x3_ * reverse );
-        int y3 = opt.screenY( it->y3_ * reverse );
+        int x1 = opt.screenXInt( it->x1_ * reverse );
+        int y1 = opt.screenYInt( it->y1_ * reverse );
+        int x2 = opt.screenXInt( it->x2_ * reverse );
+        int y2 = opt.screenYInt( it->y2_ * reverse );
+        int x3 = opt.screenXInt( it->x3_ * reverse );
+        int y3 = opt.screenYInt( it->y3_ * reverse );
 
         painter.drawLine( x1, y1,
                           x2, y2 );
@@ -668,10 +668,10 @@ DebugPainter::drawRectangles( QPainter & painter,
           it != end;
           ++it )
     {
-        int left_x = opt.screenX( it->left_x_ * reverse );
-        int top_y = opt.screenY( it->top_y_ * reverse );
-        int right_x = opt.screenX( it->right_x_ * reverse );
-        int bottom_y = opt.screenY( it->bottom_y_ * reverse );
+        int left_x = opt.screenXInt( it->left_x_ * reverse );
+        int top_y = opt.screenYInt( it->top_y_ * reverse );
+        int right_x = opt.screenXInt( it->right_x_ * reverse );
+        int bottom_y = opt.screenYInt( it->bottom_y_ * reverse );
 
         painter.drawRect( left_x,
                           top_y,
@@ -709,9 +709,9 @@ DebugPainter::drawCircles( QPainter & painter,
           it != end;
           ++it )
     {
-        int r = opt.scale( it->radius_ );
-        painter.drawEllipse( opt.screenX( it->center_x_ * reverse ) - r,
-                             opt.screenY( it->center_y_ * reverse ) - r,
+        int r = opt.scaleInt( it->radius_ );
+        painter.drawEllipse( opt.screenXInt( it->center_x_ * reverse ) - r,
+                             opt.screenYInt( it->center_y_ * reverse ) - r,
                              r * 2,
                              r * 2 );
     }
