@@ -74,15 +74,19 @@ private:
     QLabel * M_player_body;
     QLabel * M_player_head;
 
-    QLabel * M_stamina;
-    QLabel * M_stamina_capacity;
-    QLabel * M_effort;
-    QLabel * M_recovery;
+    //QLabel * M_player_catch_prob;
+    QLabel * M_player_tackle_prob;
+    QLabel * M_player_foul_prob;
 
     QLabel * M_player_pointto_pos;
 
     QLabel * M_player_focus_target;
 
+    QLabel * M_stamina;
+    QLabel * M_stamina_capacity;
+    QLabel * M_effort;
+    QLabel * M_recovery;
+
     QLabel * M_kick_count;
     QLabel * M_dash_count;
     QLabel * M_turn_count;
