@@ -462,8 +462,8 @@ FieldCanvas::mouseMoveEvent( QMouseEvent * event )
             }
 
             const Options & opt = Options::instance();
-            int new_x = opt.absScreenX( opt.focusPoint().x );
-            int new_y = opt.absScreenY( opt.focusPoint().y );
+            int new_x = opt.absScreenXInt( opt.focusPoint().x );
+            int new_y = opt.absScreenYInt( opt.focusPoint().y );
             new_x -= ( event->pos().x() - M_mouse_state[0].draggedPoint().x() );
             new_y -= ( event->pos().y() - M_mouse_state[0].draggedPoint().y() );
             emit focusChanged( QPoint( new_x, new_y ) );
@@ -614,8 +614,8 @@ FieldCanvas::drawMouseMeasure( QPainter & painter )
                         start_point.y() - 2 );
         while ( total_travel < max_length )
         {
-            QPoint pt( opt.absScreenX( ball_pos.x ) - 1,
-                       opt.absScreenY( ball_pos.y ) - 1 );
+            QPoint pt( opt.absScreenXInt( ball_pos.x ) - 1,
+                       opt.absScreenYInt( ball_pos.y ) - 1 );
             if ( std::abs( pt.x() - last_pt.x() ) < 1
                  && std::abs( pt.y() - last_pt.y() ) < 1 )
             {
@@ -717,7 +717,7 @@ FieldCanvas::drawDraggedPlayer( QPainter & painter )
     const Player & player = view->players()[index];
     const rcsc::PlayerType & type = M_main_data.viewHolder().playerType( player.type() );
 
-    int draw_radius = opt.scale( type.kickableArea() );
+    int draw_radius = opt.scaleInt( type.kickableArea() );
 
     const DrawConfig & dconf = DrawConfig::instance();
 
@@ -820,8 +820,8 @@ FieldCanvas::dragPlayer( const QPoint & point )
 
     // calculate an update area
     int player_size = ( opt.playerSize() > 0.0
-                        ? opt.scale( opt.playerSize() + 0.3 )
-                        : opt.scale( rcsc::ServerParam::i().defaultKickableArea() + 0.3 ) );
+                        ? opt.scaleInt( opt.playerSize() + 0.3 )
+                        : opt.scaleInt( rcsc::ServerParam::i().defaultKickableArea() + 0.3 ) );
 
     QRect new_rect( point.x() - player_size,
                     point.y() - player_size,
