@@ -112,17 +112,17 @@ FieldPainter::drawLinesGrass( QPainter & painter ) const
     const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
-    const int left_x = opt.fieldCenter().x - opt.scale( SP.pitchHalfLength() );
-    const int top_y = opt.fieldCenter().y - opt.scale( SP.pitchHalfWidth() );
-    const int right_x = opt.fieldCenter().x + opt.scale( SP.pitchHalfLength() );
-    const int bottom_y = opt.fieldCenter().y + opt.scale( SP.pitchHalfWidth() );
+    const int left_x = opt.fieldCenterInt().x - opt.scaleInt( SP.pitchHalfLength() );
+    const int top_y = opt.fieldCenterInt().y - opt.scaleInt( SP.pitchHalfWidth() );
+    const int right_x = opt.fieldCenterInt().x + opt.scaleInt( SP.pitchHalfLength() );
+    const int bottom_y = opt.fieldCenterInt().y + opt.scaleInt( SP.pitchHalfWidth() );
 
-    const int grid = opt.scale( 10.0 ); // grid length
+    const int grid = opt.scaleInt( 10.0 ); // grid length
 
     painter.setPen( dconf.transparentPen() );
     //painter.setBrush( dconf.fieldDarkBrush() );
 
-    int i = opt.fieldCenter().x - grid/2;
+    int i = opt.fieldCenterInt().x - grid/2;
     int cnt = 0;
     while ( i > left_x )
     {
@@ -137,7 +137,7 @@ FieldPainter::drawLinesGrass( QPainter & painter ) const
         ++cnt;
     }
 
-    i = opt.fieldCenter().x + grid/2;
+    i = opt.fieldCenterInt().x + grid/2;
     cnt = 0;
     while ( i < right_x )
     {
@@ -167,19 +167,19 @@ FieldPainter::drawCheckerGrass( QPainter & painter )const
     painter.setPen( dconf.transparentPen() );
     painter.setBrush( dconf.fieldDarkBrush() );
 
-    const int left_x = opt.fieldCenter().x - opt.scale( SP.pitchHalfLength() );
-    const int top_y = opt.fieldCenter().y - opt.scale( SP.pitchHalfWidth() );
-    const int right_x = opt.fieldCenter().x + opt.scale( SP.pitchHalfLength() );
-    const int bottom_y = opt.fieldCenter().y + opt.scale( SP.pitchHalfWidth() );
+    const int left_x = opt.fieldCenterInt().x - opt.scaleInt( SP.pitchHalfLength() );
+    const int top_y = opt.fieldCenterInt().y - opt.scaleInt( SP.pitchHalfWidth() );
+    const int right_x = opt.fieldCenterInt().x + opt.scaleInt( SP.pitchHalfLength() );
+    const int bottom_y = opt.fieldCenterInt().y + opt.scaleInt( SP.pitchHalfWidth() );
 
-    const int grid = opt.scale( 10.0 ); // grid length
+    const int grid = opt.scaleInt( 10.0 ); // grid length
 
     int flag = 0;
-    for ( int i = opt.fieldCenter().x; i > left_x; i -= grid )
+    for ( int i = opt.fieldCenterInt().x; i > left_x; i -= grid )
     {
         int cnt = flag;
         int left = i - grid;
-        for ( int j = opt.fieldCenter().y; j > top_y; j -= grid )
+        for ( int j = opt.fieldCenterInt().y; j > top_y; j -= grid )
         {
             if ( cnt % 2 )
             {
@@ -193,11 +193,11 @@ FieldPainter::drawCheckerGrass( QPainter & painter )const
     }
 
     flag = 1;
-    for ( int i = opt.fieldCenter().x;  i > left_x; i -= grid )
+    for ( int i = opt.fieldCenterInt().x;  i > left_x; i -= grid )
     {
         int cnt = flag;
         int left = i - grid;
-        for ( int j = opt.fieldCenter().y; j < bottom_y; j += grid )
+        for ( int j = opt.fieldCenterInt().y; j < bottom_y; j += grid )
         {
             if ( cnt % 2 )
             {
@@ -210,11 +210,11 @@ FieldPainter::drawCheckerGrass( QPainter & painter )const
         ++flag;
     }
     flag = 0;
-    for ( int i = opt.fieldCenter().x;  i < right_x; i += grid )
+    for ( int i = opt.fieldCenterInt().x;  i < right_x; i += grid )
     {
         int cnt = flag;
         int right = i + grid;
-        for ( int j = opt.fieldCenter().y; j < bottom_y; j += grid )
+        for ( int j = opt.fieldCenterInt().y; j < bottom_y; j += grid )
         {
             if ( cnt % 2 )
             {
@@ -227,11 +227,11 @@ FieldPainter::drawCheckerGrass( QPainter & painter )const
         ++flag;
     }
     flag = 1;
-    for ( int i = opt.fieldCenter().x;  i < right_x; i += grid )
+    for ( int i = opt.fieldCenterInt().x;  i < right_x; i += grid )
     {
         int cnt = flag;
         int right = i + grid;
-        for ( int j = opt.fieldCenter().y; j > top_y; j -= grid )
+        for ( int j = opt.fieldCenterInt().y; j > top_y; j -= grid )
         {
             if ( cnt % 2 )
             {
@@ -261,10 +261,10 @@ FieldPainter::drawLines( QPainter & painter ) const
     painter.setBrush( dconf.transparentBrush() );
 
     // set screen coordinates of field
-    int left_x   = opt.absScreenX( - SP.pitchHalfLength() );
-    int right_x  = opt.absScreenX( + SP.pitchHalfLength() );
-    int top_y    = opt.absScreenY( - SP.pitchHalfWidth() );
-    int bottom_y = opt.absScreenY( + SP.pitchHalfWidth() );
+    int left_x   = opt.absScreenXInt( - SP.pitchHalfLength() );
+    int right_x  = opt.absScreenXInt( + SP.pitchHalfLength() );
+    int top_y    = opt.absScreenYInt( - SP.pitchHalfWidth() );
+    int bottom_y = opt.absScreenYInt( + SP.pitchHalfWidth() );
 
     // side lines & goal lines
     painter.drawLine( left_x, top_y, right_x, top_y );
@@ -276,30 +276,30 @@ FieldPainter::drawLines( QPainter & painter ) const
          || SP.keepawayMode() )
     {
         // keepaway area
-        int ka_left = opt.absScreenX( - SP.keepawayLength() * 0.5 );
-        int ka_top = opt.absScreenY( - SP.keepawayWidth() * 0.5 );
-        int ka_length = opt.scale( SP.keepawayLength() );
-        int ka_width = opt.scale( SP.keepawayWidth() );
+        int ka_left = opt.absScreenXInt( - SP.keepawayLength() * 0.5 );
+        int ka_top = opt.absScreenYInt( - SP.keepawayWidth() * 0.5 );
+        int ka_length = opt.scaleInt( SP.keepawayLength() );
+        int ka_width = opt.scaleInt( SP.keepawayWidth() );
 
         painter.drawRect( ka_left, ka_top, ka_length, ka_width );
     }
     else
     {
         // center line
-        painter.drawLine( opt.fieldCenter().x, top_y,
-                          opt.fieldCenter().x, bottom_y );
+        painter.drawLine( opt.fieldCenterInt().x, top_y,
+                          opt.fieldCenterInt().x, bottom_y );
 
         // center circle
-        int center_radius = opt.scale( SP.centerCircleR() );
-        painter.drawEllipse( opt.fieldCenter().x - center_radius,
-                             opt.fieldCenter().y - center_radius,
+        int center_radius = opt.scaleInt( SP.centerCircleR() );
+        painter.drawEllipse( opt.fieldCenterInt().x - center_radius,
+                             opt.fieldCenterInt().y - center_radius,
                              center_radius * 2,
                              center_radius * 2 );
     }
 
     // corner arc
     {
-        int r = opt.scale( SP.cornerArcR() );
+        int r = opt.scaleInt( SP.cornerArcR() );
 
         painter.drawArc( left_x - r, top_y - r, r * 2, r * 2,
                          -90*16, 90*16 );
@@ -328,29 +328,29 @@ FieldPainter::drawPenaltyAreaLines( QPainter & painter ) const
     painter.setBrush( dconf.transparentBrush() );
 
     // set absScreen coordinates of field
-    int left_x   = opt.absScreenX( - SP.pitchHalfLength() );
-    int right_x  = opt.absScreenX( + SP.pitchHalfLength() );
+    int left_x   = opt.absScreenXInt( - SP.pitchHalfLength() );
+    int right_x  = opt.absScreenXInt( + SP.pitchHalfLength() );
 
     // set penalty area params
-    int pen_top_y    = opt.absScreenY( - SP.penaltyAreaHalfWidth() );
-    int pen_bottom_y = opt.absScreenY( + SP.penaltyAreaHalfWidth() );
+    int pen_top_y    = opt.absScreenYInt( - SP.penaltyAreaHalfWidth() );
+    int pen_bottom_y = opt.absScreenYInt( + SP.penaltyAreaHalfWidth() );
     double pen_circle_y_degree_abs
         = std::acos( ( SP.penaltyAreaLength()
                        - rcsc::ServerParam::DEFAULT_PENALTY_SPOT_DIST )
                      / rcsc::ServerParam::DEFAULT_PENALTY_CIRCLE_R )
         * ( 180.0 / M_PI );
     int span_angle = qRound( pen_circle_y_degree_abs * 2.0 * 16 );
-    int pen_circle_r = opt.scale( rcsc::ServerParam::DEFAULT_PENALTY_CIRCLE_R );
-    int pen_circle_size = opt.scale( rcsc::ServerParam::DEFAULT_PENALTY_CIRCLE_R * 2.0 );
+    int pen_circle_r = opt.scaleInt( rcsc::ServerParam::DEFAULT_PENALTY_CIRCLE_R );
+    int pen_circle_size = opt.scaleInt( rcsc::ServerParam::DEFAULT_PENALTY_CIRCLE_R * 2.0 );
 
     // left penalty area X
-    int pen_x = opt.absScreenX( -( SP.pitchHalfLength()
+    int pen_x = opt.absScreenXInt( -( SP.pitchHalfLength()
                                    - SP.penaltyAreaLength() ) );
     // left arc
-    int pen_spot_x = opt.absScreenX( -( SP.pitchHalfLength()
+    int pen_spot_x = opt.absScreenXInt( -( SP.pitchHalfLength()
                                         - rcsc::ServerParam::DEFAULT_PENALTY_SPOT_DIST ) );
     painter.drawArc( pen_spot_x - pen_circle_r + 1,
-                     opt.fieldCenter().y - pen_circle_r,
+                     opt.fieldCenterInt().y - pen_circle_r,
                      pen_circle_size,
                      pen_circle_size,
                      qRound( -pen_circle_y_degree_abs * 16 ),
@@ -360,16 +360,16 @@ FieldPainter::drawPenaltyAreaLines( QPainter & painter ) const
     painter.drawLine( pen_x, pen_top_y, pen_x, pen_bottom_y );
     painter.drawLine( pen_x, pen_bottom_y, left_x, pen_bottom_y );
     // left spot
-    painter.drawPoint( pen_spot_x, opt.fieldCenter().y );
+    painter.drawPoint( pen_spot_x, opt.fieldCenterInt().y );
 
     // right penalty area X
-    pen_x = opt.absScreenX( +( SP.pitchHalfLength()
+    pen_x = opt.absScreenXInt( +( SP.pitchHalfLength()
                                - SP.penaltyAreaLength() ) );
     // right arc
-    pen_spot_x = opt.absScreenX( +( SP.pitchHalfLength()
+    pen_spot_x = opt.absScreenXInt( +( SP.pitchHalfLength()
                                     - rcsc::ServerParam::DEFAULT_PENALTY_SPOT_DIST ) );
     painter.drawArc( pen_spot_x - pen_circle_r,
-                     opt.fieldCenter().y - pen_circle_r,
+                     opt.fieldCenterInt().y - pen_circle_r,
                      pen_circle_size, pen_circle_size,
                      qRound( ( 180.0 - pen_circle_y_degree_abs + 0.5 ) * 16 ),
                      span_angle );
@@ -378,7 +378,7 @@ FieldPainter::drawPenaltyAreaLines( QPainter & painter ) const
     painter.drawLine( pen_x, pen_top_y, pen_x, pen_bottom_y );
     painter.drawLine( pen_x, pen_bottom_y, right_x, pen_bottom_y );
     // right spot
-    painter.drawPoint( pen_spot_x, opt.fieldCenter().y );
+    painter.drawPoint( pen_spot_x, opt.fieldCenterInt().y );
 }
 
 /*-------------------------------------------------------------------*/
@@ -397,23 +397,23 @@ FieldPainter::drawGoalAreaLines( QPainter & painter ) const
     painter.setBrush( dconf.transparentBrush() );
 
     // set absScreen coordinates of field
-    int left_x   = opt.absScreenX( - SP.pitchHalfLength() );
-    int right_x  = opt.absScreenX( + SP.pitchHalfLength() );
+    int left_x   = opt.absScreenXInt( - SP.pitchHalfLength() );
+    int right_x  = opt.absScreenXInt( + SP.pitchHalfLength() );
 
     // set coordinates opts
-    int goal_area_y_abs = opt.scale( SP.goalAreaHalfWidth() );
-    int goal_area_top_y = opt.fieldCenter().y - goal_area_y_abs;
-    int goal_area_bottom_y = opt.fieldCenter().y + goal_area_y_abs;
+    int goal_area_y_abs = opt.scaleInt( SP.goalAreaHalfWidth() );
+    int goal_area_top_y = opt.fieldCenterInt().y - goal_area_y_abs;
+    int goal_area_bottom_y = opt.fieldCenterInt().y + goal_area_y_abs;
 
     // left goal area
-    int goal_area_x = opt.absScreenX( - SP.pitchHalfLength()
+    int goal_area_x = opt.absScreenXInt( - SP.pitchHalfLength()
                                       + SP.goalAreaLength() );
     painter.drawLine( left_x, goal_area_top_y, goal_area_x, goal_area_top_y );
     painter.drawLine( goal_area_x, goal_area_top_y, goal_area_x, goal_area_bottom_y );
     painter.drawLine( goal_area_x, goal_area_bottom_y, left_x, goal_area_bottom_y );
 
     // right goal area
-    goal_area_x = opt.absScreenX( SP.pitchHalfLength()
+    goal_area_x = opt.absScreenXInt( SP.pitchHalfLength()
                                   - SP.goalAreaLength() );
     painter.drawLine( right_x, goal_area_top_y, goal_area_x, goal_area_top_y );
     painter.drawLine( goal_area_x, goal_area_top_y, goal_area_x, goal_area_bottom_y );
@@ -435,24 +435,24 @@ FieldPainter::drawGoals( QPainter & painter ) const
     painter.setBrush( Qt::black );
 
     // set coordinates param
-    int goal_top_y = opt.absScreenY( - SP.goalHalfWidth() );
-    int goal_size_x = opt.scale( SP.goalDepth() );
-    int goal_size_y = opt.scale( SP.goalWidth() );
+    int goal_top_y = opt.absScreenYInt( - SP.goalHalfWidth() );
+    int goal_size_x = opt.scaleInt( SP.goalDepth() );
+    int goal_size_y = opt.scaleInt( SP.goalWidth() );
 
-    int post_top_y = opt.absScreenY( - SP.goalHalfWidth()
+    int post_top_y = opt.absScreenYInt( - SP.goalHalfWidth()
                                      - SP.goalPostRadius() * 2.0 );
-    int post_bottom_y = opt.absScreenY( + SP.goalHalfWidth() );
-    int post_diameter = opt.scale( SP.goalPostRadius() * 2.0 );
+    int post_bottom_y = opt.absScreenYInt( + SP.goalHalfWidth() );
+    int post_diameter = opt.scaleInt( SP.goalPostRadius() * 2.0 );
 
     // left goal
-    painter.drawRect( opt.absScreenX( - SP.pitchHalfLength()
+    painter.drawRect( opt.absScreenXInt( - SP.pitchHalfLength()
                                       - SP.goalDepth() ) - 1,
                       goal_top_y,
                       goal_size_x,
                       goal_size_y );
     if ( post_diameter >= 1 )
     {
-        int post_x = opt.absScreenX( - SP.pitchHalfLength() );
+        int post_x = opt.absScreenXInt( - SP.pitchHalfLength() );
         painter.drawEllipse( post_x,
                              post_top_y,
                              post_diameter,
@@ -463,13 +463,13 @@ FieldPainter::drawGoals( QPainter & painter ) const
                              post_diameter );
     }
     // right goal
-    painter.drawRect( opt.absScreenX( SP.pitchHalfLength() ) + 1,
+    painter.drawRect( opt.absScreenXInt( SP.pitchHalfLength() ) + 1,
                       goal_top_y,
                       goal_size_x,
                       goal_size_y );
     if ( post_diameter >= 1 )
     {
-        int post_x = opt.absScreenX( SP.pitchHalfLength()
+        int post_x = opt.absScreenXInt( SP.pitchHalfLength()
                                      - SP.goalPostRadius() * 2.0 );
         painter.drawEllipse( post_x,
                              post_top_y,
@@ -497,229 +497,229 @@ FieldPainter::drawFlags( QPainter & painter ) const
     painter.setBrush( DrawConfig::instance().transparentBrush() );
 
     // set size or coordinates params
-    int flag_radius = opt.scale( 0.5 );
+    int flag_radius = opt.scaleInt( 0.5 );
     if ( flag_radius < 2 ) flag_radius = 2;
     if ( flag_radius > 5 ) flag_radius = 5;
     int flag_diameter = flag_radius * 2;
 
     int x, y;
-    int pitch_half_length = opt.scale( SP.pitchHalfLength() );
-    int pitch_half_width = opt.scale( SP.pitchHalfWidth() );
-    int pitch_margin_x = opt.scale( SP.pitchHalfLength()
+    int pitch_half_length = opt.scaleInt( SP.pitchHalfLength() );
+    int pitch_half_width = opt.scaleInt( SP.pitchHalfWidth() );
+    int pitch_margin_x = opt.scaleInt( SP.pitchHalfLength()
                                     + rcsc::ServerParam::DEFAULT_PITCH_MARGIN );
-    int pitch_margin_y = opt.scale( SP.pitchHalfWidth()
+    int pitch_margin_y = opt.scaleInt( SP.pitchHalfWidth()
                                     + rcsc::ServerParam::DEFAULT_PITCH_MARGIN );
-    int penalty_x = opt.scale( SP.pitchHalfLength()
+    int penalty_x = opt.scaleInt( SP.pitchHalfLength()
                                - SP.penaltyAreaLength() );
-    int penalty_y = opt.scale( SP.penaltyAreaHalfWidth() );
-    int goal_y = opt.scale( SP.goalHalfWidth() );
-    int scale10 = opt.scale( 10.0 );
-    int scale20 = opt.scale( 20.0 );
-    int scale30 = opt.scale( 30.0 );
-    int scale40 = opt.scale( 40.0 );
-    int scale50 = opt.scale( 50.0 );
+    int penalty_y = opt.scaleInt( SP.penaltyAreaHalfWidth() );
+    int goal_y = opt.scaleInt( SP.goalHalfWidth() );
+    int scale10 = opt.scaleInt( 10.0 );
+    int scale20 = opt.scaleInt( 20.0 );
+    int scale30 = opt.scaleInt( 30.0 );
+    int scale40 = opt.scaleInt( 40.0 );
+    int scale50 = opt.scaleInt( 50.0 );
 
     // goal left
-    x = opt.fieldCenter().x - pitch_half_length - flag_radius;
-    y = opt.fieldCenter().y - flag_radius;
+    x = opt.fieldCenterInt().x - pitch_half_length - flag_radius;
+    y = opt.fieldCenterInt().y - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // goal right
-    x = opt.fieldCenter().x + pitch_half_length - flag_radius;
-    y = opt.fieldCenter().y - flag_radius;
+    x = opt.fieldCenterInt().x + pitch_half_length - flag_radius;
+    y = opt.fieldCenterInt().y - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag c
-    x  = opt.fieldCenter().x - flag_radius;
-    y = opt.fieldCenter().y - flag_radius;
+    x  = opt.fieldCenterInt().x - flag_radius;
+    y = opt.fieldCenterInt().y - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag c t
-    x = opt.fieldCenter().x - flag_radius;
-    y = opt.fieldCenter().y - pitch_half_width - flag_radius;
+    x = opt.fieldCenterInt().x - flag_radius;
+    y = opt.fieldCenterInt().y - pitch_half_width - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
 
     // flag c b
-    x = opt.fieldCenter().x - flag_radius;
-    y = opt.fieldCenter().y + pitch_half_width - flag_radius;
+    x = opt.fieldCenterInt().x - flag_radius;
+    y = opt.fieldCenterInt().y + pitch_half_width - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag l t
-    x = opt.fieldCenter().x - pitch_half_length - flag_radius;
-    y = opt.fieldCenter().y - pitch_half_width - flag_radius;
+    x = opt.fieldCenterInt().x - pitch_half_length - flag_radius;
+    y = opt.fieldCenterInt().y - pitch_half_width - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag l b
-    x = opt.fieldCenter().x - pitch_half_length - flag_radius;
-    y = opt.fieldCenter().y + pitch_half_width - flag_radius;
+    x = opt.fieldCenterInt().x - pitch_half_length - flag_radius;
+    y = opt.fieldCenterInt().y + pitch_half_width - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag r t
-    x = opt.fieldCenter().x + pitch_half_length - flag_radius;
-    y = opt.fieldCenter().y - pitch_half_width - flag_radius;
+    x = opt.fieldCenterInt().x + pitch_half_length - flag_radius;
+    y = opt.fieldCenterInt().y - pitch_half_width - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag r b
-    x = opt.fieldCenter().x + pitch_half_length - flag_radius;
-    y = opt.fieldCenter().y + pitch_half_width - flag_radius;
+    x = opt.fieldCenterInt().x + pitch_half_length - flag_radius;
+    y = opt.fieldCenterInt().y + pitch_half_width - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag p l t
-    x = opt.fieldCenter().x - penalty_x - flag_radius;
-    y = opt.fieldCenter().y - penalty_y - flag_radius;
+    x = opt.fieldCenterInt().x - penalty_x - flag_radius;
+    y = opt.fieldCenterInt().y - penalty_y - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag p l c
-    x = opt.fieldCenter().x - penalty_x - flag_radius;
-    y = opt.fieldCenter().y - flag_radius;
+    x = opt.fieldCenterInt().x - penalty_x - flag_radius;
+    y = opt.fieldCenterInt().y - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag p l b
-    x = opt.fieldCenter().x - penalty_x - flag_radius;
-    y = opt.fieldCenter().y + penalty_y - flag_radius;
+    x = opt.fieldCenterInt().x - penalty_x - flag_radius;
+    y = opt.fieldCenterInt().y + penalty_y - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag p r t
-    x = opt.fieldCenter().x + penalty_x - flag_radius;
-    y = opt.fieldCenter().y - penalty_y - flag_radius;
+    x = opt.fieldCenterInt().x + penalty_x - flag_radius;
+    y = opt.fieldCenterInt().y - penalty_y - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag p r c
-    x = opt.fieldCenter().x + penalty_x - flag_radius;
-    y = opt.fieldCenter().y - flag_radius;
+    x = opt.fieldCenterInt().x + penalty_x - flag_radius;
+    y = opt.fieldCenterInt().y - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag p r b
-    x = opt.fieldCenter().x + penalty_x - flag_radius;
-    y = opt.fieldCenter().y + penalty_y - flag_radius;
+    x = opt.fieldCenterInt().x + penalty_x - flag_radius;
+    y = opt.fieldCenterInt().y + penalty_y - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
 
     // flag g l t
-    x = opt.fieldCenter().x - pitch_half_length - flag_radius;
-    y = opt.fieldCenter().y - goal_y - flag_radius;
+    x = opt.fieldCenterInt().x - pitch_half_length - flag_radius;
+    y = opt.fieldCenterInt().y - goal_y - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag g l b
-    x = opt.fieldCenter().x - pitch_half_length - flag_radius;
-    y = opt.fieldCenter().y + goal_y - flag_radius;
+    x = opt.fieldCenterInt().x - pitch_half_length - flag_radius;
+    y = opt.fieldCenterInt().y + goal_y - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag g r t
-    x = opt.fieldCenter().x + pitch_half_length - flag_radius;
-    y = opt.fieldCenter().y - goal_y - flag_radius;
+    x = opt.fieldCenterInt().x + pitch_half_length - flag_radius;
+    y = opt.fieldCenterInt().y - goal_y - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag g r b
-    x = opt.fieldCenter().x + pitch_half_length - flag_radius;
-    y = opt.fieldCenter().y + goal_y - flag_radius;
+    x = opt.fieldCenterInt().x + pitch_half_length - flag_radius;
+    y = opt.fieldCenterInt().y + goal_y - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
 
     // flag t ...
 
-    y = opt.fieldCenter().y - pitch_margin_y - flag_radius;
+    y = opt.fieldCenterInt().y - pitch_margin_y - flag_radius;
     // flag t l 50
-    x = opt.fieldCenter().x - scale50 - flag_radius;
+    x = opt.fieldCenterInt().x - scale50 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag t l 40
-    x = opt.fieldCenter().x - scale40 - flag_radius;
+    x = opt.fieldCenterInt().x - scale40 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag t l 30
-    x = opt.fieldCenter().x - scale30 - flag_radius;
+    x = opt.fieldCenterInt().x - scale30 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag t l 20
-    x = opt.fieldCenter().x - scale20 - flag_radius;
+    x = opt.fieldCenterInt().x - scale20 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag t l 10
-    x = opt.fieldCenter().x - scale10 - flag_radius;
+    x = opt.fieldCenterInt().x - scale10 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag t 0
-    x = opt.fieldCenter().x - flag_radius;
+    x = opt.fieldCenterInt().x - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag t r 10
-    x = opt.fieldCenter().x + scale10 - flag_radius;
+    x = opt.fieldCenterInt().x + scale10 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag t r 20
-    x = opt.fieldCenter().x + scale20 - flag_radius;
+    x = opt.fieldCenterInt().x + scale20 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag t r 30
-    x = opt.fieldCenter().x + scale30 - flag_radius;
+    x = opt.fieldCenterInt().x + scale30 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag t r 40
-    x = opt.fieldCenter().x + scale40 - flag_radius;
+    x = opt.fieldCenterInt().x + scale40 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag t r 50
-    x = opt.fieldCenter().x + scale50 - flag_radius;
+    x = opt.fieldCenterInt().x + scale50 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
 
     // flag b ...
 
-    y = opt.fieldCenter().y + pitch_margin_y - flag_radius;
+    y = opt.fieldCenterInt().y + pitch_margin_y - flag_radius;
     // flag b l 50
-    x = opt.fieldCenter().x - scale50 - flag_radius;
+    x = opt.fieldCenterInt().x - scale50 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag b l 40
-    x = opt.fieldCenter().x - scale40 - flag_radius;
+    x = opt.fieldCenterInt().x - scale40 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag b l 30
-    x = opt.fieldCenter().x - scale30 - flag_radius;
+    x = opt.fieldCenterInt().x - scale30 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag b l 20
-    x = opt.fieldCenter().x - scale20 - flag_radius;
+    x = opt.fieldCenterInt().x - scale20 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag b l 10
-    x = opt.fieldCenter().x - scale10 - flag_radius;
+    x = opt.fieldCenterInt().x - scale10 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag b 0
-    x = opt.fieldCenter().x - flag_radius;
+    x = opt.fieldCenterInt().x - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag b r 10
-    x = opt.fieldCenter().x + scale10 - flag_radius;
+    x = opt.fieldCenterInt().x + scale10 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag b r 20
-    x = opt.fieldCenter().x + scale20 - flag_radius;
+    x = opt.fieldCenterInt().x + scale20 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag b r 30
-    x = opt.fieldCenter().x + scale30 - flag_radius;
+    x = opt.fieldCenterInt().x + scale30 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag b r 40
-    x = opt.fieldCenter().x + scale40 - flag_radius;
+    x = opt.fieldCenterInt().x + scale40 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag b r 50
-    x = opt.fieldCenter().x + scale50 - flag_radius;
+    x = opt.fieldCenterInt().x + scale50 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
 
     // flag l ...
 
-    x = opt.fieldCenter().x - pitch_margin_x - flag_radius;
+    x = opt.fieldCenterInt().x - pitch_margin_x - flag_radius;
     // flag l t 30
-    y = opt.fieldCenter().y - scale30 - flag_radius;
+    y = opt.fieldCenterInt().y - scale30 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag l t 20
-    y = opt.fieldCenter().y - scale20 - flag_radius;
+    y = opt.fieldCenterInt().y - scale20 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag l t 10
-    y = opt.fieldCenter().y - scale10 - flag_radius;
+    y = opt.fieldCenterInt().y - scale10 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag l 0
-    y = opt.fieldCenter().y - flag_radius;
+    y = opt.fieldCenterInt().y - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag l b 10
-    y = opt.fieldCenter().y + scale10 - flag_radius;
+    y = opt.fieldCenterInt().y + scale10 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag l b 20
-    y = opt.fieldCenter().y + scale20 - flag_radius;
+    y = opt.fieldCenterInt().y + scale20 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag l b 30
-    y = opt.fieldCenter().y + scale30 - flag_radius;
+    y = opt.fieldCenterInt().y + scale30 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
 
     // flag r ...
 
-    x = opt.fieldCenter().x + pitch_margin_x - flag_radius;
+    x = opt.fieldCenterInt().x + pitch_margin_x - flag_radius;
     // flag r t 30
-    y = opt.fieldCenter().y - scale30 - flag_radius;
+    y = opt.fieldCenterInt().y - scale30 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag r t 20
-    y = opt.fieldCenter().y - scale20 - flag_radius;
+    y = opt.fieldCenterInt().y - scale20 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag r t 10
-    y = opt.fieldCenter().y - scale10 - flag_radius;
+    y = opt.fieldCenterInt().y - scale10 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag r 0
-    y = opt.fieldCenter().y - flag_radius;
+    y = opt.fieldCenterInt().y - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag r b 10
-    y = opt.fieldCenter().y + scale10 - flag_radius;
+    y = opt.fieldCenterInt().y + scale10 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag r b 20
-    y = opt.fieldCenter().y + scale20 - flag_radius;
+    y = opt.fieldCenterInt().y + scale20 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
     // flag r b 30
-    y = opt.fieldCenter().y + scale30 - flag_radius;
+    y = opt.fieldCenterInt().y + scale30 - flag_radius;
     painter.drawEllipse( x, y, flag_diameter, flag_diameter );
 }
 
@@ -733,7 +733,7 @@ FieldPainter::drawGrid( QPainter & painter ) const
     const Options & opt = Options::instance();
 
     const double grid_step = 0.1 * opt.gridStep();
-    const int istep = opt.scale( grid_step );
+    const int istep = opt.scaleInt( grid_step );
 
     if ( istep <= 4 )
     {
@@ -771,7 +771,7 @@ FieldPainter::drawGrid( QPainter & painter ) const
     double x = 0.0;
     while ( x < max_x )
     {
-        int ix = opt.absScreenX( x );
+        int ix = opt.absScreenXInt( x );
         if ( istep > text_step_x )
         {
             painter.drawText( ix, max_iy, QString::number( x ) );
@@ -783,7 +783,7 @@ FieldPainter::drawGrid( QPainter & painter ) const
     x = -grid_step;
     while ( min_x < x )
     {
-        int ix = opt.absScreenX( x );
+        int ix = opt.absScreenXInt( x );
         if ( istep > text_step_x )
         {
             painter.drawText( ix, max_iy, QString::number( x ) );
@@ -796,7 +796,7 @@ FieldPainter::drawGrid( QPainter & painter ) const
     double y = 0.0;
     while ( y < max_y )
     {
-        int iy = opt.absScreenY( y );
+        int iy = opt.absScreenYInt( y );
         if ( istep > text_step_y )
         {
             painter.drawText( min_ix, iy, QString::number( y ) );
@@ -808,7 +808,7 @@ FieldPainter::drawGrid( QPainter & painter ) const
     y = -grid_step;
     while ( min_y < y )
     {
-        int iy = opt.absScreenY( y );
+        int iy = opt.absScreenYInt( y );
         if ( istep > text_step_y )
         {
             painter.drawText( min_ix, iy, QString::number( y ) );
