@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -37,12 +37,12 @@
 
 #include "image_save_dialog.h"
 
-#include "app_config.h"
 #include "main_data.h"
 #include "main_window.h"
 #include "field_canvas.h"
 #include "view_holder.h"
 //#include "dir_selector.h"
+#include "options.h"
 
 #include <algorithm>
 #include <iostream>
@@ -87,6 +87,7 @@ ImageSaveDialog::createControls()
 {
     QVBoxLayout * layout = new QVBoxLayout( this );
     layout->setMargin( 4 );
+    layout->setResizeMode( QLayout::Fixed );
 
     layout->addWidget( createCycleSelectControls(),
                        0, Qt::AlignLeft );
@@ -154,18 +155,18 @@ ImageSaveDialog::createFileNameControls()
     new QLabel( tr( "Name Prefix: " ), group_box );
 
     M_name_prefix = new QLineEdit( tr( "image-" ), group_box );
-    if ( ! AppConfig::instance().imageNamePrefix().empty() )
+    if ( ! Options::instance().imageNamePrefix().empty() )
     {
-        M_name_prefix->setText( AppConfig::instance().imageNamePrefix().c_str() );
+        M_name_prefix->setText( Options::instance().imageNamePrefix().c_str() );
     }
     M_name_prefix->setMaximumSize( 80, 24 );
 
     new QLabel( tr( " Format: " ), group_box );
 
     const QString default_format
-        = ( AppConfig::instance().imageSaveFormat().empty()
+        = ( Options::instance().imageSaveFormat().empty()
             ? QString::fromAscii( "PNG" )
-            : QString::fromAscii( AppConfig::instance().imageSaveFormat().c_str() ) );
+            : QString::fromAscii( Options::instance().imageSaveFormat().c_str() ) );
 
     M_format_choice = new QComboBox( group_box );
     {
@@ -204,9 +205,9 @@ ImageSaveDialog::createDirSelectControls()
     QHGroupBox * group_box = new QHGroupBox( tr( "Save Directry" ), this );
 
     QString dir_str;
-    if ( ! AppConfig::instance().imageSaveDir().empty() )
+    if ( ! Options::instance().imageSaveDir().empty() )
     {
-        QFileInfo file_info( AppConfig::instance().imageSaveDir().c_str() );
+        QFileInfo file_info( Options::instance().imageSaveDir().c_str() );
         if ( ! file_info.isDir() )
         {
             QDir dir;
@@ -271,7 +272,7 @@ ImageSaveDialog::createExecuteControls()
 
 */
 void
-ImageSaveDialog::showEvent( QShowEvent * event )
+ImageSaveDialog::showEvent( QShowEvent * )
 {
     const std::vector< MonitorViewPtr > & view
         = M_main_data.viewHolder().monitorViewCont();
@@ -457,7 +458,7 @@ ImageSaveDialog::saveImage( const int start_cycle,
     {
         // full file path
         char count[16];
-        std::snprintf( count, 16, "%05d", i );
+        snprintf( count, 16, "%05d", i );
 
         QString file_path_all = file_path;
         file_path_all += QString::fromAscii( count );
