@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -37,13 +37,13 @@
 
 #include "launcher_dialog.h"
 
-#include "app_config.h"
+#include "options.h"
 
 #include <iostream>
 #include <cassert>
 #include <cstdlib>
 
-const int LauncherDialog::MAX_HISTORY = 10;
+const int LauncherDialog::MAX_HISTORY = 20;
 
 /*-------------------------------------------------------------------*/
 /*!
@@ -178,7 +178,7 @@ LauncherDialog::createServerControls()
     //label->setMinimumSize( 80,  this->fontMetrics().height() + 12 );
     //label->setMaximumSize( 100, this->fontMetrics().height() + 12 );
 
-    QString sserver_path = AppConfig::instance().serverPath();
+    QString sserver_path = Options::instance().serverPath();
 
     M_rcssserver_command = new QLineEdit( sserver_path, group_box );
     M_rcssserver_command->setMinimumSize( 160, this->fontMetrics().height() + 12 );
@@ -356,11 +356,14 @@ LauncherDialog::startLeftTeam()
     updateComboBox( M_left_team_commands );
 
     std::string command = M_left_team_commands->currentText();
-
     std::cout << "start left team [" << command << "]" << std::endl;
-
     command += " &";
-    std::system( command.c_str() );
+
+    int r = std::system( command.c_str() );
+    if ( r == -1 )
+    {
+        std::cerr << "could not start the left team." << std::endl;
+    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -373,11 +376,16 @@ LauncherDialog::startRightTeam()
     updateComboBox( M_right_team_commands );
 
     std::string command = M_right_team_commands->currentText();
-
     std::cout << "start right team [" << command << "]" << std::endl;
-
     command += " &";
-    std::system( command.c_str() );
+
+    int r = std::system( command.c_str() );
+
+    if ( r == -1 )
+    {
+        std::cerr << "could not start the right team." << std::endl;
+        return;
+    }
 
     this->hide();
 }
