@@ -889,6 +889,10 @@ MainWindow::createFieldCanvas()
                  || mode == rcsc::PM_CornerKick_Right
                  || mode == rcsc::PM_GoalKick_Left
                  || mode == rcsc::PM_GoalKick_Right
+                 || mode == rcsc::PM_Foul_Charge_Left
+                 || mode == rcsc::PM_Foul_Charge_Right
+                 || mode == rcsc::PM_Foul_Push_Left
+                 || mode == rcsc::PM_Foul_Push_Right
                  || mode == rcsc::PM_Back_Pass_Left
                  || mode == rcsc::PM_Back_Pass_Right
                  || mode == rcsc::PM_GoalKick_Right
@@ -905,26 +909,51 @@ MainWindow::createFieldCanvas()
     {
         QPopupMenu * menu = new QPopupMenu( M_field_canvas );
         M_kick_off_act->addTo( menu );
+        //
         menu->insertSeparator();
+        //
         menu->insertItem( tr( "Drop Ball" ),
                           M_field_canvas, SLOT( dropBall() ) );
+        {
+            QAction * act = new QAction( tr( "Drop Ball There" ),
+                                         QKeySequence(),
+                                         this );
+            act->setStatusTip( tr( "Drop ball at the current ball position." ) );
+            connect( act, SIGNAL( activated() ),
+                     this, SLOT( dropBallThere() ) );
+            act->addTo( menu );
+        }
         menu->insertItem( tr( "Free Kick Left" ),
                           M_field_canvas, SLOT( freeKickLeft() ) );
         menu->insertItem( tr( "Free Kick Right" ),
                           M_field_canvas, SLOT( freeKickRight() ) );
+        //
         menu->insertSeparator();
-
-        QAction * drop_ball_there = new QAction( tr( "Drop Ball There" ),
-                                                 QKeySequence(),
-                                                 this );
-        drop_ball_there->setStatusTip( tr( "Drop ball at the current ball position." ) );
-        connect( drop_ball_there, SIGNAL( activated() ),
-                 this, SLOT( dropBallThere() ) );
-        drop_ball_there->addTo( menu );
-
-        menu->insertSeparator();
+        //
         M_playmode_change_act_group->addTo( menu );
+        //
+        menu->insertSeparator();
+        {
+            QAction * act = new QAction( tr( "Yellow Card" ),
+                                         QKeySequence(),
+                                         this );
+            act->setStatusTip( tr( "Call yellow card to the selected player." ) );
+            connect( act, SIGNAL( activated() ),
+                     this, SLOT( yellowCard() ) );
+            act->addTo( menu );
+        }
+        {
+            QAction * act = new QAction( tr( "Red Card" ),
+                                         QKeySequence(),
+                                         this );
+            act->setStatusTip( tr( "Call red card to the selected player." ) );
+            connect( act, SIGNAL( activated() ),
+                     this, SLOT( redCard() ) );
+            act->addTo( menu );
+        }
+        //
 
+        //
         M_field_canvas->setMonitorMenu( menu );
     }
 }
@@ -1288,7 +1317,7 @@ MainWindow::openRCG( const QString & file_path )
 
     if ( M_view_config_dialog )
     {
-        M_view_config_dialog->unzoom();
+        M_view_config_dialog->fitToScreen();
     }
 
     if ( ! Options::instance().anonymousMode() )
@@ -1614,7 +1643,7 @@ MainWindow::connectMonitorTo( const char * hostname )
 
     if ( M_view_config_dialog )
     {
-        M_view_config_dialog->unzoom();
+        M_view_config_dialog->fitToScreen();
     }
 
     if ( M_toggle_debug_server_mode_act->isOn() )
@@ -2551,12 +2580,12 @@ MainWindow::moveObjects()
     TrainerData & data = M_main_data.getTrainerData();
 
     // ball
-    if ( data.ballPos().valid() )
+    if ( data.ballPos().isValid() )
     {
         if ( rcsc::ServerParam::i().coachMode()
              || rcsc::ServerParam::i().coachWithRefereeMode() )
         {
-            if ( data.ballVel().valid() )
+            if ( data.ballVel().isValid() )
             {
                 M_monitor_client->sendTrainerMoveBall( data.ballPos().x,
                                                        data.ballPos().y,
@@ -2594,7 +2623,7 @@ MainWindow::moveObjects()
     {
         rcsc::Vector2D pos = data.playerMovePos( rcsc::LEFT, unum );
         rcsc::AngleDeg body = data.playerBody( rcsc::LEFT, unum );
-        if ( pos.valid() )
+        if ( pos.isValid() )
         {
             M_monitor_client->sendMovePlayer( rcsc::LEFT, unum,
                                               pos.x, pos.y,
@@ -2607,7 +2636,7 @@ MainWindow::moveObjects()
     {
         rcsc::Vector2D pos = data.playerMovePos( rcsc::RIGHT, unum );
         rcsc::AngleDeg body = data.playerBody( rcsc::RIGHT, unum );
-        if ( pos.valid() )
+        if ( pos.isValid() )
         {
             M_monitor_client->sendMovePlayer( rcsc::RIGHT, unum,
                                               pos.x, pos.y,
@@ -2617,6 +2646,64 @@ MainWindow::moveObjects()
 
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+MainWindow::yellowCard()
+{
+    if ( ! M_monitor_client
+         || ! M_monitor_client->isConnected() )
+    {
+        return;
+    }
+
+    int unum = Options::instance().selectedNumber();
+    if ( unum == 0 )
+    {
+        return;
+    }
+
+    rcsc::SideID side = rcsc::LEFT;
+    if ( unum < 0 )
+    {
+        unum = -unum;
+        side = rcsc::RIGHT;
+    }
+
+    M_monitor_client->sendCard( side, unum, rcsc::YELLOW );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+MainWindow::redCard()
+{
+    if ( ! M_monitor_client
+         || ! M_monitor_client->isConnected() )
+    {
+        return;
+    }
+
+    int unum = Options::instance().selectedNumber();
+    if ( unum == 0 )
+    {
+        return;
+    }
+
+    rcsc::SideID side = rcsc::LEFT;
+    if ( unum < 0 )
+    {
+        unum = -unum;
+        side = rcsc::RIGHT;
+    }
+
+    M_monitor_client->sendCard( side, unum, rcsc::RED );
+}
+
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -2709,6 +2796,22 @@ MainWindow::changePlayMode( int mode,
               ? + rcsc::ServerParam::i().goalAreaHalfWidth()
               : - rcsc::ServerParam::i().goalAreaHalfWidth() );
 
+        M_monitor_client->sendTrainerMoveBall( x, y, 0.0, 0.0 );
+        M_monitor_client->sendChangeMode( pmode );
+        break;
+    case rcsc::PM_Foul_Charge_Left:
+        M_monitor_client->sendTrainerMoveBall( x, y, 0.0, 0.0 );
+        M_monitor_client->sendChangeMode( pmode );
+        break;
+    case rcsc::PM_Foul_Charge_Right:
+        M_monitor_client->sendTrainerMoveBall( x, y, 0.0, 0.0 );
+        M_monitor_client->sendChangeMode( pmode );
+        break;
+    case rcsc::PM_Foul_Push_Left:
+        M_monitor_client->sendTrainerMoveBall( x, y, 0.0, 0.0 );
+        M_monitor_client->sendChangeMode( pmode );
+        break;
+    case rcsc::PM_Foul_Push_Right:
         M_monitor_client->sendTrainerMoveBall( x, y, 0.0, 0.0 );
         M_monitor_client->sendChangeMode( pmode );
         break;
