@@ -251,6 +251,9 @@ public slots:
     void moveDraggedPlayer( const QPoint & point );
     void moveObjects();
 
+    void yellowCard();
+    void redCard();
+
 signals:
 
     void viewUpdated();
