@@ -996,7 +996,7 @@ MonitorMoveDialog::sendCommand()
             if ( reverse )
             {
                 pos *= -1.0;
-                if ( vel.valid() ) vel *= -1.0;
+                if ( vel.isValid() ) vel *= -1.0;
             }
             M_trainer_data.setBall( pos, vel );
         }
