@@ -94,7 +94,7 @@ OffsideLinePainter::draw( QPainter & painter )
         offside_l = std::min( offside_l, ball_x );
         offside_l = std::max( offside_l, - rcsc::ServerParam::i().pitchHalfLength() );
     }
-    const int offside_line_l = opt.screenX( offside_l );
+    const int offside_line_l = opt.screenXInt( offside_l );
 
     double offside_r = 0.0;
     {
@@ -121,10 +121,10 @@ OffsideLinePainter::draw( QPainter & painter )
         offside_r = std::max( offside_r, ball_x );
         offside_r = std::min( offside_r, rcsc::ServerParam::i().pitchHalfLength() );
     }
-    const int offside_line_r = opt.screenX( offside_r );
+    const int offside_line_r = opt.screenXInt( offside_r );
 
-    const int top_y = opt.screenY( rcsc::ServerParam::i().pitchHalfWidth() );
-    const int bottom_y = opt.screenY( - rcsc::ServerParam::i().pitchHalfWidth() );
+    const int top_y = opt.screenYInt( rcsc::ServerParam::i().pitchHalfWidth() );
+    const int bottom_y = opt.screenYInt( - rcsc::ServerParam::i().pitchHalfWidth() );
 
     const DrawConfig & dconf = DrawConfig::instance();
 
