@@ -88,9 +88,8 @@ PlayerControlPainter::draw( QPainter & painter )
     const Player & player = view->players()[ ( number > 0
                                                ? number - 1
                                                : -number - 1 + 11 ) ];
-    const rcsc::ServerParam & sparam = rcsc::ServerParam::i();
-    const rcsc::PlayerType & ptype
-        = M_main_data.viewHolder().playerType( player.type() );
+    const rcsc::ServerParam & SP = rcsc::ServerParam::i();
+    const rcsc::PlayerType & ptype = M_main_data.viewHolder().playerType( player.type() );
 
     rcsc::Vector2D ppos( player.x(), player.y() );
     rcsc::Vector2D bpos( ball.x(), ball.y() );
@@ -104,24 +103,25 @@ PlayerControlPainter::draw( QPainter & painter )
     if ( ball_dist < ptype.kickableArea() )
     {
         double max_kick_accel
-            = sparam.maxPower()
-            * sparam.kickPowerRate()
+            = SP.maxPower()
+            //* SP.kickPowerRate()
+            * ptype.kickPowerRate()
             * ( 1.0 - 0.25 * player_to_ball.th().abs() / 180.0
                 - 0.25
-                * ( ball_dist - ptype.playerSize() - sparam.ballSize() )
+                * ( ball_dist - ptype.playerSize() - SP.ballSize() )
                 / ptype.kickableMargin() );
 
-        if ( max_kick_accel > sparam.ballAccelMax() )
+        if ( max_kick_accel > SP.ballAccelMax() )
         {
-            max_kick_accel = sparam.ballAccelMax();
+            max_kick_accel = SP.ballAccelMax();
         }
 
-        QPoint bpos_screen( opt.screenX( bpos.x ),
-                            opt.screenY( bpos.y ) );
-        QPoint bnext_screen( opt.screenX( bnext.x ),
-                             opt.screenY( bnext.y ) );
-        int max_speed_screen = opt.scale( sparam.ballSpeedMax() );
-        int max_kick_accel_screen = opt.scale( max_kick_accel );
+        QPoint bpos_screen( opt.screenXInt( bpos.x ),
+                            opt.screenYInt( bpos.y ) );
+        QPoint bnext_screen( opt.screenXInt( bnext.x ),
+                             opt.screenYInt( bnext.y ) );
+        int max_speed_screen = opt.scaleInt( SP.ballSpeedMax() );
+        int max_kick_accel_screen = opt.scaleInt( max_kick_accel );
 
         painter.setPen( dconf.kickAccelPen() );
         painter.setBrush( dconf.transparentBrush() );
@@ -129,7 +129,7 @@ PlayerControlPainter::draw( QPainter & painter )
         // draw no noise next ball move only by inertia
         painter.drawLine( bpos_screen, bnext_screen );
 
-        rcsc::Circle2D max_speed_circle( bpos, sparam.ballSpeedMax() );
+        rcsc::Circle2D max_speed_circle( bpos, SP.ballSpeedMax() );
         rcsc::Circle2D max_accel_circle( bnext, max_kick_accel );
         rcsc::Vector2D intersection_1, intersection_2;
 
