@@ -72,8 +72,8 @@ PlayerPainter::Param::Param( const Player & player,
 {
     const Options & opt = Options::instance();
 
-    x_ = opt.screenX( player.x() );
-    y_ = opt.screenY( player.y() );
+    x_ = opt.screenXInt( player.x() );
+    y_ = opt.screenYInt( player.y() );
 
     if ( opt.reverseSide() )
     {
@@ -81,8 +81,8 @@ PlayerPainter::Param::Param( const Player & player,
         head_ += 180.0;
     }
 
-    body_radius_ = opt.scale( ptype.playerSize() );
-    kick_radius_ = opt.scale( ptype.kickableArea() );
+    body_radius_ = opt.scaleInt( ptype.playerSize() );
+    kick_radius_ = opt.scaleInt( ptype.kickableArea() );
 
     if ( body_radius_ < 1 ) body_radius_ = 1;
     if ( kick_radius_ < 5 ) kick_radius_ = 5;
@@ -91,7 +91,7 @@ PlayerPainter::Param::Param( const Player & player,
     {
         if ( opt.playerSize() > 0.0 )
         {
-            draw_radius_ = opt.scale( Options::instance().playerSize() );
+            draw_radius_ = opt.scaleInt( Options::instance().playerSize() );
             if ( draw_radius_ < 1 ) draw_radius_ = 1;
         }
         else
@@ -199,7 +199,9 @@ PlayerPainter::drawAll( QPainter & painter,
         }
     }
 
-    if ( opt.showCatchableArea() )
+    if ( player.isGoalie()
+         && opt.showCatchableArea() )
+
     {
         drawCatchableArea( painter, param );
     }
@@ -309,13 +311,17 @@ PlayerPainter::drawBody( QPainter & painter,
         painter.setPen( dconf.tacklePen() );
         painter.setBrush( dconf.tackleFaultBrush() );
     }
+    if ( param.player_.isFoulCharged() )
+    {
+        painter.setBrush( dconf.foulChargedBrush() );
+    }
     if ( param.player_.isCollidedBall() )
     {
-        painter.setBrush( dconf.collideBallBrush() );
+        painter.setBrush( dconf.ballCollisionBrush() );
     }
     if ( param.player_.isCollidedPlayer() )
     {
-        painter.setBrush( dconf.collidePlayerBrush() );
+        painter.setBrush( dconf.playerCollisionBrush() );
     }
 
     painter.drawEllipse( param.x_ - param.draw_radius_ ,
@@ -442,12 +448,12 @@ PlayerPainter::drawEdge( QPainter & painter,
     // body direction line
     if ( ! opt.showBodyShadow() )
     {
-        int end_x = opt.absScreenX( param.player_.x() * opt.reverseValue()
-                                    + param.player_type_.kickableArea()
-                                    * std::cos( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
-        int end_y = opt.absScreenY( param.player_.y() * opt.reverseValue()
-                                    + param.player_type_.kickableArea()
-                                    * std::sin( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
+        int end_x = opt.absScreenXInt( param.player_.x() * opt.reverseValue()
+                                       + param.player_type_.kickableArea()
+                                       * std::cos( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
+        int end_y = opt.absScreenYInt( param.player_.y() * opt.reverseValue()
+                                       + param.player_type_.kickableArea()
+                                       * std::sin( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
         // same pen & brush
         painter.drawLine( param.x_, param.y_, end_x, end_y );
     }
@@ -478,7 +484,6 @@ PlayerPainter::drawEdge( QPainter & painter,
     }
 }
 
-
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -501,8 +506,8 @@ PlayerPainter::drawFuture( QPainter & painter,
 
     const int last = opt.playerFutureCycle();
 
-    QPoint first_point( opt.absScreenX( ppos.x ),
-                        opt.absScreenY( ppos.y ) ) ;
+    QPoint first_point( opt.absScreenXInt( ppos.x ),
+                        opt.absScreenYInt( ppos.y ) ) ;
     QPoint last_point = first_point;
 
     // draw kickable area edge
@@ -528,8 +533,8 @@ PlayerPainter::drawFuture( QPainter & painter,
         ppos += pvel;
         pvel *= param.player_type_.playerDecay();
 
-        QPoint pt( opt.absScreenX( ppos.x ),
-                   opt.absScreenY( ppos.y ) );
+        QPoint pt( opt.absScreenXInt( ppos.x ),
+                   opt.absScreenYInt( ppos.y ) );
         if ( std::abs( last_point.x() - pt.x() ) < 1
              && std::abs( last_point.y() - pt.y() ) < 1 )
         {
@@ -567,7 +572,7 @@ PlayerPainter::drawViewArea( QPainter & painter,
     const double view_width = param.player_.viewWidth();
 
     const double visible_dist = rcsc::ServerParam::i().visibleDistance();
-    const int visible_radius = opt.scale( visible_dist );
+    const int visible_radius = opt.scaleInt( visible_dist );
     const double view_start_angle = -param.head_ - view_width * 0.5;
 
     int view_start_angle_int
@@ -578,9 +583,9 @@ PlayerPainter::drawViewArea( QPainter & painter,
     if ( opt.isSelectedPlayer( param.player_.side(),
                                param.player_.unum() ) )
     {
-        const int UNUM_FAR = opt.scale( 20.0 );
-        const int TEAM_FAR = opt.scale( 40.0 );
-        const int TEAM_TOOFAR = opt.scale( 60.0 );
+        const int UNUM_FAR = opt.scaleInt( 20.0 );
+        const int TEAM_FAR = opt.scaleInt( 40.0 );
+        const int TEAM_TOOFAR = opt.scaleInt( 60.0 );
 
         //painter.setRasterOp( Qt::OrROP );
         painter.setPen( dconf.viewAreaPen() );
@@ -620,16 +625,16 @@ PlayerPainter::drawViewArea( QPainter & painter,
         const double view_end_angle_real = ( param.head_ + view_width * 0.5 ) * rcsc::AngleDeg::DEG2RAD;
         // left side view cone end point x
         int lx = param.x_
-            + opt.scale( 60.0 * std::cos( view_start_angle_real ) );
+            + opt.scaleInt( 60.0 * std::cos( view_start_angle_real ) );
         // left side view cone end point y
         int ly = param.y_
-            + opt.scale( 60.0  * std::sin( view_start_angle_real ) );
+            + opt.scaleInt( 60.0  * std::sin( view_start_angle_real ) );
         // right side view cone end point x
         int rx = param.x_
-            + opt.scale( 60.0 * std::cos( view_end_angle_real ) );
+            + opt.scaleInt( 60.0 * std::cos( view_end_angle_real ) );
         // right side view cone end point y
         int ry = param.y_
-            + opt.scale( 60.0 * std::sin( view_end_angle_real ) );
+            + opt.scaleInt( 60.0 * std::sin( view_end_angle_real ) );
         painter.drawLine( lx, ly, param.x_, param.y_ );
         painter.drawLine( rx, ry, param.x_, param.y_ );
 
@@ -665,10 +670,10 @@ PlayerPainter::drawViewDir( QPainter & painter,
                  ? param.player_type_.kickableArea()
                  : param.player_type_.playerSize() );
 
-    int end_x = opt.absScreenX( param.player_.x() * opt.reverseValue()
-                                + r * std::cos( param.head_ * rcsc::AngleDeg::DEG2RAD ) );
-    int end_y = opt.absScreenY( param.player_.y() * opt.reverseValue()
-                                + r * std::sin( param.head_ * rcsc::AngleDeg::DEG2RAD ) );
+    int end_x = opt.absScreenXInt( param.player_.x() * opt.reverseValue()
+                                   + r * std::cos( param.head_ * rcsc::AngleDeg::DEG2RAD ) );
+    int end_y = opt.absScreenYInt( param.player_.y() * opt.reverseValue()
+                                   + r * std::sin( param.head_ * rcsc::AngleDeg::DEG2RAD ) );
 
     painter.setPen( Qt::black );
     painter.setBrush( dconf.transparentBrush() );
@@ -685,26 +690,81 @@ PlayerPainter::drawCatchableArea( QPainter & painter,
 {
     const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
+    const rcsc::ServerParam & SP = rcsc::ServerParam::i();
 
-    if ( param.player_.isGoalie() )
+    rcsc::SideID side = param.player_.side();
+    if ( opt.reverseSide() )
     {
-        rcsc::SideID side = param.player_.side();
-        if ( opt.reverseSide() )
-        {
-            side = static_cast< rcsc::SideID >( -1 * side );
-        }
-
-        int catchable = opt.scale( rcsc::ServerParam::i().catchableArea() );
-        painter.setPen( ( side == rcsc::LEFT )
-                        ? dconf.leftGoaliePen()
-                        : dconf.rightGoaliePen() );
-        painter.setBrush( dconf.transparentBrush() );
-
-        painter.drawEllipse( param.x_ - catchable,
-                             param.y_ - catchable,
-                             catchable * 2,
-                             catchable * 2 );
+        side = static_cast< rcsc::SideID >( -1 * side );
     }
+
+    int catchable = opt.scaleInt( SP.catchableArea() );
+    painter.setPen( ( side == rcsc::LEFT )
+                    ? dconf.leftGoaliePen()
+                    : dconf.rightGoaliePen() );
+    painter.setBrush( dconf.transparentBrush() );;
+
+    painter.drawEllipse( param.x_ - catchable,
+                         param.y_ - catchable,
+                         catchable * 2,
+                         catchable * 2 );
+
+    double delta = SP.catchAreaLength() * std::max( 0.0, param.player_type_.catchAreaLengthStretch() - 1.0 );
+    double max_l = SP.catchAreaLength() + delta;
+    double min_l = SP.catchAreaLength() - delta;
+    double max_area = std::sqrt( std::pow( SP.catchAreaWidth() * 0.5, 2.0 )
+                                 + std::pow( max_l, 2.0 ) );
+    double min_area = std::sqrt( std::pow( SP.catchAreaWidth() * 0.5, 2.0 )
+                                 + std::pow( min_l, 2.0 ) );
+    int max_r = opt.scaleInt( max_area );
+    if ( max_r > catchable )
+    {
+        painter.setPen( ( side == rcsc::LEFT )
+                        ? dconf.leftGoalieStretchPen()
+                        : dconf.rightGoalieStretchPen() );
+        painter.drawEllipse( param.x_ - max_r,
+                             param.y_ - max_r,
+                             max_r * 2,
+                             max_r * 2 );
+        int min_r = opt.scaleInt( min_area );
+        painter.drawEllipse( param.x_ - min_r,
+                             param.y_ - min_r,
+                             min_r * 2,
+                             min_r * 2 );
+    }
+
+    //
+    // catch probability
+    //
+#if 0
+    const double ball_dist = std::sqrt( std::pow( param.player_.x() - param.ball_.x(), 2 )
+                                        + std::pow( param.player_.y() - param.ball_.y(), 2 ) );
+    double catch_prob = SP.catchProbability();
+    if ( ball_dist > stretch_area )
+    {
+        // catch_prob = 0.0;
+        return;
+    }
+
+    if ( ball_dist > SP.catchableArea() )
+    {
+        double x = ball_dist * ( stretch_l / stretch_area );
+        catch_prob
+            = SP.catchProbability()
+            - SP.catchProbability() * ( ( x - SP.catchAreaLength() )
+                                        / ( stretch_l - SP.catchAreaLength() ) );
+    }
+
+    int text_radius = std::min( 40, param.draw_radius_ );
+
+    painter.setPen( ( param.player_.side() == rcsc::LEFT )
+                    ? dconf.rightGoaliePen()
+                    : dconf.leftGoaliePen() );
+    painter.setFont( dconf.playerFont() );
+    painter.drawText( param.x_ + text_radius,
+                      param.y_ + 4 + painter.fontMetrics().ascent() * 2,
+                      QString( "Catch=%1" ).arg( catch_prob, 0, 'g', 3 ) );
+#endif
 }
 
 /*-------------------------------------------------------------------*/
@@ -735,12 +795,22 @@ PlayerPainter::drawTackleArea( QPainter & painter,
     // draw tackle area & probability
     double tackle_dist = ( player_to_ball.x > 0.0
                            ? SP.tackleDist()
-                           : SP.tackleBackDist() ); //: 0.0 );
+                           : SP.tackleBackDist() );
+    if ( tackle_dist < 1.0e-5 )
+    {
+        return;
+    }
+
     double tackle_prob = ( std::pow( player_to_ball.absX() / tackle_dist,
                                      SP.tackleExponent() )
                            + std::pow( player_to_ball.absY() / SP.tackleWidth(), // 1.25,
                                        SP.tackleExponent() ) );
-    if ( tackle_prob < 1.0 )
+    double foul_prob = ( std::pow( player_to_ball.absX() / tackle_dist,
+                                   SP.foulExponent() )
+                         + std::pow( player_to_ball.absY() / SP.tackleWidth(), // 1.25,
+                                     SP.foulExponent() ) );
+    if ( tackle_prob < 1.0
+         || foul_prob < 1.0 )
     {
         rcsc::AngleDeg body_angle = param.body_;
         rcsc::AngleDeg body_angle_side = body_angle + 90.0;
@@ -754,14 +824,14 @@ PlayerPainter::drawTackleArea( QPainter & painter,
         double right_y = SP.tackleWidth() * body_angle_side.sin(); // 1.25 * body_angle_side.sin();
 
         QPointArray pts( 5 );
-        pts[0].setX( opt.absScreenX( ppos.x + forward_x + right_x ) );
-        pts[0].setY( opt.absScreenY( ppos.y + forward_y + right_y ) );
-        pts[1].setX( opt.absScreenX( ppos.x + forward_x - right_x ) );
-        pts[1].setY( opt.absScreenY( ppos.y + forward_y - right_y ) );
-        pts[2].setX( opt.absScreenX( ppos.x + back_x - right_x ) );
-        pts[2].setY( opt.absScreenY( ppos.y + back_y - right_y ) );
-        pts[3].setX( opt.absScreenX( ppos.x + back_x + right_x ) );
-        pts[3].setY( opt.absScreenY( ppos.y + back_y + right_y ) );
+        pts[0].setX( opt.absScreenXInt( ppos.x + forward_x + right_x ) );
+        pts[0].setY( opt.absScreenYInt( ppos.y + forward_y + right_y ) );
+        pts[1].setX( opt.absScreenXInt( ppos.x + forward_x - right_x ) );
+        pts[1].setY( opt.absScreenYInt( ppos.y + forward_y - right_y ) );
+        pts[2].setX( opt.absScreenXInt( ppos.x + back_x - right_x ) );
+        pts[2].setY( opt.absScreenYInt( ppos.y + back_y - right_y ) );
+        pts[3].setX( opt.absScreenXInt( ppos.x + back_x + right_x ) );
+        pts[3].setY( opt.absScreenYInt( ppos.y + back_y + right_y ) );
         pts[4] = pts[0];
 
         painter.setPen( dconf.tackleAreaPen() );
@@ -779,9 +849,32 @@ PlayerPainter::drawTackleArea( QPainter & painter,
         painter.setFont( dconf.playerFont() );
         painter.setPen( dconf.tacklePen() );
 
-        painter.drawText( param.x_ + text_radius,
-                          param.y_ + 4 + painter.fontMetrics().ascent(),
-                          QString( "TackleProb=%1" ).arg( 1.0 - tackle_prob, 0, 'g', 3 ) );
+        if ( tackle_prob < 1.0
+             && foul_prob < 1.0 )
+        {
+            painter.drawText( param.x_ + text_radius,
+                              param.y_ + 4 + painter.fontMetrics().ascent(),
+                              QString( "T=%1,F=%2" )
+                              .arg( 1.0 - tackle_prob, 0, 'g', 3 )
+                              .arg( 1.0 - foul_prob, 0, 'g', 3 )
+                              );
+        }
+        else if ( tackle_prob < 1.0 )
+        {
+            painter.drawText( param.x_ + text_radius,
+                              param.y_ + 4 + painter.fontMetrics().ascent(),
+                              QString( "Tackle=%1" )
+                              .arg( 1.0 - tackle_prob, 0, 'g', 3 )
+                              );
+        }
+        else if ( foul_prob < 1.0 )
+        {
+            painter.drawText( param.x_ + text_radius,
+                              param.y_ + 4 + painter.fontMetrics().ascent(),
+                              QString( "Foul=%1" )
+                              .arg( 1.0 - foul_prob, 0, 'g', 3 )
+                              );
+        }
     }
 }
 
@@ -826,12 +919,12 @@ PlayerPainter::drawKickAccelArea( QPainter & painter,
             max_kick_accel = SP.ballAccelMax();
         }
 
-        QPoint bpos_screen( opt.screenX( bpos.x ),
-                            opt.screenY( bpos.y ) );
-        QPoint bnext_screen( opt.screenX( bnext.x ),
-                             opt.screenY( bnext.y ) );
-        int max_speed_screen = opt.scale( SP.ballSpeedMax() );
-        int max_kick_accel_screen = opt.scale( max_kick_accel );
+        QPoint bpos_screen( opt.screenXInt( bpos.x ),
+                            opt.screenYInt( bpos.y ) );
+        QPoint bnext_screen( opt.screenXInt( bnext.x ),
+                             opt.screenYInt( bnext.y ) );
+        int max_speed_screen = opt.scaleInt( SP.ballSpeedMax() );
+        int max_kick_accel_screen = opt.scaleInt( max_kick_accel );
 
         painter.setPen( dconf.kickAccelPen() );
         painter.setBrush( dconf.transparentBrush() );
@@ -973,8 +1066,8 @@ PlayerPainter::drawPointto( QPainter & painter,
         return;
     }
 
-    int px = Options::instance().screenX( param.player_.data().point_x_ );
-    int py = Options::instance().screenY( param.player_.data().point_y_ );
+    int px = Options::instance().screenXInt( param.player_.data().point_x_ );
+    int py = Options::instance().screenYInt( param.player_.data().point_y_ );
 
     painter.setPen( DrawConfig::instance().pointtoPen() );
     painter.setBrush( DrawConfig::instance().transparentBrush() );
@@ -1035,10 +1128,10 @@ PlayerPainter::drawAttentionto( QPainter & painter,
             painter.setBrush( DrawConfig::instance().transparentBrush() );
 
 
-            int px = opt.screenX( p->x() );
-            int py = opt.screenY( p->y() );
+            int px = opt.screenXInt( p->x() );
+            int py = opt.screenYInt( p->y() );
             //int r = param.draw_radius_ + 3;
-            int r = opt.scale( 2.0 );
+            int r = opt.scaleInt( 2.0 );
             int lx = px;
             int ly = py;
 
@@ -1051,8 +1144,8 @@ PlayerPainter::drawAttentionto( QPainter & painter,
             if ( len > std::pow( 2.0, 2 ) )
             {
                 rel *= 2.0 / len;
-                lx = opt.screenX( p->x() + rel.x );
-                ly = opt.screenY( p->y() + rel.y );
+                lx = opt.screenXInt( p->x() + rel.x );
+                ly = opt.screenYInt( p->y() + rel.y );
             }
 
             painter.drawLine( param.x_, param.y_, lx, ly );
@@ -1073,8 +1166,6 @@ PlayerPainter::drawText( QPainter & painter,
     const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
-    const int text_radius = std::min( 40, param.draw_radius_ );
-
     //     painter.save();
     //     painter.translate( param.x_, param.y_ );
     //     painter.rotate( -90.0 );
@@ -1082,25 +1173,47 @@ PlayerPainter::drawText( QPainter & painter,
 
     painter.setFont( dconf.playerFont() );
 
+    const int text_radius = std::min( 40, param.draw_radius_ );
+    int card_offset = 0;
+
+    if ( opt.showCard()
+         && ( param.player_.hasRedCard()
+              || param.player_.hasYellowCard() ) )
+    {
+        QFontMetrics fm = painter.fontMetrics();
+        int x_size = std::min( std::max( 4, fm.ascent() - 2 ), 20 );
+        int y_size = std::min( std::max( 4, fm.ascent() ), 24 );
+
+        card_offset = x_size + 2;
+
+        painter.setPen( Qt::black );
+        painter.setBrush( param.player_.hasRedCard()
+                          ? Qt::red
+                          : Qt::yellow );
+        painter.drawRect( param.x_ + text_radius,
+                          param.y_ + 4 - y_size,
+                          x_size, y_size );
+    }
+
     if ( opt.showPlayerNumber()
          && opt.showPlayerType() )
     {
         painter.setPen( dconf.playerNumberFontPen() );
-        painter.drawText( param.x_ + text_radius,
+        painter.drawText( param.x_ + text_radius + card_offset,
                           param.y_ + 4,
                           QString( "%1,t%2" ).arg( param.player_.unum() ).arg( param.player_.type() ) );
     }
     else if ( opt.showPlayerNumber() )
     {
         painter.setPen( dconf.playerNumberFontPen() );
-        painter.drawText( param.x_ + text_radius,
+        painter.drawText( param.x_ + text_radius + card_offset,
                           param.y_ + 4,
                           QString::number( param.player_.unum() ) );
     }
     else if ( opt.showPlayerType() )
     {
         painter.setPen( dconf.playerNumberFontPen() );
-        painter.drawText( param.x_ + text_radius,
+        painter.drawText( param.x_ + text_radius + card_offset,
                           param.y_ + 4,
                           QString( "t%1" ).arg( param.player_.type() ) );
     }
