@@ -104,8 +104,8 @@ PlayerPainterRCSS::Param::Param( const Player & player,
 {
     const Options & opt = Options::instance();
 
-    x_ = opt.screenX( player.x() );
-    y_ = opt.screenY( player.y() );
+    x_ = opt.screenXInt( player.x() );
+    y_ = opt.screenYInt( player.y() );
 
     if ( opt.reverseSide() )
     {
@@ -113,8 +113,8 @@ PlayerPainterRCSS::Param::Param( const Player & player,
         head_ += 180.0;
     }
 
-    body_radius_ = opt.scale( ptype.playerSize() );
-    kick_radius_ = opt.scale( ptype.kickableArea() );
+    body_radius_ = opt.scaleInt( ptype.playerSize() );
+    kick_radius_ = opt.scaleInt( ptype.kickableArea() );
 
     if ( body_radius_ < 1 ) body_radius_ = 1;
     if ( kick_radius_ < 5 ) kick_radius_ = 5;
@@ -215,6 +215,7 @@ PlayerPainterRCSS::drawAll( QPainter & painter,
     //drawShadow( painter, param );
     drawEdge( painter, param );
 
+
     if ( opt.isSelectedPlayer( player.side(), player.unum() )
          && opt.playerFutureCycle() > 0
          && player.hasDelta() )
@@ -323,6 +324,10 @@ PlayerPainterRCSS::drawBody( QPainter & painter,
         painter.setPen( dconf.tacklePen() );
         painter.setBrush( dconf.tackleFaultBrush() );
     }
+    if ( param.player_.isFoulCharged() )
+    {
+        painter.setBrush( dconf.foulChargedBrush() );
+    }
     if ( param.player_.isCollidedBall() )
     {
         painter.setBrush( BALL_COLLIDE_BRUSH );
@@ -389,12 +394,12 @@ PlayerPainterRCSS::drawEdge( QPainter & painter,
 
     // body direction line
     {
-        int end_x = opt.absScreenX( param.player_.x() * opt.reverseValue()
-                                    + param.player_type_.kickableArea()
-                                    * std::cos( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
-        int end_y = opt.absScreenY( param.player_.y() * opt.reverseValue()
-                                    + param.player_type_.kickableArea()
-                                    * std::sin( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
+        int end_x = opt.absScreenXInt( param.player_.x() * opt.reverseValue()
+                                       + param.player_type_.kickableArea()
+                                       * std::cos( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
+        int end_y = opt.absScreenYInt( param.player_.y() * opt.reverseValue()
+                                       + param.player_type_.kickableArea()
+                                       * std::sin( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
 
         painter.setPen( PLAYER_PEN );
         painter.setBrush( dconf.transparentBrush() );
@@ -403,7 +408,6 @@ PlayerPainterRCSS::drawEdge( QPainter & painter,
     }
 }
 
-
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -426,8 +430,8 @@ PlayerPainterRCSS::drawFuture( QPainter & painter,
 
     const int last = opt.playerFutureCycle();
 
-    QPoint first_point( opt.absScreenX( ppos.x ),
-                        opt.absScreenY( ppos.y ) ) ;
+    QPoint first_point( opt.absScreenXInt( ppos.x ),
+                        opt.absScreenYInt( ppos.y ) ) ;
     QPoint last_point = first_point;
 
     // draw kickable area edge
@@ -453,8 +457,8 @@ PlayerPainterRCSS::drawFuture( QPainter & painter,
         ppos += pvel;
         pvel *= param.player_type_.playerDecay();
 
-        QPoint pt( opt.absScreenX( ppos.x ),
-                   opt.absScreenY( ppos.y ) );
+        QPoint pt( opt.absScreenXInt( ppos.x ),
+                   opt.absScreenYInt( ppos.y ) );
         if ( std::abs( last_point.x() - pt.x() ) < 1
              && std::abs( last_point.y() - pt.y() ) < 1 )
         {
@@ -490,9 +494,9 @@ PlayerPainterRCSS::drawViewAreaBackground( QPainter & painter,
     const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
-    const int x = opt.screenX( player.x() );
-    const int y = opt.screenY( player.y() );
-    const int visible_radius = opt.scale( rcsc::ServerParam::i().visibleDistance() );
+    const int x = opt.screenXInt( player.x() );
+    const int y = opt.screenYInt( player.y() );
+    const int visible_radius = opt.scaleInt( rcsc::ServerParam::i().visibleDistance() );
     const int view_start_angle
         = static_cast< int >
         ( rint( ( - player.head() - player.viewWidth()*0.5
@@ -500,8 +504,8 @@ PlayerPainterRCSS::drawViewAreaBackground( QPainter & painter,
     const int view_width
         = static_cast< int >( rint( player.viewWidth() * 16 ) );
 
-    const int TEAM_FAR = opt.scale( 40.0 );
-    const int TEAM_TOOFAR = opt.scale( 60.0 );
+    const int TEAM_FAR = opt.scaleInt( 40.0 );
+    const int TEAM_TOOFAR = opt.scaleInt( 60.0 );
 
     painter.setRasterOp( Qt::OrROP );
     painter.setPen( dconf.transparentPen() );
@@ -547,12 +551,12 @@ PlayerPainterRCSS::drawViewArea( QPainter & painter,
     painter.setPen( NECK_PEN );
     painter.setBrush( dconf.transparentBrush() );
 
-    int end_x = opt.absScreenX( param.player_.x() * opt.reverseValue()
-                                + param.player_type_.kickableArea()
-                                * std::cos( param.head_ * rcsc::AngleDeg::DEG2RAD ) );
-    int end_y = opt.absScreenY( param.player_.y() * opt.reverseValue()
-                                + param.player_type_.kickableArea()
-                                * std::sin( param.head_ * rcsc::AngleDeg::DEG2RAD ) );;
+    int end_x = opt.absScreenXInt( param.player_.x() * opt.reverseValue()
+                                   + param.player_type_.kickableArea()
+                                   * std::cos( param.head_ * rcsc::AngleDeg::DEG2RAD ) );
+    int end_y = opt.absScreenYInt( param.player_.y() * opt.reverseValue()
+                                   + param.player_type_.kickableArea()
+                                   * std::sin( param.head_ * rcsc::AngleDeg::DEG2RAD ) );;
 
     painter.drawLine( param.x_, param.y_, end_x, end_y );
 }
@@ -567,6 +571,7 @@ PlayerPainterRCSS::drawCatchableArea( QPainter & painter,
 {
     const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
+    const rcsc::ServerParam & SP = rcsc::ServerParam::i();
 
     rcsc::SideID side = param.player_.side();
     if ( opt.reverseSide() )
@@ -574,16 +579,73 @@ PlayerPainterRCSS::drawCatchableArea( QPainter & painter,
         side = static_cast< rcsc::SideID >( -1 * side );
     }
 
-    int catchable = opt.scale( rcsc::ServerParam::i().catchableArea() );
+    int catchable = opt.scaleInt( SP.catchableArea() );
     painter.setPen( ( side == rcsc::LEFT )
                     ? dconf.leftGoaliePen()
                     : dconf.rightGoaliePen() );
-    painter.setBrush( dconf.transparentBrush() );
+    painter.setBrush( dconf.transparentBrush() );;
 
     painter.drawEllipse( param.x_ - catchable,
                          param.y_ - catchable,
                          catchable * 2,
                          catchable * 2 );
+
+    double delta = SP.catchAreaLength() * std::max( 0.0, param.player_type_.catchAreaLengthStretch() - 1.0 );
+    double max_l = SP.catchAreaLength() + delta;
+    double min_l = SP.catchAreaLength() - delta;
+    double max_area = std::sqrt( std::pow( SP.catchAreaWidth() * 0.5, 2.0 )
+                                 + std::pow( max_l, 2.0 ) );
+    double min_area = std::sqrt( std::pow( SP.catchAreaWidth() * 0.5, 2.0 )
+                                 + std::pow( min_l, 2.0 ) );
+    int max_r = opt.scaleInt( max_area );
+    if ( max_r > catchable )
+    {
+        painter.setPen( ( side == rcsc::LEFT )
+                        ? dconf.leftGoalieStretchPen()
+                        : dconf.rightGoalieStretchPen() );
+        painter.drawEllipse( param.x_ - max_r,
+                             param.y_ - max_r,
+                             max_r * 2,
+                             max_r * 2 );
+        int min_r = opt.scaleInt( min_area );
+        painter.drawEllipse( param.x_ - min_r,
+                             param.y_ - min_r,
+                             min_r * 2,
+                             min_r * 2 );
+    }
+
+    //
+    // catch probability
+    //
+#if 0
+    const double ball_dist = std::sqrt( std::pow( param.player_.x() - param.ball_.x(), 2 )
+                                        + std::pow( param.player_.y() - param.ball_.y(), 2 ) );
+    double catch_prob = SP.catchProbability();
+    if ( ball_dist > stretch_area )
+    {
+        // catch_prob = 0.0;
+        return;
+    }
+
+    if ( ball_dist > SP.catchableArea() )
+    {
+        double x = ball_dist * ( stretch_l / stretch_area );
+        catch_prob
+            = SP.catchProbability()
+            - SP.catchProbability() * ( ( x - SP.catchAreaLength() )
+                                        / ( stretch_l - SP.catchAreaLength() ) );
+    }
+
+    int text_radius = std::min( 40, param.draw_radius_ );
+
+    painter.setPen( ( param.player_.side() == rcsc::LEFT )
+                    ? dconf.rightGoaliePen()
+                    : dconf.leftGoaliePen() );
+    painter.setFont( dconf.playerFont() );
+    painter.drawText( param.x_ + text_radius,
+                      param.y_ + 4 + painter.fontMetrics().ascent() * 2,
+                      QString( "Catch=%1" ).arg( catch_prob, 0, 'g', 3 ) );
+#endif
 }
 
 /*-------------------------------------------------------------------*/
@@ -633,14 +695,14 @@ PlayerPainterRCSS::drawTackleArea( QPainter & painter,
         double right_y = SP.tackleWidth() * body_angle_side.sin();
 
         QPointArray pts( 4 );
-        pts[0].setX( opt.absScreenX( ppos.x + forward_x + right_x ) );
-        pts[0].setY( opt.absScreenY( ppos.y + forward_y + right_y ) );
-        pts[1].setX( opt.absScreenX( ppos.x + forward_x - right_x ) );
-        pts[1].setY( opt.absScreenY( ppos.y + forward_y - right_y ) );
-        pts[2].setX( opt.absScreenX( ppos.x + back_x - right_x ) );
-        pts[2].setY( opt.absScreenY( ppos.y + back_y - right_y ) );
-        pts[3].setX( opt.absScreenX( ppos.x + back_x + right_x ) );
-        pts[3].setY( opt.absScreenY( ppos.y + back_y + right_y ) );
+        pts[0].setX( opt.absScreenXInt( ppos.x + forward_x + right_x ) );
+        pts[0].setY( opt.absScreenYInt( ppos.y + forward_y + right_y ) );
+        pts[1].setX( opt.absScreenXInt( ppos.x + forward_x - right_x ) );
+        pts[1].setY( opt.absScreenYInt( ppos.y + forward_y - right_y ) );
+        pts[2].setX( opt.absScreenXInt( ppos.x + back_x - right_x ) );
+        pts[2].setY( opt.absScreenYInt( ppos.y + back_y - right_y ) );
+        pts[3].setX( opt.absScreenXInt( ppos.x + back_x + right_x ) );
+        pts[3].setY( opt.absScreenYInt( ppos.y + back_y + right_y ) );
         pts[4] = pts[0];
 
         painter.setPen( dconf.tackleAreaPen() );
@@ -671,6 +733,19 @@ void
 PlayerPainterRCSS::drawText( QPainter & painter,
                              const PlayerPainterRCSS::Param & param ) const
 {
+    static bool first = true;
+    static QFont font( "6x13bold", 9, QFont::Bold );
+    if ( first )
+    {
+        font.setPointSize( 9 );
+        font.setBold( true );
+        //font.setStyleHint( QFont::System, QFont::PreferBitmap );
+        font.setBold( true );
+        font.setFixedPitch( true );
+        first = false;
+    }
+    painter.setFont( font );
+
     const Options & opt = Options::instance();
 
     QString str;
@@ -709,22 +784,30 @@ PlayerPainterRCSS::drawText( QPainter & painter,
         str += QString::number( param.player_.type() );
     }
 
+    const int text_radius = std::min( 40, param.draw_radius_ );
+    int card_offset = 0;
+
+    if ( opt.showCard()
+         && ( param.player_.hasRedCard()
+              || param.player_.hasYellowCard() ) )
+    {
+        QFontMetrics fm = painter.fontMetrics();
+        int x_size = std::min( std::max( 4, fm.ascent() - 2 ), 20 );
+        int y_size = std::min( std::max( 4, fm.ascent() ), 24 );
+
+        card_offset = x_size + 2;
+
+        painter.setPen( Qt::black );
+        painter.setBrush( param.player_.hasRedCard()
+                          ? Qt::red
+                          : Qt::yellow );
+        painter.drawRect( param.x_ + text_radius,
+                          param.y_ - y_size,
+                          x_size, y_size );
+    }
+
     if ( ! str.isEmpty() )
     {
-        static bool first = true;
-        static QFont font( "6x13bold", 9, QFont::Bold );
-        if ( first )
-        {
-            font.setPointSize( 9 );
-            font.setBold( true );
-            //font.setStyleHint( QFont::System, QFont::PreferBitmap );
-            font.setBold( true );
-            font.setFixedPitch( true );
-            first = false;
-        }
-
-        painter.setFont( font );
-
         rcsc::SideID side = param.player_.side();
         if ( opt.reverseSide() )
         {
@@ -734,6 +817,8 @@ PlayerPainterRCSS::drawText( QPainter & painter,
         painter.setPen( side == rcsc::LEFT
                         ? LEFT_TEAM_PEN
                         : RIGHT_TEAM_PEN );
-        painter.drawText( param.x_, param.y_, str );
+        painter.drawText( param.x_ + text_radius + card_offset,
+                          param.y_,
+                          str );
     }
 }
