@@ -64,7 +64,7 @@ TeamGraphicPainter::draw( QPainter & painter )
     const Options & opt = Options::instance();
 
     if ( opt.anonymousMode()
-         || ! opt.showTeamLogo() )
+         || ! opt.showTeamGraphic() )
     {
         return;
     }
