@@ -377,6 +377,11 @@ ViewConfigDialog::createPlayersDetailControls( QWidget * parent )
         M_attentionto_cb->setChecked( opt.showAttentionto() );
         connect( M_attentionto_cb, SIGNAL( toggled( bool ) ),
                  this, SLOT( clickShowAttentionto( bool ) ) );
+        //
+        M_card_cb = new QCheckBox( tr( "Card" ), box );
+        M_card_cb->setChecked( opt.showCard() );
+        connect( M_card_cb, SIGNAL( toggled( bool ) ),
+                 this, SLOT( clickShowCard( bool ) ) );
     }
     return group_box;
 }
@@ -416,10 +421,10 @@ ViewConfigDialog::createShowControls( QWidget * parent )
         connect( M_show_score_board_cb, SIGNAL( toggled( bool ) ),
                  this, SLOT( clickShowScoreBoard( bool ) ) );
         //
-        M_show_team_logo_cb = new QCheckBox( tr( "Team Logo" ), top_box );
-        M_show_team_logo_cb->setChecked( opt.showTeamLogo() );
-        connect( M_show_team_logo_cb, SIGNAL( toggled( bool ) ),
-                 this, SLOT( clickShowTeamLogo( bool ) ) );
+        M_show_team_graphic_cb = new QCheckBox( tr( "Team Graphic" ), top_box );
+        M_show_team_graphic_cb->setChecked( opt.showTeamGraphic() );
+        connect( M_show_team_graphic_cb, SIGNAL( toggled( bool ) ),
+                 this, SLOT( clickShowTeamGraphic( bool ) ) );
         //
         M_show_flags_cb = new QCheckBox( tr( "Flag" ), top_box );
         M_show_flags_cb->setChecked( opt.showFlags() );
@@ -922,6 +927,7 @@ ViewConfigDialog::updateAll()
     M_kick_accel_area_cb->setChecked( opt.showKickAccelArea() );
     M_pointto_cb->setChecked( opt.showPointto() );
     M_attentionto_cb->setChecked( opt.showAttentionto() );
+    M_card_cb->setChecked( opt.showCard() );
 
     M_anonymous_mode_cb->setChecked( opt.anonymousMode() );
     M_show_score_board_cb->setChecked( opt.showScoreBoard() );
@@ -1069,9 +1075,9 @@ ViewConfigDialog::zoomOut()
 
  */
 void
-ViewConfigDialog::unzoom()
+ViewConfigDialog::fitToScreen()
 {
-    Options::instance().unzoom();
+    Options::instance().fitToScreen();
     updateFieldScale();
     M_focus_fix_rb->setChecked( true );
 
@@ -1578,6 +1584,33 @@ ViewConfigDialog::toggleShowAttentionto()
     emit configured();
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+void
+ViewConfigDialog::clickShowCard( bool checked )
+{
+    if ( Options::instance().showCard() != checked )
+    {
+        Options::instance().toggleShowCard();
+        emit configured();
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+void
+ViewConfigDialog::toggleShowCard()
+{
+    Options::instance().toggleShowCard();
+    M_card_cb->setChecked( Options::instance().showCard() );
+
+    emit configured();
+}
+
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -1637,11 +1670,11 @@ ViewConfigDialog::toggleShowScoreBoard()
 
  */
 void
-ViewConfigDialog::clickShowTeamLogo( bool checked )
+ViewConfigDialog::clickShowTeamGraphic( bool checked )
 {
-    if ( Options::instance().showTeamLogo() != checked )
+    if ( Options::instance().showTeamGraphic() != checked )
     {
-        Options::instance().toggleShowTeamLogo();
+        Options::instance().toggleShowTeamGraphic();
         emit configured();
     }
 }
@@ -1651,10 +1684,10 @@ ViewConfigDialog::clickShowTeamLogo( bool checked )
 
  */
 void
-ViewConfigDialog::toggleShowTeamLogo()
+ViewConfigDialog::toggleShowTeamGraphic()
 {
-    Options::instance().toggleShowTeamLogo();
-    M_show_team_logo_cb->setChecked( Options::instance().showTeamLogo() );
+    Options::instance().toggleShowTeamGraphic();
+    M_show_team_graphic_cb->setChecked( Options::instance().showTeamGraphic() );
 
     emit configured();
 }
