@@ -85,11 +85,12 @@ private:
     QCheckBox * M_kick_accel_area_cb;
     QCheckBox * M_pointto_cb;
     QCheckBox * M_attentionto_cb;
+    QCheckBox * M_card_cb;
 
     // show/hide control
     QCheckBox * M_anonymous_mode_cb;
     QCheckBox * M_show_score_board_cb;
-    QCheckBox * M_show_team_logo_cb;
+    QCheckBox * M_show_team_graphic_cb;
     QCheckBox * M_show_ball_cb;
     QCheckBox * M_show_players_cb;
     QCheckBox * M_show_flags_cb;
@@ -205,10 +206,11 @@ private slots:
     void clickShowKickAccelArea( bool checked );
     void clickShowPointto( bool checked );
     void clickShowAttentionto( bool checked );
+    void clickShowCard( bool checked );
 
     void clickAnonymousMode( bool checked );
     void clickShowScoreBoard( bool checked );
-    void clickShowTeamLogo( bool checked );
+    void clickShowTeamGraphic( bool checked );
     void clickShowBall( bool checked );
     void clickShowPlayers( bool checked );
     void clickShowFlags( bool checked );
@@ -267,7 +269,7 @@ public slots:
     // emit configured()
     void zoomIn();
     void zoomOut();
-    void unzoom();
+    void fitToScreen();
 
     void toggleEnlarge();
     void editBallSize( const QString & text );
@@ -289,10 +291,11 @@ public slots:
     void toggleShowKickAccelArea();
     void toggleShowPointto();
     void toggleShowAttentionto();
+    void toggleShowCard();
 
     void toggleAnonymousMode();
     void toggleShowScoreBoard();
-    void toggleShowTeamLogo();
+    void toggleShowTeamGraphic();
     void toggleShowPlayers();
     void toggleShowBall();
     void toggleShowFlags();
