@@ -175,34 +175,34 @@ VoronoiDiagramPainter::drawVoronoiDiagram( QPainter & painter,
         int n = PITCH_RECT.intersection( *s, &pos1, &pos2 );
         if ( n == 2 )
         {
-            painter.drawLine( opt.absScreenX( pos1.x ),
-                              opt.absScreenY( pos1.y ),
-                              opt.absScreenX( pos2.x ),
-                              opt.absScreenY( pos2.y ) );
+            painter.drawLine( opt.absScreenXInt( pos1.x ),
+                              opt.absScreenYInt( pos1.y ),
+                              opt.absScreenXInt( pos2.x ),
+                              opt.absScreenYInt( pos2.y ) );
         }
         else if ( n == 1 )
         {
             if ( PITCH_RECT.contains( s->origin() ) )
             {
-                painter.drawLine( opt.absScreenX( s->origin().x ),
-                                  opt.absScreenY( s->origin().y ),
-                                  opt.absScreenX( pos1.x ),
-                                  opt.absScreenY( pos1.y ) );
+                painter.drawLine( opt.absScreenXInt( s->origin().x ),
+                                  opt.absScreenYInt( s->origin().y ),
+                                  opt.absScreenXInt( pos1.x ),
+                                  opt.absScreenYInt( pos1.y ) );
             }
             else
             {
-                painter.drawLine( opt.absScreenX( s->terminal().x ),
-                                  opt.absScreenY( s->terminal().y ),
-                                  opt.absScreenX( pos1.x ),
-                                  opt.absScreenY( pos1.y ) );
+                painter.drawLine( opt.absScreenXInt( s->terminal().x ),
+                                  opt.absScreenYInt( s->terminal().y ),
+                                  opt.absScreenXInt( pos1.x ),
+                                  opt.absScreenYInt( pos1.y ) );
             }
         }
         else if ( PITCH_RECT.contains( s->origin() ) )
         {
-            painter.drawLine( opt.absScreenX( s->origin().x ),
-                              opt.absScreenY( s->origin().y ),
-                              opt.absScreenX( s->terminal().x ),
-                              opt.absScreenY( s->terminal().y ) );
+            painter.drawLine( opt.absScreenXInt( s->origin().x ),
+                              opt.absScreenYInt( s->origin().y ),
+                              opt.absScreenXInt( s->terminal().x ),
+                              opt.absScreenYInt( s->terminal().y ) );
         }
     }
 
@@ -215,10 +215,10 @@ VoronoiDiagramPainter::drawVoronoiDiagram( QPainter & painter,
         rcsc::Vector2D pos;
         if ( PITCH_RECT.intersection( *r, &pos, static_cast< rcsc::Vector2D * >( 0 ) ) == 1 )
         {
-            painter.drawLine( opt.absScreenX( r->origin().x ),
-                              opt.absScreenY( r->origin().y ),
-                              opt.absScreenX( pos.x ),
-                              opt.absScreenY( pos.y ) );
+            painter.drawLine( opt.absScreenXInt( r->origin().x ),
+                              opt.absScreenYInt( r->origin().y ),
+                              opt.absScreenXInt( pos.x ),
+                              opt.absScreenYInt( pos.y ) );
         }
     }
 #endif
@@ -250,10 +250,10 @@ VoronoiDiagramPainter::drawDelaunayTriangulation( QPainter & painter,
           e != e_end;
           ++e )
     {
-        painter.drawLine( opt.absScreenX( e->second->vertex( 0 )->pos().x ),
-                          opt.absScreenY( e->second->vertex( 0 )->pos().y ),
-                          opt.absScreenX( e->second->vertex( 1 )->pos().x ),
-                          opt.absScreenY( e->second->vertex( 1 )->pos().y ) );
+        painter.drawLine( opt.absScreenXInt( e->second->vertex( 0 )->pos().x ),
+                          opt.absScreenYInt( e->second->vertex( 0 )->pos().y ),
+                          opt.absScreenXInt( e->second->vertex( 1 )->pos().x ),
+                          opt.absScreenYInt( e->second->vertex( 1 )->pos().y ) );
     }
 #else
     rcsc::Triangulation triangulation;
@@ -270,10 +270,10 @@ VoronoiDiagramPainter::drawDelaunayTriangulation( QPainter & painter,
           e != e_end;
           ++e )
     {
-        painter.drawLine( opt.absScreenX( points[e->first].x ),
-                          opt.absScreenY( points[e->first].y ),
-                          opt.absScreenX( points[e->second].x ),
-                          opt.absScreenY( points[e->second].y ) );
+        painter.drawLine( opt.absScreenXInt( points[e->first].x ),
+                          opt.absScreenYInt( points[e->first].y ),
+                          opt.absScreenXInt( points[e->second].x ),
+                          opt.absScreenYInt( points[e->second].y ) );
     }
 #endif
 }
@@ -372,7 +372,7 @@ VoronoiDiagramPainter::drawOld( QPainter & painter )
                                                             players_pos[k] );
                 rcsc::Vector2D perpend_intersect
                     = rcsc::Line2D::intersection( first_perpend, k_perpend );
-                if ( ! perpend_intersect.valid() )
+                if ( ! perpend_intersect.isValid() )
                 {
                     continue; // intersection does not exist
                 }
@@ -482,10 +482,10 @@ VoronoiDiagramPainter::drawOld( QPainter & painter )
               it != end;
               ++it )
         {
-            painter.drawLine( opt.absScreenX( (*it)->point1.x ),
-                              opt.absScreenY( (*it)->point1.y ),
-                              opt.absScreenX( (*it)->point2.x ),
-                              opt.absScreenY( (*it)->point2.y ) );
+            painter.drawLine( opt.absScreenXInt( (*it)->point1.x ),
+                              opt.absScreenYInt( (*it)->point1.y ),
+                              opt.absScreenXInt( (*it)->point2.x ),
+                              opt.absScreenYInt( (*it)->point2.y ) );
         }
     }
 
@@ -499,10 +499,10 @@ VoronoiDiagramPainter::drawOld( QPainter & painter )
               it != end;
               ++it )
         {
-            painter.drawLine( opt.absScreenX( players_pos[(*it)->player1].x ),
-                              opt.absScreenY( players_pos[(*it)->player1].y ),
-                              opt.absScreenX( players_pos[(*it)->player2].x ),
-                              opt.absScreenY( players_pos[(*it)->player2].y ) );
+            painter.drawLine( opt.absScreenXInt( players_pos[(*it)->player1].x ),
+                              opt.absScreenYInt( players_pos[(*it)->player1].y ),
+                              opt.absScreenXInt( players_pos[(*it)->player2].x ),
+                              opt.absScreenYInt( players_pos[(*it)->player2].y ) );
         }
     }
 }
