@@ -150,19 +150,25 @@ DebugMessageWindow::DebugMessageWindow( QWidget * parent,
     M_debug_text[++i] = tr( "Clear" );
     M_debug_text[++i] = tr( "Block" );
     M_debug_text[++i] = tr( "Mark" );
+    M_debug_text[++i] = tr( "Position" );
     M_debug_text[++i] = tr( "Role" );
+    M_debug_text[++i] = tr( "Plan" );
     M_debug_text[++i] = tr( "Team" );
+    M_debug_text[++i] = tr( "Comm" );
     M_debug_text[++i] = tr( "Analyzer" );
-    M_debug_text[++i] = tr( "Recursive" );
+    M_debug_text[++i] = tr( "ActChain" );
 
 
     createActions();
+    createMenus();
     createWindows();
 
     readSettings();
 
-    this->resize( 800, 600 );
+    this->resize( 800, 800 );
+#ifndef Q_WS_MAC
     this->setWindowFlags( this->windowFlags() | Qt::WindowStaysOnTopHint );
+#endif
 }
 
 /*-------------------------------------------------------------------*/
@@ -549,6 +555,49 @@ DebugMessageWindow::createActions()
     }
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+DebugMessageWindow::createMenus()
+{
+    {
+        QMenu * menu = menuBar()->addMenu( "Data" );
+
+        menu->addAction( M_open_debug_log_dir_act );
+        menu->addAction( M_clear_data_act );
+
+        menu->addAction( M_sync_act );
+        menu->addAction( M_decrement_act );
+        menu->addAction( M_increment_act );
+    }
+
+    {
+        QMenu * menu = menuBar()->addMenu( "View" );
+
+        menu->addAction( M_show_debug_view_all_act );
+        menu->addAction( M_show_debug_view_self_act );
+        menu->addAction( M_show_debug_view_ball_act );
+        menu->addAction( M_show_debug_view_teammates_act );
+        menu->addAction( M_show_debug_view_opponents_act );
+        menu->addAction( M_show_debug_view_comment_act );
+        menu->addAction( M_show_debug_view_shape_act );
+        menu->addAction( M_show_debug_view_target_act );
+        menu->addAction( M_show_debug_view_message_act );
+        menu->addAction( M_show_debug_log_objects_act );
+    }
+
+    {
+        QMenu * menu = menuBar()->addMenu( "Level" );
+        for ( int i = 0; i < 32; ++i )
+        {
+            menu->addAction( M_debug_level_act[i] );
+        }
+    }
+
+    menuBar()->hide();
+}
 
 /*-------------------------------------------------------------------*/
 /*!
@@ -570,6 +619,7 @@ DebugMessageWindow::createWindows()
     {
         M_message[i] = new QTextEdit();
         M_message[i]->setReadOnly( true );
+        M_message[i]->setWordWrapMode( QTextOption::NoWrap );
         M_tab_widget->addTab( M_message[i],
                               QString( "   %1 " ).arg( i + 1, 2 ) );
     }
@@ -664,30 +714,29 @@ DebugMessageWindow::createToolBar()
         this->addToolBar( Qt::TopToolBarArea, tbar );
     }
 
-    const int max_visible_level = 18;
     // left tool bar
     {
         QToolBar * tbar = new QToolBar( tr( "Debug Level" ), this );
         tbar->setIconSize( QSize( 16, 16 ) );
-        for ( int i = 0; i < max_visible_level; ++i )
+
+        QToolBar * hbar = new QToolBar( tr( "Debug Level Hidden" ), this );
+        hbar->setIconSize( QSize( 16, 16 ) );
+
+        for ( int i = 0; i < 32; ++i )
         {
-            tbar->addAction( M_debug_level_act[i] );
+             if ( ! M_debug_text[i].isEmpty() )
+            {
+                tbar->addAction( M_debug_level_act[i] );
+            }
+            else
+            {
+                hbar->addAction( M_debug_level_act[i] );
+            }
         }
 
         this->addToolBar( Qt::LeftToolBarArea, tbar );
-    }
-    //this->addToolBarBreak( Qt::LeftToolBarArea );
-    {
-        QToolBar * tbar = new QToolBar( tr( "Debug Level Hidden" ), this );
-        tbar->setIconSize( QSize( 16, 16 ) );
-
-        for ( int i = max_visible_level; i < 32; ++i )
-        {
-            tbar->addAction( M_debug_level_act[i] );
-        }
-
-        this->addToolBar( Qt::LeftToolBarArea, tbar );
-        tbar->hide();
+        this->addToolBar( Qt::LeftToolBarArea, hbar );
+        hbar->hide();
     }
 }
 
@@ -896,7 +945,7 @@ DebugMessageWindow::showDebugLogDirDialog()
 
     if ( ! dlg.exec() )
     {
-        std::cerr << "DebugLogDirDialog is not ok." << std::endl;
+        //std::cerr << "DebugLogDirDialog is not ok." << std::endl;
         return;
     }
 
