@@ -43,6 +43,7 @@
 #include "main_data.h"
 
 #include <rcsc/common/server_param.h>
+#include <rcsc/geom/angle_deg.h>
 
 #include <iostream>
 #include <cmath>
@@ -67,8 +68,8 @@ draw_communication( QPainter & painter,
                              ? -1.0
                              : 1.0 );
 
-    const int say_r = std::max( 3, opt.scale( 3.5 ) );
-    const int hear_r = std::max( 2, opt.scale( 3.0 ) );
+    const double say_r = std::max( 3.0, opt.scaleF( 3.5 ) );
+    const double hear_r = std::max( 2.0, opt.scaleF( 3.0 ) );
 
     //
     // draw say message sender marks
@@ -86,10 +87,10 @@ draw_communication( QPainter & painter,
         const boost::shared_ptr< DebugViewData::SelfT > self = v->self();
         if ( ! self ) continue;
 
-        const int ix = opt.screenX( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
-        const int iy = opt.screenY( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+        const double ix = opt.screenXF( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+        const double iy = opt.screenYF( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
 
-        painter.drawEllipse( ix - say_r, iy - say_r, say_r * 2, say_r * 2 );
+        painter.drawEllipse( QRectF( ix - say_r, iy - say_r, say_r * 2, say_r * 2 ) );
     }
 
     //
@@ -103,8 +104,8 @@ draw_communication( QPainter & painter,
             return;
         }
 
-        const int self_x = opt.screenX( self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
-        const int self_y = opt.screenY( self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+        const QPointF self_pos( opt.screenXF( self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse ),
+                                opt.screenYF( self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse ) );
 
         const std::vector< Player > & players = monitor_view.players();
         const std::vector< Player >::const_iterator p_end = players.end();
@@ -124,12 +125,14 @@ draw_communication( QPainter & painter,
                 if ( p->side() == self_side
                      && p->unum() == it->first )
                 {
-                    int px = opt.screenX( p->x() );
-                    int py = opt.screenY( p->y() );
+                    QPointF pos( opt.screenXF( p->x() ),
+                                 opt.screenYF( p->y() ) );
 
-                    painter.drawEllipse( px - hear_r, py - hear_r,
-                                         hear_r * 2, hear_r * 2 );
-                    painter.drawLine( px, py, self_x, self_y );
+                    painter.drawEllipse( QRectF( pos.x() - hear_r,
+                                                 pos.y() - hear_r,
+                                                 hear_r * 2,
+                                                 hear_r * 2 ) );
+                    painter.drawLine( QLineF( pos, self_pos ) );
                     break;
                 }
             }
@@ -270,12 +273,13 @@ DebugPainter::draw( QPainter & painter )
     {
         if ( opt.antiAliasing() )
         {
-            painter.setRenderHints(
-#ifdef USE_GLWIDGET
-                                   //QPainter::HighQualityAntialiasing |
+#ifdef USE_HIGH_QUALITY_ANTIALIASING
+            painter.setRenderHints( QPainter::HighQualityAntialiasing |
+                                    QPainter::Antialiasing,
+                                    false );
+#else
+            painter.setRenderHint( QPainter::Antialiasing, false );
 #endif
-                                   QPainter::Antialiasing,
-                                   false );
         }
 
         drawLines( painter, self_side, *view );
@@ -285,12 +289,13 @@ DebugPainter::draw( QPainter & painter )
 
         if ( opt.antiAliasing() )
         {
-            painter.setRenderHints(
-#ifdef USE_GLWIDGET
-                                   //QPainter::HighQualityAntialiasing |
+#ifdef USE_HIGH_QUALITY_ANTIALIASING
+            painter.setRenderHints( QPainter::HighQualityAntialiasing |
+                                    QPainter::Antialiasing,
+                                    true );
+#else
+            painter.setRenderHint( QPainter::Antialiasing, true );
 #endif
-                                   QPainter::Antialiasing,
-                                   true );
         }
     }
 
@@ -322,27 +327,27 @@ DebugPainter::drawSelf( QPainter & painter,
     const double reverse = ( self_side == rcsc::RIGHT
                              ? -1.0
                              : 1.0 );
-    const int ix = opt.screenX( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
-    const int iy = opt.screenY( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
-    const int r = std::max( 2,
-                            ( opt.enlargeMode()
-                              ? opt.scale( 1.0 )
-                              : opt.scale( 0.3 ) ) );
+    const double ix = opt.screenXF( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+    const double iy = opt.screenYF( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+    const double r = std::max( 2.0,
+                               ( opt.enlargeMode()
+                                 ? opt.scaleF( 1.0 )
+                                 : opt.scaleF( 0.3 ) ) );
 
     // draw targets
     if ( opt.showDebugViewTarget()
          &&  view.targetPoint() )
     {
-        int tx = opt.screenX( view.targetPoint()->x * reverse );
-        int ty = opt.screenY( view.targetPoint()->y * reverse );
+        double tx = opt.screenXF( view.targetPoint()->x * reverse );
+        double ty = opt.screenYF( view.targetPoint()->y * reverse );
 
         painter.setPen( dconf.debugTargetPen() );
         painter.setBrush( dconf.transparentBrush() );
 
-        painter.drawLine( tx - r, ty - r,
-                          tx + r, ty + r );
-        painter.drawLine( tx - r, ty + r,
-                          tx + r, ty - r );
+        painter.drawLine( QLineF( tx - r, ty - r,
+                                  tx + r, ty + r ) );
+        painter.drawLine( QLineF( tx - r, ty + r,
+                                  tx + r, ty - r ) );
     }
 
     // draw body half circle
@@ -353,10 +358,10 @@ DebugPainter::drawSelf( QPainter & painter,
     painter.setPen( dconf.transparentPen() );
     painter.setBrush( dconf.debugSelfBrush() );
 
-    painter.drawPie( ix - r,
-                     iy - r,
-                     r * 2,
-                     r * 2,
+    painter.drawPie( QRectF( ix - r,
+                             iy - r,
+                             r * 2,
+                             r * 2 ),
                      static_cast< int >( rint( body_start_dir * 16 ) ),
                      180 * 16 );
 
@@ -365,10 +370,25 @@ DebugPainter::drawSelf( QPainter & painter,
     painter.setPen( dconf.debugPlayerPen() );
     painter.setBrush( dconf.transparentBrush() );
 
-    painter.drawEllipse( ix - r,
-                         iy - r,
-                         r * 2,
-                         r * 2 );
+    painter.drawEllipse( QRectF( ix - r,
+                                 iy - r,
+                                 r * 2,
+                                 r * 2 ) );
+
+    // draw face angle
+    {
+        rcsc::AngleDeg face_angle = (double)self->body_ + (double)self->neck_;
+        if ( opt.reverseSide() ) face_angle += 180.0;
+
+        double face_r = 3.0;
+        double end_x = ix + opt.scaleF( face_r * face_angle.cos() );
+        double end_y = iy + opt.scaleF( face_r * face_angle.sin() );
+
+        painter.setPen( dconf.viewConePen() );
+        painter.setBrush( dconf.transparentBrush() );
+
+        painter.drawLine( QLineF( ix, iy, end_x, end_y ) );
+    }
 
     // draw comment
     if ( opt.showDebugViewComment()
@@ -377,8 +397,8 @@ DebugPainter::drawSelf( QPainter & painter,
         painter.setFont( dconf.debugCommentFont() );
         painter.setPen( dconf.debugCommentFontPen() );
 
-        painter.drawText( ix - r,
-                          iy + r + painter.fontMetrics().ascent(),
+        painter.drawText( QPointF( ix - r,
+                                   iy + r + painter.fontMetrics().ascent() ),
                           QString::fromStdString( self->comment_ ) );
     }
 }
@@ -412,30 +432,29 @@ DebugPainter::drawBall( QPainter & painter,
                          (double)ball->vy_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
 
     const double ball_decay = rcsc::ServerParam::i().ballDecay();
-    const int ix = opt.screenX( bpos.x );
-    const int iy = opt.screenY( bpos.y );
-    const int r = std::max( 2,
-                            ( opt.enlargeMode()
-                              ? opt.scale( 0.25 )
-                              : opt.scale( 0.07 ) ) );
+    const QPointF p( opt.screenXF( bpos.x ),
+                     opt.screenYF( bpos.y ) );
+    const double r = std::max( 2.0,
+                               ( opt.enlargeMode()
+                                 ? opt.scaleF( 0.25 )
+                                 : opt.scaleF( 0.07 ) ) );
 
     painter.setPen( dconf.transparentPen() );
     painter.setBrush( dconf.debugBallBrush() );
 
-    painter.drawEllipse( ix - r,
-                         iy - r,
-                         r * 2,
-                         r * 2 );
+    painter.drawEllipse( QRectF( p.x() - r, p.y() - r,
+                                 r * 2, r * 2 ) );
 
 
     if ( opt.antiAliasing() )
     {
-        painter.setRenderHints(
-#ifdef USE_GLWIDGET
-                               //QPainter::HighQualityAntialiasing |
+#ifdef USE_HIGH_QUALITY_ANTIALIASING
+        painter.setRenderHints( QPainter::HighQualityAntialiasing |
+                                QPainter::Antialiasing,
+                                false );
+#else
+        painter.setRenderHint( QPainter::Antialiasing, false );
 #endif
-                               QPainter::Antialiasing,
-                               false );
     }
 
 #ifdef USE_GLWIDGET
@@ -443,18 +462,17 @@ DebugPainter::drawBall( QPainter & painter,
         painter.setPen( dconf.debugBallTracePen() );
         painter.setBrush( dconf.debugBallTraceBrush() );
 
-        int bx = ix;
-        int by = iy;
+        QPointF b = p;
         for ( int i = 0; i < 10; ++i )
         {
             bpos += bvel;
             bvel *= ball_decay;
-            bx = opt.screenX( bpos.x );
-            by = opt.screenY( bpos.y );
-            painter.drawEllipse( bx - 2, by - 2, 4, 4 );
+            b.setX( opt.screenXF( bpos.x ) );
+            b.setY( opt.screenYF( bpos.y ) );
+            painter.drawEllipse( QRectF( b.x() - 2, b.y() - 2, 4, 4 ) );
         }
 
-        painter.drawLine( ix, iy, bx, by );
+        painter.drawLine( QLineF( p, b ) );
     }
 #else
     {
@@ -462,18 +480,26 @@ DebugPainter::drawBall( QPainter & painter,
         painter.setPen( dconf.debugBallTracePen() );
         painter.setBrush( dconf.debugBallTraceBrush() );
 
-        int bx = ix;
-        int by = iy;
+        //QPointF b = p;
+        double bx = p.x();
+        double by = p.y();
+
+        const double len = r * 0.8;
+        const double half = len * 0.5;
+
         for ( int i = 0; i < 10; ++i )
         {
             bpos += bvel;
             bvel *= ball_decay;
-            bx = opt.screenX( bpos.x );
-            by = opt.screenY( bpos.y );
-            path.addEllipse( bx, by, 4, 4 );
+            // b.setX( opt.screenXF( bpos.x ) );
+            // b.setY( opt.screenYF( bpos.y ) );
+            // path.addEllipse( b, rr, rr );
+            bx = opt.screenXF( bpos.x );
+            by = opt.screenYF( bpos.y );
+            path.addEllipse( bx - half, by - half, len, len );
         }
-        path.moveTo( ix, iy );
-        path.lineTo( bx, by );
+        path.moveTo( p );
+        path.lineTo( QPointF( bx, by ) );
 
         painter.drawPath( path );
     }
@@ -482,12 +508,13 @@ DebugPainter::drawBall( QPainter & painter,
 
     if ( opt.antiAliasing() )
     {
-        painter.setRenderHints(
-#ifdef USE_GLWIDGET
-                               //QPainter::HighQualityAntialiasing |
+#ifdef USE_HIGH_QUALITY_ANTIALIASING
+        painter.setRenderHints( QPainter::HighQualityAntialiasing |
+                                QPainter::Antialiasing,
+                                true );
+#else
+        painter.setRenderHint( QPainter::Antialiasing, true );
 #endif
-                               QPainter::Antialiasing,
-                               true );
     }
 
     // draw comment
@@ -497,8 +524,8 @@ DebugPainter::drawBall( QPainter & painter,
         painter.setFont( dconf.debugCommentFont() );
         painter.setPen( dconf.debugCommentFontPen() );
 
-        painter.drawText( ix - r,
-                          iy + r + painter.fontMetrics().ascent(),
+        painter.drawText( QPointF( p.x() - r,
+                                   p.y() + r + painter.fontMetrics().ascent() ),
                           QString::fromStdString( ball->comment_ ) );
     }
 }
@@ -522,18 +549,18 @@ DebugPainter::drawPlayers( QPainter & painter,
                              ? -1.0
                              : 1.0 );
     const bool comment = opt.showDebugViewComment();
-    const int r = std::max( 2,
-                            ( opt.enlargeMode()
-                              ? opt.scale( 1.0 )
-                              : opt.scale( 0.3 ) ) );
+    const double r = std::max( 2.0,
+                             ( opt.enlargeMode()
+                               ? opt.scaleF( 1.0 )
+                               : opt.scaleF( 0.3 ) ) );
 
     const DebugViewData::PlayerCont::const_iterator end = players.end();
     for ( DebugViewData::PlayerCont::const_iterator it = players.begin();
           it != end;
           ++it )
     {
-        const int ix = opt.screenX( (double)(*it)->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
-        const int iy = opt.screenY( (double)(*it)->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+        const QPointF p( opt.screenXF( (double)(*it)->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse ),
+                         opt.screenYF( (double)(*it)->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse ) );
 
         if ( (*it)->body_ != -360 )
         {
@@ -544,21 +571,21 @@ DebugPainter::drawPlayers( QPainter & painter,
             // draw body half circle
             painter.setPen( dconf.transparentPen() );
             painter.setBrush( body_brush );
-            painter.drawPie( ix - r, iy - r,
-                             r * 2, r * 2,
+            painter.drawPie( QRectF( p.x() - r, p.y() - r,
+                                     r * 2, r * 2 ),
                              static_cast< int >( rint( body_start_dir * 16 ) ),
                              180 * 16 );
             // draw edge
             painter.setPen( dconf.debugPlayerPen() );
             painter.setBrush( dconf.transparentBrush() );
-            painter.drawEllipse( ix - r, iy - r, r * 2, r * 2 );
+            painter.drawEllipse( QRectF( p.x() - r, p.y() - r, r * 2, r * 2 ) );
         }
         else
         {
             // draw simple circle
             painter.setPen( dconf.transparentPen() );
             painter.setBrush( body_brush );
-            painter.drawEllipse( ix - r, iy - r, r * 2, r * 2 );
+            painter.drawEllipse( QRectF( p.x() - r, p.y() - r, r * 2, r * 2 ) );
         }
 
         if ( (*it)->unum_ > 0 )
@@ -566,8 +593,8 @@ DebugPainter::drawPlayers( QPainter & painter,
             painter.setFont( dconf.debugCommentFont() );
             painter.setPen( dconf.debugCommentFontPen() );
 
-            painter.drawText( ix + r, iy + 4,
-            QString::number( (*it)->unum_ ) );
+            painter.drawText( QPointF( p.x() + r, p.y() + 4 ),
+                              QString::number( (*it)->unum_ ) );
         }
 
         if ( target_number == (*it)->unum_ )
@@ -578,14 +605,14 @@ DebugPainter::drawPlayers( QPainter & painter,
             const boost::shared_ptr< DebugViewData::SelfT > self = view.self();
             if ( self )
             {
-                const int sx = opt.screenX( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
-                const int sy = opt.screenY( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
-                painter.drawLine( sx, sy, ix, iy );
+                QPointF s( opt.screenXF( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse ),
+                           opt.screenYF( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse ) );
+                painter.drawLine( QLineF( s, p ) );
             }
 
-            int rr = r + 2;
-            painter.drawEllipse( ix - rr, iy - rr,
-                                 rr * 2, rr * 2 );
+            double rr = r + 2;
+            painter.drawEllipse( QRectF( p.x() - rr, p.y() - rr,
+                                         rr * 2, rr * 2 ) );
         }
 
         if ( comment
@@ -594,8 +621,8 @@ DebugPainter::drawPlayers( QPainter & painter,
             painter.setFont( dconf.debugCommentFont() );
             painter.setPen( dconf.debugCommentFontPen() );
 
-            painter.drawText( ix - r,
-                              iy + r + painter.fontMetrics().ascent(),
+            painter.drawText( QPointF( p.x() - r,
+                                       p.y() + r + painter.fontMetrics().ascent() ),
                               QString::fromStdString( (*it)->comment_ ) );
         }
     }
@@ -631,10 +658,10 @@ DebugPainter::drawLines( QPainter & painter,
           it != end;
           ++it )
     {
-        painter.drawLine( opt.screenX( it->x1_ * reverse ),
-                          opt.screenY( it->y1_ * reverse ),
-                          opt.screenX( it->x2_ * reverse ),
-                          opt.screenY( it->y2_ * reverse ) );
+        painter.drawLine( QLineF( opt.screenXF( it->x1_ * reverse ),
+                                  opt.screenYF( it->y1_ * reverse ),
+                                  opt.screenXF( it->x2_ * reverse ),
+                                  opt.screenYF( it->y2_ * reverse ) ) );
     }
 #else
     QPainterPath path;
@@ -644,10 +671,10 @@ DebugPainter::drawLines( QPainter & painter,
           it != end;
           ++it )
     {
-        path.moveTo( opt.screenX( it->x1_ * reverse ),
-                     opt.screenY( it->y1_ * reverse ) );
-        path.lineTo( opt.screenX( it->x2_ * reverse ),
-                     opt.screenY( it->y2_ * reverse ) );
+        path.moveTo( opt.screenXF( it->x1_ * reverse ),
+                     opt.screenYF( it->y1_ * reverse ) );
+        path.lineTo( opt.screenXF( it->x2_ * reverse ),
+                     opt.screenYF( it->y2_ * reverse ) );
     }
 
     painter.drawPath( path );
@@ -684,12 +711,12 @@ DebugPainter::drawTriangles( QPainter & painter,
           it != end;
           ++it )
     {
-        QPoint points[3] = { QPoint( opt.screenX( it->x1_ * reverse ),
-                                     opt.screenY( it->y1_ * reverse ) ),
-                             QPoint( opt.screenX( it->x2_ * reverse ),
-                                     opt.screenY( it->y2_ * reverse ) ),
-                             QPoint( opt.screenX( it->x3_ * reverse ),
-                                     opt.screenY( it->y3_ * reverse ) ) };
+        QPointF points[3] = { QPointF( opt.screenXF( it->x1_ * reverse ),
+                                       opt.screenYF( it->y1_ * reverse ) ),
+                              QPointF( opt.screenXF( it->x2_ * reverse ),
+                                       opt.screenYF( it->y2_ * reverse ) ),
+                              QPointF( opt.screenXF( it->x3_ * reverse ),
+                                       opt.screenYF( it->y3_ * reverse ) ) };
         painter.drawLines( points, 3 );
     }
 #else
@@ -700,12 +727,12 @@ DebugPainter::drawTriangles( QPainter & painter,
           it != end;
           ++it )
     {
-        int x1 = opt.screenX( it->x1_ * reverse );
-        int y1 = opt.screenY( it->y1_ * reverse );
-        int x2 = opt.screenX( it->x2_ * reverse );
-        int y2 = opt.screenY( it->y2_ * reverse );
-        int x3 = opt.screenX( it->x3_ * reverse );
-        int y3 = opt.screenY( it->y3_ * reverse );
+        double x1 = opt.screenXF( it->x1_ * reverse );
+        double y1 = opt.screenYF( it->y1_ * reverse );
+        double x2 = opt.screenXF( it->x2_ * reverse );
+        double y2 = opt.screenYF( it->y2_ * reverse );
+        double x3 = opt.screenXF( it->x3_ * reverse );
+        double y3 = opt.screenYF( it->y3_ * reverse );
 
         path.moveTo( x1, y1 );
         path.lineTo( x2, y2 );
@@ -747,15 +774,15 @@ DebugPainter::drawRectangles( QPainter & painter,
           it != end;
           ++it )
     {
-        int left_x = opt.screenX( it->left_x_ * reverse );
-        int top_y = opt.screenY( it->top_y_ * reverse );
-        int right_x = opt.screenX( it->right_x_ * reverse );
-        int bottom_y = opt.screenY( it->bottom_y_ * reverse );
+        double left_x = opt.screenXF( it->left_x_ * reverse );
+        double top_y = opt.screenYF( it->top_y_ * reverse );
+        double right_x = opt.screenXF( it->right_x_ * reverse );
+        double bottom_y = opt.screenYF( it->bottom_y_ * reverse );
 
-        painter.drawRect( left_x,
-                          top_y,
-                          ( right_x - left_x ),
-                          ( bottom_y - top_y ) );
+        painter.drawRect( QRectF( left_x,
+                                  top_y,
+                                  ( right_x - left_x ),
+                                  ( bottom_y - top_y ) ) );
     }
 #else
     QPainterPath path;
@@ -765,15 +792,15 @@ DebugPainter::drawRectangles( QPainter & painter,
           it != end;
           ++it )
     {
-        int left_x = opt.screenX( it->left_x_ * reverse );
-        int top_y = opt.screenY( it->top_y_ * reverse );
-        int right_x = opt.screenX( it->right_x_ * reverse );
-        int bottom_y = opt.screenY( it->bottom_y_ * reverse );
+        double left_x = opt.screenXF( it->left_x_ * reverse );
+        double top_y = opt.screenYF( it->top_y_ * reverse );
+        double right_x = opt.screenXF( it->right_x_ * reverse );
+        double bottom_y = opt.screenYF( it->bottom_y_ * reverse );
 
-        path.addRect( left_x,
-                      top_y,
-                      ( right_x - left_x ),
-                      ( bottom_y - top_y ) );
+        path.addRect( QRectF( left_x,
+                              top_y,
+                              ( right_x - left_x ),
+                              ( bottom_y - top_y ) ) );
     }
 
     painter.drawPath( path );
@@ -810,11 +837,11 @@ DebugPainter::drawCircles( QPainter & painter,
           it != end;
           ++it )
     {
-        int r = opt.scale( it->radius_ );
-        painter.drawEllipse( opt.screenX( it->center_x_ * reverse ) - r,
-                             opt.screenY( it->center_y_ * reverse ) - r,
-                             r * 2,
-                             r * 2 );
+        double r = opt.scaleF( it->radius_ );
+        painter.drawEllipse( QRectF( opt.screenXF( it->center_x_ * reverse ) - r,
+                                     opt.screenYF( it->center_y_ * reverse ) - r,
+                                     r * 2,
+                                     r * 2 ) );
     }
 #else
     QPainterPath path;
@@ -824,11 +851,11 @@ DebugPainter::drawCircles( QPainter & painter,
           it != end;
           ++it )
     {
-        int r = opt.scale( it->radius_ );
-        path.addEllipse( opt.screenX( it->center_x_ * reverse ) - r,
-                         opt.screenY( it->center_y_ * reverse ) - r,
-                         r * 2,
-                         r * 2 );
+        double r = opt.scaleF( it->radius_ );
+        path.addEllipse( QRectF( opt.screenXF( it->center_x_ * reverse ) - r,
+                                 opt.screenYF( it->center_y_ * reverse ) - r,
+                                 r * 2,
+                                 r * 2 ) );
     }
 
     painter.drawPath( path );
@@ -851,7 +878,7 @@ DebugPainter::drawMessage( QPainter & painter,
 
     if ( ! view.message().empty() )
     {
-        painter.drawText( 10, y,
+        painter.drawText( QPoint( 10, y ),
                           QString::fromStdString( view.message() ) );
         y += height;
     }
@@ -862,7 +889,7 @@ DebugPainter::drawMessage( QPainter & painter,
         text += QString::fromStdString( view.sayMessage() );
         //text += QObject::tr( "\"" );
 
-        painter.drawText( 10, y, text );
+        painter.drawText( QPoint( 10, y ), text );
         y+= height;
     }
 
@@ -879,6 +906,6 @@ DebugPainter::drawMessage( QPainter & painter,
             text += QObject::tr( ")" );
         }
 
-        painter.drawText( 10, y, text );
+        painter.drawText( QPoint( 10, y ), text );
     }
 }
