@@ -305,14 +305,17 @@ FieldCanvas::paintEvent( QPaintEvent * )
 
     if ( Options::instance().antiAliasing() )
     {
-        painter.setRenderHints(
-#ifdef USE_GLWIDGET
-                               //QPainter::HighQualityAntialiasing |
+#ifdef USE_HIGH_QUALITY_ANTIALIASING
+        painter.setRenderHints( QPainter::HighQualityAntialiasing |
+                                QPainter::Antialiasing,
+                                true );
+#else
+        painter.setRenderHint( QPainter::Antialiasing, true );
 #endif
-                               QPainter::Antialiasing,
-                               true );
     }
 
+    painter.setRenderHint( QPainter::TextAntialiasing, false );
+
     M_main_data.update( this->width(),
                         this->height() );
 
@@ -373,7 +376,7 @@ FieldCanvas::mousePressEvent( QMouseEvent * event )
         else if ( M_mouse_state[2].isPressed()
                   && M_mouse_state[2].isDragged() )
         {
-            M_measure_ball_speed_point = event->pos();
+            M_measure_first_length_point = event->pos();
         }
         else if ( M_main_data.trainerData().dragMode() )
         {
@@ -395,7 +398,7 @@ FieldCanvas::mousePressEvent( QMouseEvent * event )
         {
             M_mouse_state[2].setMenuFailed( false );
         }
-        M_measure_ball_speed_point = event->pos();
+        M_measure_first_length_point = event->pos();
 
         if ( M_main_data.trainerData().dragMode()
              && M_main_data.trainerData().isPlayerDragged() )
@@ -513,8 +516,8 @@ FieldCanvas::mouseMoveEvent( QMouseEvent * event )
                 this->setCursor( QCursor( Qt::ClosedHandCursor ) );
             }
 
-            int new_x = Options::instance().absScreenX( Options::instance().focusPoint().x );
-            int new_y = Options::instance().absScreenY( Options::instance().focusPoint().y );
+            double new_x = Options::instance().absScreenXF( Options::instance().focusPoint().x );
+            double new_y = Options::instance().absScreenYF( Options::instance().focusPoint().y );
             new_x -= ( event->pos().x() - M_mouse_state[0].draggedPoint().x() );
             new_y -= ( event->pos().y() - M_mouse_state[0].draggedPoint().y() );
             emit focusChanged( QPoint( new_x, new_y ) );
@@ -593,7 +596,6 @@ FieldCanvas::draw( QPainter & painter )
 void
 FieldCanvas::drawMouseMeasure( QPainter & painter )
 {
-    const rcsc::ServerParam & SP = rcsc::ServerParam::i();
     const Options & opt = Options::instance();
 
     QPoint start_point = M_mouse_state[2].pressedPoint();
@@ -602,9 +604,10 @@ FieldCanvas::drawMouseMeasure( QPainter & painter )
     // draw straight line
     painter.setPen( DrawConfig::instance().measurePen() );
     painter.setBrush( DrawConfig::instance().transparentBrush() );
-    painter.drawLine( start_point, end_point );
+    painter.drawLine( QLine( start_point, end_point ) );
 
     QPainterPath mark_path;
+    QPainterPath text_path;
     mark_path.addEllipse( start_point.x() - 2,
                           start_point.y() - 2,
                           4,
@@ -614,71 +617,32 @@ FieldCanvas::drawMouseMeasure( QPainter & painter )
                           4,
                           4 );
 
-    rcsc::Vector2D start_real( opt.fieldX( start_point.x() ),
-                               opt.fieldY( start_point.y() ) );
-    rcsc::Vector2D end_real( opt.fieldX( end_point.x() ),
-                             opt.fieldY( end_point.y() ) );
-
-    // ball travel marks
+    if ( opt.mouseMeasureMode() == Options::MEASURE_BALL_MOVE )
     {
-        double ball_speed = SP.ballSpeedMax();
-
-        if ( ! M_measure_ball_speed_point.isNull()
-             && M_measure_ball_speed_point != start_point )
-        {
-            rcsc::Vector2D vel( std::abs( M_measure_ball_speed_point.x()
-                                          - start_point.x() )
-                                / opt.fieldScale(),
-                                std::abs( M_measure_ball_speed_point.y()
-                                          - start_point.y() )
-                                / opt.fieldScale() );
-            ball_speed = vel.r();
-            if ( ball_speed > SP.ballSpeedMax() )
-            {
-                ball_speed = SP.ballSpeedMax();
-            }
-        }
-
-        rcsc::Vector2D ball_pos = start_real;
-        rcsc::Vector2D ball_vel = end_real - start_real;
-        ball_vel.setLength( ball_speed );
-
-        const double max_length = start_real.dist( end_real );
-        double total_travel = 0.0;
-
-        ball_pos += ball_vel;
-        ball_vel *= SP.ballDecay();
-        total_travel += ball_vel.r();
-        QPoint last_pt( start_point.x() - 2,
-                        start_point.y() - 2 );
-        while ( total_travel < max_length )
-        {
-            QPoint pt( opt.absScreenX( ball_pos.x ) - 2,
-                       opt.absScreenY( ball_pos.y ) - 2 );
-            if ( std::abs( pt.x() - last_pt.x() ) < 1
-                 && std::abs( pt.y() - last_pt.y() ) < 1 )
-            {
-                break;
-            }
-            last_pt = pt;
-            mark_path.addEllipse( pt.x(),
-                                  pt.y(),
-                                  4,
-                                  4 );
-            ball_pos += ball_vel;
-            ball_vel *= SP.ballDecay();
-            ball_speed *= SP.ballDecay();
-            total_travel += ball_speed;
-        }
+        createBallMovePath( start_point, end_point, mark_path, text_path );
+    }
+    else if ( opt.mouseMeasureMode() == Options::MEASURE_PLAYER_MOVE )
+    {
+        createPlayerMovePath( start_point, end_point, mark_path, text_path );
     }
 
-    painter.setPen( Qt::red );
+    painter.setPen( QPen( Qt::red, 0, Qt::SolidLine ) );
     painter.drawPath( mark_path );
 
     // draw distance & angle text
     painter.setFont( DrawConfig::instance().measureFont() );
     painter.setPen( DrawConfig::instance().measureFontPen() );
     //painter.setBackgroundMode( Qt::TransparentMode );
+    painter.drawPath( text_path );
+
+    //
+    // draw text
+    //
+
+    rcsc::Vector2D start_real( opt.fieldX( start_point.x() ),
+                               opt.fieldY( start_point.y() ) );
+    rcsc::Vector2D end_real( opt.fieldX( end_point.x() ),
+                             opt.fieldY( end_point.y() ) );
 
     char buf[64];
     snprintf( buf, 64,
@@ -700,9 +664,7 @@ FieldCanvas::drawMouseMeasure( QPainter & painter )
               end_real.x,
               end_real.y );
 
-    painter.drawText( end_point.x(),
-                      end_point.y(),
-                      QString::fromAscii( buf ) );
+    painter.drawText( end_point, QString::fromAscii( buf ) );
 
     painter.setPen( QColor( 224, 224, 192 ) );
     rcsc::Vector2D rel( end_real - start_real );
@@ -713,11 +675,265 @@ FieldCanvas::drawMouseMeasure( QPainter & painter )
     int dist_add_y = ( end_point.y() > start_point.y()
                        ? + painter.fontMetrics().height()
                        : - painter.fontMetrics().height() );
-    painter.drawText( end_point.x(),
-                      end_point.y() + dist_add_y,
+    painter.drawText( QPoint( end_point.x(),
+                              end_point.y() + dist_add_y ),
                       QString::fromAscii( buf ) );
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+FieldCanvas::createBallMovePath( const QPoint & start_point,
+                                 const QPoint & end_point,
+                                 QPainterPath & mark_path,
+                                 QPainterPath & text_path )
+{
+    const rcsc::ServerParam & SP = rcsc::ServerParam::i();
+    const Options & opt = Options::instance();
+
+
+    rcsc::Vector2D start_real( opt.fieldX( start_point.x() ),
+                               opt.fieldY( start_point.y() ) );
+    rcsc::Vector2D end_real( opt.fieldX( end_point.x() ),
+                             opt.fieldY( end_point.y() ) );
+
+
+    double ball_speed = ( opt.mouseMeasureFirstLength() < 0.0
+                          ? SP.ballSpeedMax()
+                          : std::min( opt.mouseMeasureFirstLength(), SP.ballSpeedMax() ) );
+
+    if ( ! M_measure_first_length_point.isNull()
+         && M_measure_first_length_point != start_point )
+    {
+        rcsc::Vector2D vel( std::abs( M_measure_first_length_point.x()
+                                      - start_point.x() )
+                            / opt.fieldScale(),
+                            std::abs( M_measure_first_length_point.y()
+                                      - start_point.y() )
+                            / opt.fieldScale() );
+        ball_speed = vel.r();
+        if ( ball_speed > SP.ballSpeedMax() )
+        {
+            ball_speed = SP.ballSpeedMax();
+        }
+    }
+
+    rcsc::Vector2D ball_pos = start_real;
+    rcsc::Vector2D ball_vel = end_real - start_real;
+    ball_vel.setLength( ball_speed );
+
+    const double max_length = start_real.dist( end_real );
+    double total_travel = 0.0;
+
+    QPointF last_pos( opt.absScreenXF( ball_pos.x ),
+                      opt.absScreenYF( ball_pos.y ) );
+
+    QFont font = DrawConfig::instance().measureFont();
+    font.setPointSize( 6 );
+
+    int count = 0;
+    for ( int i = 1; i <= 100; ++i )
+    {
+        ++count;
+        total_travel += ball_speed;
+
+        if ( total_travel >= max_length )
+        {
+            break;
+        }
+
+        if ( ball_speed < 0.001 )
+        {
+            break;
+        }
+
+        ball_pos += ball_vel;
+        ball_vel *= SP.ballDecay();
+        ball_speed *= SP.ballDecay();
+
+        QPoint pos( opt.absScreenXF( ball_pos.x ),
+                    opt.absScreenYF( ball_pos.y ) );
+        double d2
+            = std::pow( pos.x() - last_pos.x(), 2 )
+            + std::pow( pos.y() - last_pos.y(), 2 );
+        if ( d2 < 16.0 )
+        {
+            //continue;
+            break;
+        }
+
+        mark_path.addEllipse( pos.x() - 1.0, pos.y() - 1.0, 2.0, 2.0 );
+        //mark_path.addEllipse( pos, 2.0, 2.0 );
+        if ( i <= 10
+             || d2 > std::pow( 13.0, 2 ) )
+        {
+#if 1
+            text_path.addText( pos, font,
+                               tr( "%1:%2")
+                               .arg( i )
+                               .arg( ball_speed, 0, 'g', 3  ) );
+#else
+            text_path.addText( pos, font, QString::number( i ) );
+#endif
+        }
+
+        last_pos = pos;
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+FieldCanvas::createPlayerMovePath( const QPoint & start_point,
+                                   const QPoint & end_point,
+                                   QPainterPath & mark_path,
+                                   QPainterPath & text_path )
+{
+    const rcsc::ServerParam & SP = rcsc::ServerParam::i();
+    const Options & opt = Options::instance();
+
+    if ( opt.selectedNumber() == 0 )
+    {
+        // no selected player
+        //std::cerr << "(createPlayerMovePath) no selected player." << std::endl;
+        return;
+    }
+
+    //
+    // decide player type
+    //
+
+    MonitorViewConstPtr view = M_main_data.getCurrentViewData();
+
+    if ( ! view )
+    {
+        //std::cerr << "(createPlayerMovePath) no view data." << std::endl;
+        return;
+    }
+
+    rcsc::SideID side = ( opt.selectedNumber() > 0
+                          ? rcsc::LEFT
+                          : rcsc::RIGHT );
+    int unum = std::abs( opt.selectedNumber() );
+    int player_type = -1;
+    for ( std::vector< Player >::const_iterator p = view->players().begin();
+          p != view->players().end();
+          ++p )
+    {
+        if ( p->side() == side
+             && p->unum() == unum )
+        {
+            player_type = p->type();
+            break;
+        }
+    }
+
+    if ( player_type < 0 )
+    {
+        std::cerr << "(createPlayerMovePath) invalid player type id=" << player_type
+                  << std::endl;
+        return;
+    }
+
+    const rcsc::PlayerType & ptype = M_main_data.viewHolder().playerType( player_type );
+
+    if ( opt.scaleInt( ptype.realSpeedMax() ) <= 4 )
+    {
+        return;
+    }
+
+    const rcsc::Vector2D start_real( opt.fieldX( start_point.x() ),
+                                     opt.fieldY( start_point.y() ) );
+    const rcsc::Vector2D end_real( opt.fieldX( end_point.x() ),
+                                   opt.fieldY( end_point.y() ) );
+
+    const rcsc::Vector2D player_accel
+        = ( end_real - start_real ).setLengthVector( SP.maxDashPower()
+                                                     * ptype.dashPowerRate()
+                                                     * ptype.effortMax() );
+
+    const double max_length = start_real.dist( end_real );
+    double total_travel = 0.0;
+
+    rcsc::Vector2D player_pos = start_real;
+#if 1
+    rcsc::Vector2D player_vel( 0.0, 0.0 );
+#else
+    double player_speed = ( opt.mouseMeasureFirstLength() < 0.0
+                            ? 0.0
+                            : std::min( opt.mouseMeasureFirstLength(), ptype.realSpeedMax() ) );
+
+    if ( ! M_measure_first_length_point.isNull()
+         && M_measure_first_length_point != start_point )
+    {
+        rcsc::Vector2D len( std::abs( M_measure_first_length_point.x()
+                                      - start_point.x() )
+                            / opt.fieldScale(),
+                            std::abs( M_measure_first_length_point.y()
+                                      - start_point.y() )
+                            / opt.fieldScale() );
+        player_speed = len.r();
+        if ( player_speed > ptype.realSpeedMax() )
+        {
+            player_speed = ptype.realSpeedMax();
+        }
+    }
+
+    rcsc::Vector2D player_vel = ( end_real - start_real ).setLengthVector( player_speed );
+
+    player_pos += player_vel;
+    player_vel *= ptype.playerDecay();
+    total_travel += player_vel.r();
+
+    {
+        QPoint pt( opt.absScreenXInt( player_pos.x ) - 2,
+                   opt.absScreenYInt( player_pos.y ) - 2 );
+        mark_path.addEllipse( pt.x(), pt.y(), 4, 4 );
+    }
+#endif
+
+    // std::cerr << "pos=" << player_pos
+    //           << " vel=" << player_vel.r()
+    //           << " accel=" << player_accel << ":" << player_accel.r()
+    //           << " max_length=" << max_length
+    //           << std::endl;
+
+    QFont font = DrawConfig::instance().measureFont();
+    font.setPointSize( 6 );
+
+    const bool draw_text = ( opt.scaleInt( ptype.realSpeedMax() ) > 8 );
+
+    for ( int i = 1; i <= 100; ++i )
+    {
+        player_vel += player_accel;
+        total_travel += player_vel.r();
+
+        if ( total_travel > max_length )
+        {
+            break;
+        }
+
+        player_pos += player_vel;
+        player_vel *= ptype.playerDecay();
+
+        QPointF pos( opt.absScreenXF( player_pos.x ),
+                     opt.absScreenYF( player_pos.y ) );
+
+        mark_path.addEllipse( pos.x() - 1.0, pos.y() - 1.0,
+                              2.0, 2.0 );
+        //mark_path.addEllipse( pos, 2.0, 2.0 );
+        if ( draw_text
+             && i <= 30 )
+        {
+            text_path.addText( pos, font,
+                               tr( "%1:%2" ).arg( i ).arg( total_travel, 0, 'g', 3 ) );
+        }
+    }
+}
 
 /*-------------------------------------------------------------------*/
 /*!
@@ -758,17 +974,17 @@ FieldCanvas::drawDraggedPlayer( QPainter & painter )
     const Player & player = view->players()[index];
     const rcsc::PlayerType & type = M_main_data.viewHolder().playerType( player.type() );
 
-    int draw_radius = Options::instance().scale( type.kickableArea() );
+    double draw_radius = Options::instance().scaleF( type.kickableArea() );
 
     const DrawConfig & dconf = DrawConfig::instance();
 
     painter.setPen( dconf.playerPen() );
     painter.setBrush( dconf.transparentBrush() );
 
-    painter.drawEllipse( M_player_dragged_point.x() - draw_radius,
-                         M_player_dragged_point.y() - draw_radius,
-                         draw_radius * 2,
-                         draw_radius * 2 );
+    painter.drawEllipse( QRectF( M_player_dragged_point.x() - draw_radius,
+                                 M_player_dragged_point.y() - draw_radius,
+                                 draw_radius * 2,
+                                 draw_radius * 2 ) );
 }
 
 /*-------------------------------------------------------------------*/
@@ -861,8 +1077,9 @@ FieldCanvas::dragPlayer( const QPoint & point )
 
     // calculate an update area
     int player_size = ( opt.playerSize() > 0.0
-                        ? opt.scale( opt.playerSize() + 0.3 )
-                        : opt.scale( rcsc::ServerParam::i().defaultKickableArea() + 0.3 ) );
+                        ? opt.scaleInt( opt.playerSize() + 0.3 )
+                        : opt.scaleInt( rcsc::ServerParam::i().defaultKickableArea() + 0.3 ) )
+        + 1;
 
     QRect new_rect( point.x() - player_size,
                     point.y() - player_size,
