@@ -127,6 +127,52 @@ private:
     QAction * M_show_font_setting_dialog_act;
     QAction * M_show_view_config_dialog_act;
 
+    // view config actions
+    QAction * M_zoom_in_act;
+    QAction * M_zoom_out_act;
+    QAction * M_fit_to_screen_act;
+    QAction * M_toggle_enlarge_act;
+    QAction * M_toggle_reverse_side_act;
+    QAction * M_toggle_player_reverse_draw_act;
+    QAction * M_toggle_show_player_number_act;
+    QAction * M_toggle_show_player_type_act;
+    QAction * M_toggle_show_stamina_act;
+    QAction * M_toggle_show_view_area_act;
+    QAction * M_toggle_show_body_shadow_act;
+    QAction * M_toggle_show_catchable_area_act;
+    QAction * M_toggle_show_tackle_area_act;
+    QAction * M_toggle_show_kick_accel_area_act;
+    QAction * M_toggle_show_pointto_act;
+    QAction * M_toggle_show_attentionto_act;
+    QAction * M_toggle_show_score_board_act;
+    QAction * M_toggle_show_team_graphic_act;
+    QAction * M_toggle_show_ball_act;
+    QAction * M_toggle_show_players_act;
+    QAction * M_toggle_show_flags_act;
+    QAction * M_toggle_show_offside_line_act;
+    QAction * M_select_left_player_act[11];
+    QAction * M_select_right_player_act[11];
+    QAction * M_toggle_focus_ball_act;
+    QAction * M_toggle_focus_player_act;
+    QAction * M_toggle_select_auto_all_act;
+    QAction * M_toggle_select_auto_left_act;
+    QAction * M_toggle_select_auto_right_act;
+    QAction * M_unselect_act;
+
+    // log player actions
+    QAction * M_log_player_go_first_act;
+    QAction * M_log_player_go_prev_score_act;
+    QAction * M_log_player_one_step_back_act;
+    QAction * M_log_player_play_or_stop_act;
+    QAction * M_log_player_one_step_forward_act;
+    QAction * M_log_player_go_next_score_act;
+    QAction * M_log_player_go_last_act;
+    QAction * M_log_player_shift_down_act;
+    QAction * M_log_player_shift_up_act;
+    QAction * M_log_player_stop_act;
+    QAction * M_log_player_play_back_act;
+    QAction * M_log_player_play_forward_act;
+
     // tool actions
     QAction * M_show_debug_message_window_act;
     QAction * M_toggle_debug_server_act;
@@ -157,12 +203,15 @@ private:
     void createActionsFile();
     void createActionsMonitor();
     void createActionsView();
+    void createActionsViewConfig();
+    void createActionsLogPlayer();
     void createActionsDebug();
     void createActionsHelp();
 
     void createMenus();
     void createMenuFile();
     void createMenuMonitor();
+    void createMenuLogPlayer();
     void createMenuView();
     void createMenuDebug();
     void createMenuHelp();
@@ -254,6 +303,9 @@ public slots:
     void movePlayer( const QPoint & point );
     void moveObjects();
 
+    void yellowCard();
+    void redCard();
+
 signals:
 
     void viewUpdated();
