@@ -128,6 +128,10 @@ public:
     void sendDiscard( const rcsc::SideID side,
                       const int unum );
 
+    void sendCard( const rcsc::SideID side,
+                   const int unum,
+                   const rcsc::Card card );
+
 
     // trainer command
 
