@@ -59,12 +59,12 @@ inline
 PlayerPainter::Param::Param( const Player & player,
                              const Ball & ball,
                              const rcsc::PlayerType & ptype )
-    : x_( 0 )
-    , y_( 0 )
+    : x_( 0.0 )
+    , y_( 0.0 )
     , body_( player.body() )
     , head_( player.head() )
-    , body_radius_( 1 )
-    , kick_radius_( 5 )
+    , body_radius_( 1.0 )
+    , kick_radius_( 5.0 )
     , have_full_effort_( player.hasFullEffort( ptype.effortMax() ) )
     , player_( player )
     , ball_( ball )
@@ -72,8 +72,8 @@ PlayerPainter::Param::Param( const Player & player,
 {
     const Options & opt = Options::instance();
 
-    x_ = opt.screenX( player.x() );
-    y_ = opt.screenY( player.y() );
+    x_ = opt.screenXF( player.x() );
+    y_ = opt.screenYF( player.y() );
 
     if ( opt.reverseSide() )
     {
@@ -81,17 +81,17 @@ PlayerPainter::Param::Param( const Player & player,
         head_ += 180.0;
     }
 
-    body_radius_ = opt.scale( ptype.playerSize() );
-    kick_radius_ = opt.scale( ptype.kickableArea() );
+    body_radius_ = opt.scaleF( ptype.playerSize() );
+    kick_radius_ = opt.scaleF( ptype.kickableArea() );
 
-    if ( body_radius_ < 1 ) body_radius_ = 1;
-    if ( kick_radius_ < 5 ) kick_radius_ = 5;
+    if ( body_radius_ < 1.0 ) body_radius_ = 1.0;
+    if ( kick_radius_ < 5.0 ) kick_radius_ = 5.0;
 
     if ( opt.enlargeMode() )
     {
         if ( opt.playerSize() > 0.0 )
         {
-            draw_radius_ = opt.scale( opt.playerSize() );
+            draw_radius_ = opt.scaleF( opt.playerSize() );
             if ( draw_radius_ < 1 ) draw_radius_ = 1;
         }
         else
@@ -313,13 +313,17 @@ PlayerPainter::drawBody( QPainter & painter,
         painter.setPen( dconf.tacklePen() );
         painter.setBrush( dconf.tackleFaultBrush() );
     }
+    if ( param.player_.isFoulCharged() )
+    {
+        painter.setBrush( dconf.foulChargedBrush() );
+    }
     if ( param.player_.isCollidedBall() )
     {
-        painter.setBrush( dconf.collideBallBrush() );
+        painter.setBrush( dconf.ballCollisionBrush() );
     }
     if ( param.player_.isCollidedPlayer() )
     {
-        painter.setBrush( dconf.collidePlayerBrush() );
+        painter.setBrush( dconf.playerCollisionBrush() );
     }
 
     if ( Options::instance().gradient() )
@@ -337,10 +341,10 @@ PlayerPainter::drawBody( QPainter & painter,
         painter.setBrush( gradient );
     }
 
-    painter.drawEllipse( param.x_ - param.draw_radius_ ,
-                         param.y_ - param.draw_radius_ ,
-                         param.draw_radius_ * 2,
-                         param.draw_radius_ * 2 );
+    painter.drawEllipse( QRectF( param.x_ - param.draw_radius_ ,
+                                 param.y_ - param.draw_radius_ ,
+                                 param.draw_radius_ * 2,
+                                 param.draw_radius_ * 2 ) );
 }
 
 /*-------------------------------------------------------------------*/
@@ -421,10 +425,10 @@ PlayerPainter::drawShadow( QPainter & painter,
     int shadow_start_dir
         = static_cast< int >( rint( ( - param.body_ + 90.0 ) * 16 ) );
     //double shadow_end_dir = shadow_start_dir + 180.0;
-    painter.drawPie( param.x_ - shadow_radius, // left x
-                     param.y_ - shadow_radius, // top y
-                     shadow_radius * 2, // width
-                     shadow_radius * 2, // height
+    painter.drawPie( QRectF( param.x_ - shadow_radius, // left x
+                             param.y_ - shadow_radius, // top y
+                             shadow_radius * 2, // width
+                             shadow_radius * 2 ), // height
                      shadow_start_dir,
                      180 * 16 ); // span angle
 #endif
@@ -441,7 +445,7 @@ PlayerPainter::drawEdge( QPainter & painter,
     const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
-    int edge_radius = 1;
+    double edge_radius = 1;
     if ( opt.enlargeMode() )
     {
         if ( opt.showBodyShadow() )
@@ -477,22 +481,22 @@ PlayerPainter::drawEdge( QPainter & painter,
     }
 
     painter.setBrush( dconf.transparentBrush() );
-    painter.drawEllipse( param.x_ - edge_radius,
-                         param.y_ - edge_radius,
-                         edge_radius * 2,
-                         edge_radius * 2 );
+    painter.drawEllipse( QRectF( param.x_ - edge_radius,
+                                 param.y_ - edge_radius,
+                                 edge_radius * 2,
+                                 edge_radius * 2 ) );
 
     // body direction line
     if ( ! opt.showBodyShadow() )
     {
-        int end_x = opt.absScreenX( param.player_.x() * opt.reverseValue()
-                                    + param.player_type_.kickableArea()
+        double end_x = opt.absScreenXF( param.player_.x() * opt.reverseValue()
+                                        + param.player_type_.kickableArea()
                                     * std::cos( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
-        int end_y = opt.absScreenY( param.player_.y() * opt.reverseValue()
-                                    + param.player_type_.kickableArea()
-                                    * std::sin( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
+        double end_y = opt.absScreenYF( param.player_.y() * opt.reverseValue()
+                                        + param.player_type_.kickableArea()
+                                        * std::sin( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
         // same pen & brush
-        painter.drawLine( param.x_, param.y_, end_x, end_y );
+        painter.drawLine( QLineF( param.x_, param.y_, end_x, end_y ) );
     }
 
     // draw stamina status if effort or recovery is decayed.
@@ -502,26 +506,25 @@ PlayerPainter::drawEdge( QPainter & painter,
         {
             painter.setPen( dconf.effortDecayedPen() );
             painter.setBrush( dconf.transparentBrush() );
-            int radius = param.draw_radius_ + 2;
-            painter.drawEllipse( param.x_ - radius,
-                                 param.y_ - radius,
-                                 radius * 2,
-                                 radius * 2 );
+            double radius = param.draw_radius_ + 2;
+            painter.drawEllipse( QRectF( param.x_ - radius,
+                                         param.y_ - radius,
+                                         radius * 2,
+                                         radius * 2 ) );
         }
         else if ( ! param.player_.hasFullRecovery() )
         {
             painter.setPen( dconf.recoveryDecayedPen() );
             painter.setBrush( dconf.transparentBrush() );
-            int radius = param.draw_radius_ + 2;
-            painter.drawEllipse( param.x_ - radius,
-                                 param.y_ - radius,
-                                 radius * 2,
-                                 radius * 2 );
+            double radius = param.draw_radius_ + 2;
+            painter.drawEllipse( QRectF( param.x_ - radius,
+                                         param.y_ - radius,
+                                         radius * 2,
+                                         radius * 2 ) );
         }
     }
 }
 
-
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -535,12 +538,13 @@ PlayerPainter::drawFuture( QPainter & painter,
 
     if ( opt.antiAliasing() )
     {
-        painter.setRenderHints(
-#ifdef USE_GLWIDGET
-                               //QPainter::HighQualityAntialiasing |
+#ifdef USE_HIGH_QUALITY_ANTIALIASING
+        painter.setRenderHints( QPainter::HighQualityAntialiasing |
+                                QPainter::Antialiasing,
+                                false );
+#else
+        painter.setRenderHint( QPainter::Antialiasing, false );
 #endif
-                               QPainter::Antialiasing,
-                               false );
     }
 
     // draw future state
@@ -554,9 +558,9 @@ PlayerPainter::drawFuture( QPainter & painter,
 
     const int last = opt.playerFutureCycle();
 
-    QPoint first_point( opt.absScreenX( ppos.x ),
-                        opt.absScreenY( ppos.y ) ) ;
-    QPoint last_point = first_point;
+    QPointF first_point( opt.absScreenXF( ppos.x ),
+                         opt.absScreenYF( ppos.y ) ) ;
+    QPointF last_point = first_point;
 
     // draw kickable area edge
     rcsc::SideID side = param.player_.side();
@@ -582,22 +586,22 @@ PlayerPainter::drawFuture( QPainter & painter,
         ppos += pvel;
         pvel *= param.player_type_.playerDecay();
 
-        QPoint pt( opt.absScreenX( ppos.x ),
-                   opt.absScreenY( ppos.y ) );
-        if ( std::abs( last_point.x() - pt.x() ) < 1
-             && std::abs( last_point.y() - pt.y() ) < 1 )
+        QPointF pt( opt.absScreenXF( ppos.x ),
+                    opt.absScreenYF( ppos.y ) );
+        if ( std::fabs( last_point.x() - pt.x() ) < 1
+             && std::fabs( last_point.y() - pt.y() ) < 1 )
         {
             break;
         }
 
-        path.addEllipse( pt.x() - 1,
-                         pt.y() - 1,
-                         2,
-                         2 );
-        path.addEllipse( pt.x() - param.kick_radius_,
-                         pt.y() - param.kick_radius_,
-                         param.kick_radius_ * 2,
-                         param.kick_radius_ * 2 );
+        path.addEllipse( QRectF( pt.x() - 1,
+                                 pt.y() - 1,
+                                 2,
+                                 2 ) );
+        path.addEllipse( QRectF( pt.x() - param.kick_radius_,
+                                 pt.y() - param.kick_radius_,
+                                 param.kick_radius_ * 2,
+                                 param.kick_radius_ * 2 ) );
 
         last_point = pt;
     }
@@ -606,16 +610,17 @@ PlayerPainter::drawFuture( QPainter & painter,
 
     // draw move line
     painter.setPen( dconf.debugTargetPen() );
-    painter.drawLine( first_point, last_point );
+    painter.drawLine( QLineF( first_point, last_point ) );
 
     if ( opt.antiAliasing() )
     {
-        painter.setRenderHints(
-#ifdef USE_GLWIDGET
-                               //QPainter::HighQualityAntialiasing |
+#ifdef USE_HIGH_QUALITY_ANTIALIASING
+        painter.setRenderHints( QPainter::HighQualityAntialiasing |
+                                QPainter::Antialiasing,
+                                true );
+#else
+        painter.setRenderHint( QPainter::Antialiasing, true );
 #endif
-                               QPainter::Antialiasing,
-                               true );
     }
 }
 
@@ -633,7 +638,7 @@ PlayerPainter::drawViewArea( QPainter & painter,
     const double view_width = param.player_.viewWidth();
 
     const double visible_dist = rcsc::ServerParam::i().visibleDistance();
-    const int visible_radius = opt.scale( visible_dist );
+    const double visible_radius = opt.scaleF( visible_dist );
 
     const double view_start_angle = -param.head_ - view_width * 0.5;
 
@@ -641,9 +646,9 @@ PlayerPainter::drawViewArea( QPainter & painter,
     if ( opt.isSelectedPlayer( param.player_.side(),
                                param.player_.unum() ) )
     {
-        const int UNUM_FAR = opt.scale( 20.0 );
-        const int TEAM_FAR = opt.scale( 40.0 );
-        const int TEAM_TOOFAR = opt.scale( 60.0 );
+        const double UNUM_FAR = opt.scaleF( 20.0 );
+        const double TEAM_FAR = opt.scaleF( 40.0 );
+        const double TEAM_TOOFAR = opt.scaleF( 60.0 );
 
         painter.setPen( dconf.linePen() );
         //painter.setBrush( dconf.transparentBrush() );
@@ -652,50 +657,50 @@ PlayerPainter::drawViewArea( QPainter & painter,
             = static_cast< int >( rint( view_start_angle * 16 ) );
         int span_angle = static_cast< int >( rint( view_width * 16 ) );
 
-        painter.drawArc( param.x_ - UNUM_FAR, // left x
-                         param.y_ - UNUM_FAR, // toop y
-                         UNUM_FAR * 2, // width
-                         UNUM_FAR * 2, // height
+        painter.drawArc( QRectF( param.x_ - UNUM_FAR, // left x
+                                 param.y_ - UNUM_FAR, // toop y
+                                 UNUM_FAR * 2, // width
+                                 UNUM_FAR * 2 ), // height
                          view_start_angle_int,
                          span_angle );
-        painter.drawArc( param.x_ - TEAM_FAR, // left x
-                         param.y_ - TEAM_FAR, // toop y
-                         TEAM_FAR * 2, // width
-                         TEAM_FAR * 2, // height
+        painter.drawArc( QRectF( param.x_ - TEAM_FAR, // left x
+                                 param.y_ - TEAM_FAR, // toop y
+                                 TEAM_FAR * 2, // width
+                                 TEAM_FAR * 2 ), // height
                          view_start_angle_int,
                          span_angle );
         // pie, no an arc
-        painter.drawArc( param.x_ - TEAM_TOOFAR, // left x
-                         param.y_ - TEAM_TOOFAR, // toop y
-                         TEAM_TOOFAR * 2, // width
-                         TEAM_TOOFAR * 2, // height
+        painter.drawArc( QRectF( param.x_ - TEAM_TOOFAR, // left x
+                                 param.y_ - TEAM_TOOFAR, // toop y
+                                 TEAM_TOOFAR * 2, // width
+                                 TEAM_TOOFAR * 2 ), // height
                          view_start_angle_int,
                          span_angle );
 
         // draw feeling area circle
-        painter.drawArc( param.x_ - visible_radius,
-                         param.y_ - visible_radius,
-                         visible_radius * 2,
-                         visible_radius * 2,
+        painter.drawArc( QRectF( param.x_ - visible_radius,
+                                 param.y_ - visible_radius,
+                                 visible_radius * 2,
+                                 visible_radius * 2 ),
                          0,
                          360 * 16 );
 
         const double view_start_angle_real = ( param.head_ - view_width * 0.5 ) * rcsc::AngleDeg::DEG2RAD;
         const double view_end_angle_real = ( param.head_ + view_width * 0.5 ) * rcsc::AngleDeg::DEG2RAD;
         // left side view cone end point x
-        int lx = param.x_
-            + opt.scale( 60.0 * std::cos( view_start_angle_real ) );
+        double lx = param.x_
+            + opt.scaleF( 60.0 * std::cos( view_start_angle_real ) );
         // left side view cone end point y
-        int ly = param.y_
-            + opt.scale( 60.0  * std::sin( view_start_angle_real ) );
+        double ly = param.y_
+            + opt.scaleF( 60.0  * std::sin( view_start_angle_real ) );
         // right side view cone end point x
-        int rx = param.x_
-            + opt.scale( 60.0 * std::cos( view_end_angle_real ) );
+        double rx = param.x_
+            + opt.scaleF( 60.0 * std::cos( view_end_angle_real ) );
         // right side view cone end point y
-        int ry = param.y_
-            + opt.scale( 60.0 * std::sin( view_end_angle_real ) );
-        painter.drawLine( lx, ly, param.x_, param.y_ );
-        painter.drawLine( rx, ry, param.x_, param.y_ );
+        double ry = param.y_
+            + opt.scaleF( 60.0 * std::sin( view_end_angle_real ) );
+        painter.drawLine( QLineF( lx, ly, param.x_, param.y_ ) );
+        painter.drawLine( QLineF( rx, ry, param.x_, param.y_ ) );
     }
     else
     {
@@ -732,14 +737,14 @@ PlayerPainter::drawViewDir( QPainter & painter,
                  ? param.player_type_.kickableArea()
                  : param.player_type_.playerSize() );
 
-    int end_x = opt.absScreenX( param.player_.x() * opt.reverseValue()
-                                + r * std::cos( param.head_ * rcsc::AngleDeg::DEG2RAD ) );
-    int end_y = opt.absScreenY( param.player_.y() * opt.reverseValue()
-                                + r * std::sin( param.head_ * rcsc::AngleDeg::DEG2RAD ) );
+    double end_x = opt.absScreenXF( param.player_.x() * opt.reverseValue()
+                                    + r * std::cos( param.head_ * rcsc::AngleDeg::DEG2RAD ) );
+    double end_y = opt.absScreenYF( param.player_.y() * opt.reverseValue()
+                                   + r * std::sin( param.head_ * rcsc::AngleDeg::DEG2RAD ) );
 
     painter.setPen( Qt::black );
     painter.setBrush( dconf.transparentBrush() );
-    painter.drawLine( param.x_, param.y_, end_x, end_y );
+    painter.drawLine( QLineF( param.x_, param.y_, end_x, end_y ) );
 }
 
 /*-------------------------------------------------------------------*/
@@ -752,6 +757,7 @@ PlayerPainter::drawCatchableArea( QPainter & painter,
 {
     const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
+    const rcsc::ServerParam & SP = rcsc::ServerParam::instance();
 
     rcsc::SideID side = param.player_.side();
     if ( opt.reverseSide() )
@@ -759,16 +765,76 @@ PlayerPainter::drawCatchableArea( QPainter & painter,
         side = static_cast< rcsc::SideID >( -1 * side );
     }
 
-    int catchable = opt.scale( rcsc::ServerParam::i().catchableArea() );
+    double catchable = opt.scaleF( SP.catchableArea() );
     painter.setPen( ( side == rcsc::LEFT )
                     ? dconf.leftGoaliePen()
                     : dconf.rightGoaliePen() );
     painter.setBrush( dconf.transparentBrush() );;
 
-    painter.drawEllipse( param.x_ - catchable,
-                         param.y_ - catchable,
-                         catchable * 2,
-                         catchable * 2 );
+    painter.drawEllipse( QRectF( param.x_ - catchable,
+                                 param.y_ - catchable,
+                                 catchable * 2,
+                                 catchable * 2 ) );
+
+    double delta = SP.catchAreaLength() * std::max( 0.0, param.player_type_.catchAreaLengthStretch() - 1.0 );
+    double max_l = SP.catchAreaLength() + delta;
+    double min_l = SP.catchAreaLength() - delta;
+    double max_area = std::sqrt( std::pow( SP.catchAreaWidth() * 0.5, 2.0 )
+                                 + std::pow( max_l, 2.0 ) );
+    double min_area = std::sqrt( std::pow( SP.catchAreaWidth() * 0.5, 2.0 )
+                                 + std::pow( min_l, 2.0 ) );
+    double max_r = opt.scaleF( max_area );
+    if ( max_r > catchable )
+    {
+        painter.setPen( ( side == rcsc::LEFT )
+                        ? dconf.leftGoalieStretchPen()
+                        : dconf.rightGoalieStretchPen() );
+        painter.drawEllipse( QRectF( param.x_ - max_r,
+                                     param.y_ - max_r,
+                                     max_r * 2,
+                                     max_r * 2 ) );
+        double min_r = opt.scaleF( min_area );
+        painter.drawEllipse( QRectF( param.x_ - min_r,
+                                     param.y_ - min_r,
+                                     min_r * 2,
+                                     min_r * 2 ) );
+    }
+
+    //
+    // catch probability
+    //
+#if 0
+    const double ball_dist = std::sqrt( std::pow( param.player_.x() - param.ball_.x(), 2 )
+                                        + std::pow( param.player_.y() - param.ball_.y(), 2 ) );
+    double catch_prob = SP.catchProbability();
+    if ( ball_dist > max_area )
+    {
+        // catch_prob = 0.0;
+        return;
+    }
+
+    if ( ball_dist > min_area )
+    {
+        double x = ball_dist * ( max_l / max_area ); // sin(theta)
+        catch_prob
+            = SP.catchProbability()
+            - SP.catchProbability() * ( ( x - min_l ) / ( max_l - min_l ) );
+//         catch_prob
+//             = SP.catchProbability()
+//             - SP.catchProbability() * ( ( ball_dist - min_area )
+//                                         / ( max_area - min_area ) );
+    }
+
+    int text_radius = std::min( 40, param.draw_radius_ );
+
+    painter.setPen( ( param.player_.side() == rcsc::LEFT )
+                    ? dconf.rightGoaliePen()
+                    : dconf.leftGoaliePen() );
+    painter.setFont( dconf.playerFont() );
+    painter.drawText( QPointF( param.x_ + text_radius,
+                               param.y_ + 4 + painter.fontMetrics().ascent() * 2 ),
+                      QString( "Catch=%1" ).arg( catch_prob, 0, 'g', 3 ) );
+#endif
 }
 
 /*-------------------------------------------------------------------*/
@@ -800,11 +866,21 @@ PlayerPainter::drawTackleArea( QPainter & painter,
     double tackle_dist = ( player_to_ball.x > 0.0
                            ? SP.tackleDist()
                            : SP.tackleBackDist() );
+    if ( tackle_dist < 1.0e-5 )
+    {
+        return;
+    }
+
     double tackle_prob = ( std::pow( player_to_ball.absX() / tackle_dist,
                                      SP.tackleExponent() )
                            + std::pow( player_to_ball.absY() / SP.tackleWidth(),
                                        SP.tackleExponent() ) );
-    if ( tackle_prob < 1.0 )
+    double foul_prob = ( std::pow( player_to_ball.absX() / tackle_dist,
+                                   SP.foulExponent() )
+                         + std::pow( player_to_ball.absY() / SP.tackleWidth(), // 1.25,
+                                     SP.foulExponent() ) );
+    if ( tackle_prob < 1.0
+         || foul_prob < 1.0 )
     {
         rcsc::AngleDeg body_angle = param.body_;
         rcsc::AngleDeg body_angle_side = body_angle + 90.0;
@@ -817,15 +893,15 @@ PlayerPainter::drawTackleArea( QPainter & painter,
         double right_x = SP.tackleWidth() * body_angle_side.cos();
         double right_y = SP.tackleWidth() * body_angle_side.sin();
 
-        QPoint pts[5];
-        pts[0].setX( opt.absScreenX( ppos.x + forward_x + right_x ) );
-        pts[0].setY( opt.absScreenY( ppos.y + forward_y + right_y ) );
-        pts[1].setX( opt.absScreenX( ppos.x + forward_x - right_x ) );
-        pts[1].setY( opt.absScreenY( ppos.y + forward_y - right_y ) );
-        pts[2].setX( opt.absScreenX( ppos.x + back_x - right_x ) );
-        pts[2].setY( opt.absScreenY( ppos.y + back_y - right_y ) );
-        pts[3].setX( opt.absScreenX( ppos.x + back_x + right_x ) );
-        pts[3].setY( opt.absScreenY( ppos.y + back_y + right_y ) );
+        QPointF pts[5];
+        pts[0].setX( opt.absScreenXF( ppos.x + forward_x + right_x ) );
+        pts[0].setY( opt.absScreenYF( ppos.y + forward_y + right_y ) );
+        pts[1].setX( opt.absScreenXF( ppos.x + forward_x - right_x ) );
+        pts[1].setY( opt.absScreenYF( ppos.y + forward_y - right_y ) );
+        pts[2].setX( opt.absScreenXF( ppos.x + back_x - right_x ) );
+        pts[2].setY( opt.absScreenYF( ppos.y + back_y - right_y ) );
+        pts[3].setX( opt.absScreenXF( ppos.x + back_x + right_x ) );
+        pts[3].setY( opt.absScreenYF( ppos.y + back_y + right_y ) );
         pts[4] = pts[0];
 
         painter.setPen( dconf.tackleAreaPen() );
@@ -843,10 +919,30 @@ PlayerPainter::drawTackleArea( QPainter & painter,
         painter.setFont( dconf.playerFont() );
         painter.setPen( dconf.tacklePen() );
 
-        //snprintf( msg, 32, "TackleProb=%.3f", 1.0 - tackle_prob );
-        painter.drawText( param.x_ + text_radius,
-                          param.y_ - text_radius + painter.fontMetrics().height(),
-                          QString( "TackleProb=%1" ).arg( 1.0 - tackle_prob, 0, 'g', 3 ) );
+        if ( tackle_prob < 1.0
+             && foul_prob < 1.0 )
+        {
+            painter.drawText( QPointF( param.x_ + text_radius,
+                                       param.y_ + 4 + painter.fontMetrics().ascent() ),
+                              QString( "T=%1,F=%2" )
+                              .arg( 1.0 - tackle_prob, 0, 'g', 3 )
+                              .arg( 1.0 - foul_prob, 0, 'g', 3 )
+                              );
+        }
+        else if ( tackle_prob < 1.0 )
+        {
+            painter.drawText( QPointF( param.x_ + text_radius,
+                                       param.y_ + 4 + painter.fontMetrics().ascent() ),
+                              QString( "Tackle=%1" )
+                              .arg( 1.0 - tackle_prob, 0, 'g', 3 ) );
+        }
+        else if ( foul_prob < 1.0 )
+        {
+            painter.drawText( QPointF( param.x_ + text_radius,
+                                       param.y_ + 4 + painter.fontMetrics().ascent() ),
+                              QString( "Foul=%1" )
+                              .arg( 1.0 - foul_prob, 0, 'g', 3 ) );
+        }
     }
 }
 
@@ -891,18 +987,18 @@ PlayerPainter::drawKickAccelArea( QPainter & painter,
             max_kick_accel = SP.ballAccelMax();
         }
 
-        QPoint bpos_screen( opt.screenX( bpos.x ),
-                            opt.screenY( bpos.y ) );
-        QPoint bnext_screen( opt.screenX( bnext.x ),
-                             opt.screenY( bnext.y ) );
-        int max_speed_screen = opt.scale( SP.ballSpeedMax() );
-        int max_kick_accel_screen = opt.scale( max_kick_accel );
+        QPointF bpos_screen( opt.screenXF( bpos.x ),
+                             opt.screenYF( bpos.y ) );
+        QPointF bnext_screen( opt.screenXF( bnext.x ),
+                              opt.screenYF( bnext.y ) );
+        double max_speed_screen = opt.scaleF( SP.ballSpeedMax() );
+        double max_kick_accel_screen = opt.scaleF( max_kick_accel );
 
         painter.setPen( dconf.kickAccelPen() );
         painter.setBrush( dconf.transparentBrush() );
 
         // draw no noise next ball move only by inertia
-        painter.drawLine( bpos_screen, bnext_screen );
+        painter.drawLine( QLineF( bpos_screen, bnext_screen ) );
 
         rcsc::Circle2D max_speed_circle( bpos, SP.ballSpeedMax() );
         rcsc::Circle2D max_accel_circle( bnext, max_kick_accel );
@@ -916,10 +1012,10 @@ PlayerPainter::drawKickAccelArea( QPainter & painter,
             // no intersection points
 
             // just draw a next ball reachable area by max accel
-            painter.drawEllipse( bnext_screen.x() - max_kick_accel_screen,
-                                 bnext_screen.y() - max_kick_accel_screen,
-                                 max_kick_accel_screen * 2,
-                                 max_kick_accel_screen * 2 );
+            painter.drawEllipse( QRectF( bnext_screen.x() - max_kick_accel_screen,
+                                         bnext_screen.y() - max_kick_accel_screen,
+                                         max_kick_accel_screen * 2,
+                                         max_kick_accel_screen * 2 ) );
         }
         else
         {
@@ -962,10 +1058,10 @@ PlayerPainter::drawKickAccelArea( QPainter & painter,
                 = static_cast< int >( rint( - bpos_start_angle->degree() * 16 ) );
             int bpos_angle_span_int
                 = static_cast< int >( rint( bpos_angle_span * 16 ) );
-            painter.drawArc( bpos_screen.x() - max_speed_screen,
-                             bpos_screen.y() - max_speed_screen,
-                             max_speed_screen * 2,
-                             max_speed_screen * 2,
+            painter.drawArc( QRectF( bpos_screen.x() - max_speed_screen,
+                                     bpos_screen.y() - max_speed_screen,
+                                     max_speed_screen * 2,
+                                     max_speed_screen * 2 ),
                              bpos_start_angle_int,
                              bpos_angle_span_int  );
 
@@ -1006,10 +1102,10 @@ PlayerPainter::drawKickAccelArea( QPainter & painter,
                 = static_cast< int >( rint( - bnext_start_angle->degree() * 16 ) );
             int bnext_angle_span_int
                 = static_cast< int >( rint( bnext_angle_span * 16 ) );
-            painter.drawArc( bnext_screen.x() - max_kick_accel_screen,
-                             bnext_screen.y() - max_kick_accel_screen,
-                             max_kick_accel_screen * 2,
-                             max_kick_accel_screen * 2,
+            painter.drawArc( QRectF( bnext_screen.x() - max_kick_accel_screen,
+                                     bnext_screen.y() - max_kick_accel_screen,
+                                     max_kick_accel_screen * 2,
+                                     max_kick_accel_screen * 2 ),
                              bnext_start_angle_int,
                              bnext_angle_span_int );
         }
@@ -1019,8 +1115,8 @@ PlayerPainter::drawKickAccelArea( QPainter & painter,
         painter.setPen( dconf.kickAccelPen() );
 
         //snprintf( buf, 32, "MaxAccel=%.3f", max_kick_accel );
-        painter.drawText( bnext_screen.x() + 10,
-                          bnext_screen.y() + painter.fontMetrics().ascent(),
+        painter.drawText( QPointF( bnext_screen.x() + 10,
+                                   bnext_screen.y() + painter.fontMetrics().ascent() ),
                           QString( "MaxAccel=%1" ).arg( max_kick_accel, 0, 'g', 4 ) );
     }
 }
@@ -1038,15 +1134,15 @@ PlayerPainter::drawPointto( QPainter & painter,
         return;
     }
 
-    int px = Options::instance().screenX( param.player_.data().point_x_ );
-    int py = Options::instance().screenY( param.player_.data().point_y_ );
+    double px = Options::instance().screenXF( param.player_.data().point_x_ );
+    double py = Options::instance().screenYF( param.player_.data().point_y_ );
 
     painter.setPen( DrawConfig::instance().pointtoPen() );
     painter.setBrush( DrawConfig::instance().transparentBrush() );
 
-    painter.drawLine( param.x_, param.y_, px, py );
-    painter.drawLine( px - 2, py - 2, px + 2, py + 2 );
-    painter.drawLine( px - 2, py + 2, px + 2, py - 2 );
+    painter.drawLine( QLineF( param.x_, param.y_, px, py ) );
+    painter.drawLine( QLineF( px - 2, py - 2, px + 2, py + 2 ) );
+    painter.drawLine( QLineF( px - 2, py + 2, px + 2, py - 2 ) );
 
 }
 
@@ -1100,12 +1196,12 @@ PlayerPainter::drawAttentionto( QPainter & painter,
             //painter.setPen( DrawConfig::instance().attentiontoPen() );
             painter.setBrush( DrawConfig::instance().transparentBrush() );
 
-            int px = opt.screenX( p->x() );
-            int py = opt.screenY( p->y() );
+            double px = opt.screenXF( p->x() );
+            double py = opt.screenYF( p->y() );
             //int r = param.draw_radius_ + 3;
-            int r = opt.scale( 2.0 );
-            int lx = px;
-            int ly = py;
+            double r = opt.scaleF( 2.0 );
+            double lx = px;
+            double ly = py;
 
             rcsc::Vector2D rel( param.player_.x(),
                                 param.player_.y() );
@@ -1116,12 +1212,12 @@ PlayerPainter::drawAttentionto( QPainter & painter,
             if ( len > std::pow( 2.0, 2 ) )
             {
                 rel *= 2.0 / len;
-                lx = opt.screenX( p->x() + rel.x );
-                ly = opt.screenY( p->y() + rel.y );
+                lx = opt.screenXF( p->x() + rel.x );
+                ly = opt.screenYF( p->y() + rel.y );
             }
 
-            painter.drawLine( param.x_, param.y_, lx, ly );
-            painter.drawEllipse( px - r, py - r, r * 2, r * 2 );
+            painter.drawLine( QLineF( param.x_, param.y_, lx, ly ) );
+            painter.drawEllipse( QRectF( px - r, py - r, r * 2, r * 2 ) );
             break;
         }
     }
@@ -1138,30 +1234,70 @@ PlayerPainter::drawText( QPainter & painter,
     const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
-    const int text_radius = std::min( 40, param.draw_radius_ );
+    painter.setFont( dconf.playerFont() );
+
+    const double text_radius = std::min( 40.0, param.draw_radius_ );
+    int card_offset = 0;
+
+    if ( opt.showCard()
+         && ( param.player_.hasRedCard()
+              || param.player_.hasYellowCard() ) )
+    {
+        QFontMetrics fm = painter.fontMetrics();
+        int x_size = std::min( std::max( 4, fm.ascent() - 2 ), 20 );
+        int y_size = std::min( std::max( 4, fm.ascent() ), 24 );
+
+        card_offset = x_size + 2;
+
+        if ( opt.antiAliasing() )
+        {
+#ifdef USE_HIGH_QUALITY_ANTIALIASING
+            painter.setRenderHints( QPainter::HighQualityAntialiasing |
+                                    QPainter::Antialiasing,
+                                    false );
+#else
+            painter.setRenderHint( QPainter::Antialiasing, false );
+#endif
+        }
+        painter.setPen( Qt::black );
+        painter.setBrush( param.player_.hasRedCard()
+                          ? Qt::red
+                          : Qt::yellow );
+        painter.drawRect( QRectF( param.x_ + text_radius,
+                                  param.y_ + 4 - y_size,
+                                  x_size, y_size ) );
+        if ( opt.antiAliasing() )
+        {
+#ifdef USE_HIGH_QUALITY_ANTIALIASING
+            painter.setRenderHints( QPainter::HighQualityAntialiasing |
+                                    QPainter::Antialiasing,
+                                    true );
+#else
+            painter.setRenderHint( QPainter::Antialiasing, true );
+#endif
+        }
+    }
 
     if ( opt.showPlayerNumber()
          && opt.showPlayerType() )
     {
         painter.setPen( dconf.playerNumberFontPen() );
-        painter.setFont( dconf.playerFont() );
-        painter.drawText( param.x_ + text_radius,
-                          param.y_ + 4,
+        painter.drawText( QPointF( param.x_ + text_radius + card_offset,
+                                   param.y_ + 4 ),
                           QString( "%1,t%2" ).arg( param.player_.unum() ).arg( param.player_.type() ) );
     }
     else if ( opt.showPlayerNumber() )
     {
         painter.setPen( dconf.playerNumberFontPen() );
-        painter.setFont( dconf.playerFont() );
-        painter.drawText( param.x_ + text_radius,
-                          param.y_ + 4,
+        painter.drawText( QPointF( param.x_ + text_radius + card_offset,
+                                   param.y_ + 4 ),
                           QString::number( param.player_.unum() ) );
     }
     else if ( opt.showPlayerType() )
     {
         painter.setPen( dconf.playerNumberFontPen() );
-        painter.drawText( param.x_ + text_radius,
-                          param.y_ + 4,
+        painter.drawText( QPointF( param.x_ + text_radius + card_offset,
+                                   param.y_ + 4 ),
                           QString( "t%1" ).arg( param.player_.type() ) );
     }
 
@@ -1196,9 +1332,8 @@ PlayerPainter::drawText( QPainter & painter,
                 painter.setPen( dconf.playerStaminaFontPen() );
             }
 
-            painter.setFont( dconf.playerFont() );
-            painter.drawText( param.x_ - text_radius,
-                              param.y_ - text_radius - 3,
+            painter.drawText( QPointF( param.x_ - text_radius,
+                                       param.y_ - text_radius - 3 ),
                               str );
         }
     }
