@@ -57,13 +57,13 @@ class PlayerPainter
 private:
 
     struct Param {
-        int x_; //!< screen X coordinates
-        int y_; //!< screen Y coordinates
+        double x_; //!< screen X coordinates
+        double y_; //!< screen Y coordinates
         double body_;
         double head_;
-        int body_radius_; //!< pixel of body radius
-        int kick_radius_; //!< pixel of kick area radius
-        int draw_radius_; //!< pixel of main draw radius.
+        double body_radius_; //!< pixel of body radius
+        double kick_radius_; //!< pixel of kick area radius
+        double draw_radius_; //!< pixel of main draw radius.
         bool have_full_effort_; //!< flag to check effort value
         const Player & player_;
         const Ball & ball_;
