@@ -97,12 +97,12 @@ inline
 PlayerPainterRCSS::Param::Param( const Player & player,
                                  const Ball & ball,
                                  const rcsc::PlayerType & ptype )
-    : x_( 0 )
-    , y_( 0 )
+    : x_( 0.0 )
+    , y_( 0.0 )
     , body_( player.body() )
     , head_( player.head() )
-    , body_radius_( 1 )
-    , kick_radius_( 5 )
+    , body_radius_( 1.0 )
+    , kick_radius_( 5.0 )
     , have_full_effort_( player.hasFullEffort( ptype.effortMax() ) )
     , player_( player )
     , ball_( ball )
@@ -110,8 +110,8 @@ PlayerPainterRCSS::Param::Param( const Player & player,
 {
     const Options & opt = Options::instance();
 
-    x_ = opt.screenX( player.x() );
-    y_ = opt.screenY( player.y() );
+    x_ = opt.screenXF( player.x() );
+    y_ = opt.screenYF( player.y() );
 
     if ( opt.reverseSide() )
     {
@@ -119,11 +119,11 @@ PlayerPainterRCSS::Param::Param( const Player & player,
         head_ += 180.0;
     }
 
-    body_radius_ = opt.scale( ptype.playerSize() );
-    kick_radius_ = opt.scale( ptype.kickableArea() );
+    body_radius_ = opt.scaleF( ptype.playerSize() );
+    kick_radius_ = opt.scaleF( ptype.kickableArea() );
 
-    if ( body_radius_ < 1 ) body_radius_ = 1;
-    if ( kick_radius_ < 5 ) kick_radius_ = 5;
+    if ( body_radius_ < 1.0 ) body_radius_ = 1.0;
+    if ( kick_radius_ < 5.0 ) kick_radius_ = 5.0;
 
     draw_radius_ =  kick_radius_;
 }
@@ -331,6 +331,10 @@ PlayerPainterRCSS::drawBody( QPainter & painter,
         painter.setPen( dconf.tacklePen() );
         painter.setBrush( dconf.tackleFaultBrush() );
     }
+    if ( param.player_.isFoulCharged() )
+    {
+        painter.setBrush( dconf.foulChargedBrush() );
+    }
     if ( param.player_.isCollidedBall() )
     {
         painter.setBrush( BALL_COLLIDE_BRUSH );
@@ -340,10 +344,10 @@ PlayerPainterRCSS::drawBody( QPainter & painter,
 //         painter.setBrush( dconf.collidePlayerBrush() );
 //     }
 
-    painter.drawEllipse( param.x_ - param.kick_radius_ ,
-                         param.y_ - param.kick_radius_ ,
-                         param.kick_radius_ * 2 ,
-                         param.kick_radius_ * 2 );
+    painter.drawEllipse( QRectF( param.x_ - param.kick_radius_ ,
+                                 param.y_ - param.kick_radius_ ,
+                                 param.kick_radius_ * 2 ,
+                                 param.kick_radius_ * 2 ) );
 
 }
 
@@ -375,10 +379,10 @@ PlayerPainterRCSS::drawEdge( QPainter & painter,
         // draw real body edge
         painter.setPen( PLAYER_PEN );
         painter.setBrush( dconf.transparentBrush() );
-        painter.drawEllipse( param.x_ - param.body_radius_,
-                             param.y_ - param.body_radius_,
-                             param.body_radius_ * 2 ,
-                             param.body_radius_ * 2  );
+        painter.drawEllipse( QRectF( param.x_ - param.body_radius_,
+                                     param.y_ - param.body_radius_,
+                                     param.body_radius_ * 2 ,
+                                     param.body_radius_ * 2  ) );
     }
 
     // draw stamina status if effort or recovery is decayed.
@@ -387,30 +391,29 @@ PlayerPainterRCSS::drawEdge( QPainter & painter,
         if ( param.player_.stamina() < 1500.0 )
         {
             painter.setBrush( dconf.shadowBrush() );
-            painter.drawEllipse( param.x_ - param.body_radius_,
-                                 param.y_ - param.body_radius_,
-                                 param.body_radius_ * 2 ,
-                                 param.body_radius_ * 2  );
+            painter.drawEllipse( QRectF( param.x_ - param.body_radius_,
+                                         param.y_ - param.body_radius_,
+                                         param.body_radius_ * 2 ,
+                                         param.body_radius_ * 2  ) );
         }
     }
 
     // body direction line
     {
-        int end_x = opt.absScreenX( param.player_.x() * opt.reverseValue()
-                                      + param.player_type_.kickableArea()
-                                      * std::cos( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
-        int end_y = opt.absScreenY( param.player_.y() * opt.reverseValue()
-                                      + param.player_type_.kickableArea()
-                                      * std::sin( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
+        double end_x = opt.absScreenXF( param.player_.x() * opt.reverseValue()
+                                        + param.player_type_.kickableArea()
+                                        * std::cos( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
+        double end_y = opt.absScreenYF( param.player_.y() * opt.reverseValue()
+                                        + param.player_type_.kickableArea()
+                                        * std::sin( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
 
         painter.setPen( PLAYER_PEN );
         painter.setBrush( dconf.transparentBrush() );
-        painter.drawLine( param.x_, param.y_, end_x, end_y );
+        painter.drawLine( QLineF( param.x_, param.y_, end_x, end_y ) );
 
     }
 }
 
-
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -424,12 +427,13 @@ PlayerPainterRCSS::drawFuture( QPainter & painter,
 
     if ( opt.antiAliasing() )
     {
-        painter.setRenderHints(
-#ifdef USE_GLWIDGET
-                               //QPainter::HighQualityAntialiasing |
+#ifdef USE_HIGH_QUALITY_ANTIALIASING
+        painter.setRenderHints( QPainter::HighQualityAntialiasing |
+                                QPainter::Antialiasing,
+                                false );
+#else
+        painter.setRenderHint( QPainter::Antialiasing, false );
 #endif
-                               QPainter::Antialiasing,
-                               false );
     }
 
     // draw future state
@@ -443,9 +447,9 @@ PlayerPainterRCSS::drawFuture( QPainter & painter,
 
     const int last = opt.playerFutureCycle();
 
-    QPoint first_point( opt.absScreenX( ppos.x ),
-                        opt.absScreenY( ppos.y ) ) ;
-    QPoint last_point = first_point;
+    QPointF first_point( opt.absScreenXF( ppos.x ),
+                         opt.absScreenYF( ppos.y ) ) ;
+    QPointF last_point = first_point;
 
     // draw kickable area edge
     rcsc::SideID side = param.player_.side();
@@ -470,38 +474,39 @@ PlayerPainterRCSS::drawFuture( QPainter & painter,
         ppos += pvel;
         pvel *= param.player_type_.playerDecay();
 
-        QPoint pt( opt.absScreenX( ppos.x ),
-                   opt.absScreenY( ppos.y ) );
-        if ( std::abs( last_point.x() - pt.x() ) < 1
-             && std::abs( last_point.y() - pt.y() ) < 1 )
+        QPointF pt( opt.absScreenXF( ppos.x ),
+                    opt.absScreenYF( ppos.y ) );
+        if ( std::fabs( last_point.x() - pt.x() ) < 1
+             && std::fabs( last_point.y() - pt.y() ) < 1 )
         {
             break;
         }
 
-        painter.drawEllipse( pt.x() - 1,
-                             pt.y() - 1,
-                             3,
-                             3 );
-        painter.drawEllipse( pt.x() - param.kick_radius_,
-                             pt.y() - param.kick_radius_,
-                             param.kick_radius_ * 2,
-                             param.kick_radius_ * 2 );
+        painter.drawEllipse( QRectF( pt.x() - 1,
+                                     pt.y() - 1,
+                                     3,
+                                     3 ) );
+        painter.drawEllipse( QRectF( pt.x() - param.kick_radius_,
+                                     pt.y() - param.kick_radius_,
+                                     param.kick_radius_ * 2,
+                                     param.kick_radius_ * 2 ) );
 
         last_point = pt;
     }
 
     // draw move line
     painter.setPen( dconf.debugTargetPen() );
-    painter.drawLine( first_point, last_point );
+    painter.drawLine( QLineF( first_point, last_point ) );
 
     if ( opt.antiAliasing() )
     {
-        painter.setRenderHints(
-#ifdef USE_GLWIDGET
-                               //QPainter::HighQualityAntialiasing |
+#ifdef USE_HIGH_QUALITY_ANTIALIASING
+        painter.setRenderHints( QPainter::HighQualityAntialiasing |
+                                QPainter::Antialiasing,
+                                true );
+#else
+        painter.setRenderHint( QPainter::Antialiasing, true );
 #endif
-                               QPainter::Antialiasing,
-                               true );
     }
 }
 
@@ -517,18 +522,16 @@ PlayerPainterRCSS::drawViewAreaBackground( QPainter & painter,
     const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
-    const int x = opt.screenX( player.x() );
-    const int y = opt.screenY( player.y() );
-    const int visible_radius = opt.scale( rcsc::ServerParam::i().visibleDistance() );
+    const double x = opt.screenXF( player.x() );
+    const double y = opt.screenYF( player.y() );
+    const double visible_radius = opt.scaleF( rcsc::ServerParam::i().visibleDistance() );
     const int view_start_angle
-        = static_cast< int >
-        ( rint( ( - player.head() - player.viewWidth()*0.5
-                  + ( opt.reverseSide() ? 180.0 : 0.0 ) ) * 16 ) );
-    const int view_width
-        = static_cast< int >( rint( player.viewWidth() * 16 ) );
+        = qRound( ( - player.head() - player.viewWidth()*0.5
+                    + ( opt.reverseSide() ? 180.0 : 0.0 ) ) * 16 );
+    const int view_width = qRound( player.viewWidth() * 16 );
 
-    const int TEAM_FAR = opt.scale( 40.0 );
-    const int TEAM_TOOFAR = opt.scale( 60.0 );
+    const double TEAM_FAR = opt.scaleF( 40.0 );
+    const double TEAM_TOOFAR = opt.scaleF( 60.0 );
 
     //painter.setRasterOp( Qt::OrROP );
     //QPainter::CompositionMode mode = painter.compositionMode();
@@ -538,31 +541,27 @@ PlayerPainterRCSS::drawViewAreaBackground( QPainter & painter,
     painter.setBrush( VIEW_AREA_TOOFAR_BRUSH );
     //painter.setOpacity( 0.2 );
 
-    painter.drawPie( x - TEAM_TOOFAR,
-                     y - TEAM_TOOFAR,
-                     TEAM_TOOFAR * 2,
-                     TEAM_TOOFAR * 2,
+    painter.drawPie( QRectF( x - TEAM_TOOFAR,
+                             y - TEAM_TOOFAR,
+                             TEAM_TOOFAR * 2,
+                             TEAM_TOOFAR * 2 ),
                      view_start_angle,
                      view_width );
-//     painter.drawEllipse( x - visible_radius,
-//                          y - visible_radius,
-//                          visible_radius * 2,
-//                          visible_radius * 2 );
 
-    painter.drawPie( x - visible_radius,
-                     y - visible_radius,
-                     visible_radius * 2,
-                     visible_radius * 2,
+    painter.drawPie( QRectF( x - visible_radius,
+                             y - visible_radius,
+                             visible_radius * 2,
+                             visible_radius * 2 ),
                      view_start_angle,
                      view_width - 360*16 );
 
     painter.setBrush( VIEW_AREA_BRUSH );
     //painter.setOpacity( 0.1 );
 
-    painter.drawPie( x - TEAM_FAR,
-                     y - TEAM_FAR,
-                     TEAM_FAR * 2,
-                     TEAM_FAR * 2,
+    painter.drawPie( QRectF( x - TEAM_FAR,
+                             y - TEAM_FAR,
+                             TEAM_FAR * 2,
+                             TEAM_FAR * 2 ),
                      view_start_angle,
                      view_width );
 
@@ -586,14 +585,14 @@ PlayerPainterRCSS::drawViewArea( QPainter & painter,
     painter.setPen( NECK_PEN );
     painter.setBrush( dconf.transparentBrush() );
 
-    int end_x = opt.absScreenX( param.player_.x() * opt.reverseValue()
-                                  + param.player_type_.kickableArea()
-                                  * std::cos( param.head_ * rcsc::AngleDeg::DEG2RAD ) );
-    int end_y = opt.absScreenY( param.player_.y() * opt.reverseValue()
-                                  + param.player_type_.kickableArea()
-                                  * std::sin( param.head_ * rcsc::AngleDeg::DEG2RAD ) );;
+    double end_x = opt.absScreenXF( param.player_.x() * opt.reverseValue()
+                                    + param.player_type_.kickableArea()
+                                    * std::cos( param.head_ * rcsc::AngleDeg::DEG2RAD ) );
+    double end_y = opt.absScreenYF( param.player_.y() * opt.reverseValue()
+                                    + param.player_type_.kickableArea()
+                                    * std::sin( param.head_ * rcsc::AngleDeg::DEG2RAD ) );;
 
-    painter.drawLine( param.x_, param.y_, end_x, end_y );
+    painter.drawLine( QLineF( param.x_, param.y_, end_x, end_y ) );
 }
 
 /*-------------------------------------------------------------------*/
@@ -606,6 +605,7 @@ PlayerPainterRCSS::drawCatchableArea( QPainter & painter,
 {
     const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
+    const rcsc::ServerParam & SP = rcsc::ServerParam::i();
 
     rcsc::SideID side = param.player_.side();
     if ( opt.reverseSide() )
@@ -613,16 +613,73 @@ PlayerPainterRCSS::drawCatchableArea( QPainter & painter,
         side = static_cast< rcsc::SideID >( -1 * side );
     }
 
-    int catchable = opt.scale( rcsc::ServerParam::i().catchableArea() );
+    double catchable = opt.scaleF( SP.catchableArea() );
     painter.setPen( ( side == rcsc::LEFT )
                     ? dconf.leftGoaliePen()
                     : dconf.rightGoaliePen() );
-    painter.setBrush( dconf.transparentBrush() );
+    painter.setBrush( dconf.transparentBrush() );;
 
-    painter.drawEllipse( param.x_ - catchable,
-                         param.y_ - catchable,
-                         catchable * 2,
-                         catchable * 2 );
+    painter.drawEllipse( QRectF( param.x_ - catchable,
+                                 param.y_ - catchable,
+                                 catchable * 2,
+                                 catchable * 2 ) );
+
+    double delta = SP.catchAreaLength() * std::max( 0.0, param.player_type_.catchAreaLengthStretch() - 1.0 );
+    double max_l = SP.catchAreaLength() + delta;
+    double min_l = SP.catchAreaLength() - delta;
+    double max_area = std::sqrt( std::pow( SP.catchAreaWidth() * 0.5, 2.0 )
+                                 + std::pow( max_l, 2.0 ) );
+    double min_area = std::sqrt( std::pow( SP.catchAreaWidth() * 0.5, 2.0 )
+                                 + std::pow( min_l, 2.0 ) );
+    double max_r = opt.scaleF( max_area );
+    if ( max_r > catchable )
+    {
+        painter.setPen( ( side == rcsc::LEFT )
+                        ? dconf.leftGoalieStretchPen()
+                        : dconf.rightGoalieStretchPen() );
+        painter.drawEllipse( QRectF( param.x_ - max_r,
+                                     param.y_ - max_r,
+                                     max_r * 2,
+                                     max_r * 2 ) );
+        double min_r = opt.scaleF( min_area );
+        painter.drawEllipse( QRectF( param.x_ - min_r,
+                                     param.y_ - min_r,
+                                     min_r * 2,
+                                     min_r * 2 ) );
+    }
+
+    //
+    // catch probability
+    //
+#if 0
+    const double ball_dist = std::sqrt( std::pow( param.player_.x() - param.ball_.x(), 2 )
+                                        + std::pow( param.player_.y() - param.ball_.y(), 2 ) );
+    double catch_prob = SP.catchProbability();
+    if ( ball_dist > stretch_area )
+    {
+        // catch_prob = 0.0;
+        return;
+    }
+
+    if ( ball_dist > SP.catchableArea() )
+    {
+        double x = ball_dist * ( stretch_l / stretch_area );
+        catch_prob
+            = SP.catchProbability()
+            - SP.catchProbability() * ( ( x - SP.catchAreaLength() )
+                                        / ( stretch_l - SP.catchAreaLength() ) );
+    }
+
+    int text_radius = std::min( 40, param.draw_radius_ );
+
+    painter.setPen( ( param.player_.side() == rcsc::LEFT )
+                    ? dconf.rightGoaliePen()
+                    : dconf.leftGoaliePen() );
+    painter.setFont( dconf.playerFont() );
+    painter.drawText( QPointF( param.x_ + text_radius,
+                               param.y_ + 4 + painter.fontMetrics().ascent() * 2 ),
+                      QString( "Catch=%1" ).arg( catch_prob, 0, 'g', 3 ) );
+#endif
 }
 
 /*-------------------------------------------------------------------*/
@@ -671,15 +728,15 @@ PlayerPainterRCSS::drawTackleArea( QPainter & painter,
         double right_x = SP.tackleWidth() * body_angle_side.cos();
         double right_y = SP.tackleWidth() * body_angle_side.sin();
 
-        QPoint pts[5];
-        pts[0].setX( opt.absScreenX( ppos.x + forward_x + right_x ) );
-        pts[0].setY( opt.absScreenY( ppos.y + forward_y + right_y ) );
-        pts[1].setX( opt.absScreenX( ppos.x + forward_x - right_x ) );
-        pts[1].setY( opt.absScreenY( ppos.y + forward_y - right_y ) );
-        pts[2].setX( opt.absScreenX( ppos.x + back_x - right_x ) );
-        pts[2].setY( opt.absScreenY( ppos.y + back_y - right_y ) );
-        pts[3].setX( opt.absScreenX( ppos.x + back_x + right_x ) );
-        pts[3].setY( opt.absScreenY( ppos.y + back_y + right_y ) );
+        QPointF pts[5];
+        pts[0].setX( opt.absScreenXF( ppos.x + forward_x + right_x ) );
+        pts[0].setY( opt.absScreenYF( ppos.y + forward_y + right_y ) );
+        pts[1].setX( opt.absScreenXF( ppos.x + forward_x - right_x ) );
+        pts[1].setY( opt.absScreenYF( ppos.y + forward_y - right_y ) );
+        pts[2].setX( opt.absScreenXF( ppos.x + back_x - right_x ) );
+        pts[2].setY( opt.absScreenYF( ppos.y + back_y - right_y ) );
+        pts[3].setX( opt.absScreenXF( ppos.x + back_x + right_x ) );
+        pts[3].setY( opt.absScreenYF( ppos.y + back_y + right_y ) );
         pts[4] = pts[0];
 
         painter.setPen( dconf.tackleAreaPen() );
@@ -712,8 +769,22 @@ void
 PlayerPainterRCSS::drawText( QPainter & painter,
                              const PlayerPainterRCSS::Param & param ) const
 {
+    static bool first = true;
+    static QFont font( "6x13bold", 9, QFont::Bold );
+    if ( first )
+    {
+        font.setPointSize( 9 );
+        font.setBold( true );
+        //font.setStyleHint( QFont::System, QFont::PreferBitmap );
+        font.setBold( true );
+        font.setFixedPitch( true );
+        first = false;
+    }
+    painter.setFont( font );
+
     const Options & opt = Options::instance();
 
+
     QString str;
 
     if ( opt.showPlayerNumber() )
@@ -750,22 +821,51 @@ PlayerPainterRCSS::drawText( QPainter & painter,
         str += QString::number( param.player_.type() );
     }
 
+    const double text_radius = std::min( 40.0, param.draw_radius_ );
+    int card_offset = 0;
+
+    if ( opt.showCard()
+         && ( param.player_.hasRedCard()
+              || param.player_.hasYellowCard() ) )
+    {
+        QFontMetrics fm = painter.fontMetrics();
+        int x_size = std::min( std::max( 4, fm.ascent() - 2 ), 20 );
+        int y_size = std::min( std::max( 4, fm.ascent() ), 24 );
+
+        card_offset = x_size + 2;
+
+        if ( opt.antiAliasing() )
+        {
+#ifdef USE_HIGH_QUALITY_ANTIALIASING
+            painter.setRenderHints( QPainter::HighQualityAntialiasing |
+                                    QPainter::Antialiasing,
+                                    false );
+#else
+            painter.setRenderHint( QPainter::Antialiasing, false );
+#endif
+        }
+        painter.setPen( Qt::black );
+        painter.setBrush( param.player_.hasRedCard()
+                          ? Qt::red
+                          : Qt::yellow );
+        painter.drawRect( QRectF( param.x_ + text_radius,
+                                  param.y_ - y_size,
+                                  x_size,
+                                  y_size ) );
+        if ( opt.antiAliasing() )
+        {
+#ifdef USE_HIGH_QUALITY_ANTIALIASING
+            painter.setRenderHints( QPainter::HighQualityAntialiasing |
+                                    QPainter::Antialiasing,
+                                    true );
+#else
+            painter.setRenderHint( QPainter::Antialiasing, true );
+#endif
+        }
+    }
+
     if ( ! str.isEmpty() )
     {
-        static bool first = true;
-        static QFont font( "6x13bold", 9, QFont::Bold );
-        if ( first )
-        {
-            font.setPointSize( 9 );
-            font.setBold( true );
-            //font.setStyleHint( QFont::System, QFont::PreferBitmap );
-            font.setBold( true );
-            font.setFixedPitch( true );
-            first = false;
-        }
-
-        painter.setFont( font );
-
         rcsc::SideID side = param.player_.side();
         if ( opt.reverseSide() )
         {
@@ -775,6 +875,8 @@ PlayerPainterRCSS::drawText( QPainter & painter,
         painter.setPen( side == rcsc::LEFT
                         ? LEFT_TEAM_PEN
                         : RIGHT_TEAM_PEN );
-        painter.drawText( param.x_, param.y_, str );
+        painter.drawText( QPointF( param.x_ + text_radius + card_offset,
+                                   param.y_ ),
+                          str );
     }
 }
