@@ -91,14 +91,14 @@ ViewConfigDialog::createWidgets()
 
         layout->addWidget( createZoomControls(),
                            0, Qt::AlignLeft );
-        layout->addWidget( createObjectSizeControls(),
-                           0, Qt::AlignLeft );
         layout->addWidget( createCanvasSizeControls(),
                            0, Qt::AlignLeft );
         layout->addWidget( createFieldStyleControls(),
                            0, Qt::AlignLeft );
         layout->addWidget( createMiscControls(),
                            0, Qt::AlignLeft );
+        layout->addWidget( createMouseMeasureControls(),
+                           0, Qt::AlignLeft );
 
         frame->setLayout( layout );
         M_tab_widget->addTab( frame, tr( "Canvas" ) );
@@ -125,7 +125,7 @@ ViewConfigDialog::createWidgets()
         frame->setLayout( layout );
         M_tab_widget->addTab( frame, tr( "Show" ) );
     }
-    // trace / future
+    // object features
     {
         QFrame * frame = new QFrame();
         QVBoxLayout * layout = new QVBoxLayout();
@@ -133,6 +133,8 @@ ViewConfigDialog::createWidgets()
         layout->setMargin( 4 );
         layout->setSpacing( 4 );
 
+        layout->addWidget( createObjectSizeControls(),
+                           0, Qt::AlignLeft );
         layout->addWidget( createFocusControls(),
                            0, Qt::AlignLeft );
         layout->addWidget( createPlayerSelectionControls(),
@@ -143,7 +145,7 @@ ViewConfigDialog::createWidgets()
                            0, Qt::AlignLeft );
 
         frame->setLayout( layout );
-        M_tab_widget->addTab( frame, tr( "Trace" ) );
+        M_tab_widget->addTab( frame, tr( "Object" ) );
     }
 
     top_layout->addWidget( M_tab_widget );
@@ -188,7 +190,7 @@ ViewConfigDialog::createZoomControls()
         QPushButton * fit_button = new QPushButton( tr( "Fit" ) );
         fit_button->setMaximumSize( 60, button_height );
         connect( fit_button, SIGNAL( clicked() ),
-                 this, SLOT( unzoom() ) );
+                 this, SLOT( fitToScreen() ) );
         layout->addWidget( fit_button );
 
         top_layout->addLayout( layout );
@@ -458,6 +460,12 @@ ViewConfigDialog::createPlayersDetailControls()
         connect( M_attentionto_cb, SIGNAL( clicked( bool ) ),
                  this, SLOT( clickShowAttentionto( bool ) ) );
         layout->addWidget( M_attentionto_cb );
+        //
+        M_card_cb = new QCheckBox( tr( "Card" ) );
+        M_card_cb->setChecked( opt.showCard() );
+        connect( M_card_cb, SIGNAL( clicked( bool ) ),
+                 this, SLOT( clickShowCard( bool ) ) );
+        layout->addWidget( M_card_cb );
 
         top_layout->addLayout( layout );
     }
@@ -505,11 +513,11 @@ ViewConfigDialog::createShowControls()
                  this, SLOT( clickShowScoreBoard( bool ) ) );
         layout->addWidget( M_show_score_board_cb );
         //
-        M_show_team_logo_cb = new QCheckBox( tr( "Team Logo" ) );
-        M_show_team_logo_cb->setChecked( opt.showTeamLogo() );
-        connect( M_show_team_logo_cb, SIGNAL( clicked( bool ) ),
-                 this, SLOT( clickShowTeamLogo( bool ) ) );
-        layout->addWidget( M_show_team_logo_cb );
+        M_show_team_graphic_cb = new QCheckBox( tr( "Team Graphic" ) );
+        M_show_team_graphic_cb->setChecked( opt.showTeamGraphic() );
+        connect( M_show_team_graphic_cb, SIGNAL( clicked( bool ) ),
+                 this, SLOT( clickShowTeamGraphic( bool ) ) );
+        layout->addWidget( M_show_team_graphic_cb );
         //
         M_show_flags_cb = new QCheckBox( tr( "Flag" ) );
         M_show_flags_cb->setChecked( opt.showFlags() );
@@ -810,7 +818,7 @@ ViewConfigDialog::createMiscControls()
     top_layout->setMargin( 1 );
     top_layout->setSpacing( 0 );
 
-    M_cursor_hide_cb = new QCheckBox( tr( "Audomatic Cursor Hide" ) );
+    M_cursor_hide_cb = new QCheckBox( tr( "Automatic Cursor Hide" ) );
     M_cursor_hide_cb->setChecked( opt.cursorHide() );
     connect( M_cursor_hide_cb, SIGNAL( toggled( bool ) ),
              this, SLOT( clickCursorHide( bool ) ) );
@@ -1098,6 +1106,56 @@ ViewConfigDialog::createInertiaMoveControls()
     return group_box;
 }
 
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+QWidget *
+ViewConfigDialog::createMouseMeasureControls()
+{
+    QGroupBox * group_box = new QGroupBox( tr( "Mouse Measure" ) );
+
+    QHBoxLayout * top_layout = new QHBoxLayout();
+    top_layout->setMargin( 0 );
+    top_layout->setSpacing( 0 );
+
+
+    top_layout->addWidget( new QLabel( tr( "Mode:" ) ) );
+    top_layout->addSpacing( 2 );
+    {
+        M_mouse_measure_choice = new QComboBox();
+        M_mouse_measure_choice->addItem( tr( "None" ) );
+        M_mouse_measure_choice->addItem( tr( "Ball Move" ) );
+        M_mouse_measure_choice->addItem( tr( "Player Move" ) );
+        M_mouse_measure_choice->setMaximumSize( this->fontMetrics().width( tr( "Player Move" ) ) + 32,
+                                                this->fontMetrics().height() + 12 );
+        M_mouse_measure_choice->setCurrentIndex( static_cast< int >( Options::MEASURE_BALL_MOVE ) );
+        connect( M_mouse_measure_choice, SIGNAL( currentIndexChanged( int ) ),
+                 this, SLOT( selectMouseMeasureMode( int ) ) );
+        top_layout->addWidget( M_mouse_measure_choice );
+    }
+
+    top_layout->addSpacing( 6 );
+
+    top_layout->addWidget( new QLabel( tr( "First Length:" ) ) );
+    top_layout->addSpacing( 2 );
+    {
+        M_mouse_measure_first_length = new QDoubleSpinBox();
+        M_mouse_measure_first_length->setRange( -1.0, 5.0 );
+        M_mouse_measure_first_length->setValue( -1.0 );
+        M_mouse_measure_first_length->setDecimals( 3 );
+        M_mouse_measure_first_length->setSingleStep( 0.01 );
+        connect( M_mouse_measure_first_length, SIGNAL( valueChanged( double ) ),
+                 this, SLOT( changeMouseMeasureFirstLength( double ) ) );
+        top_layout->addWidget( M_mouse_measure_first_length );
+    }
+
+    group_box->setLayout( top_layout );
+    return group_box;
+}
+
+
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -1138,10 +1196,11 @@ ViewConfigDialog::updateAll()
     M_kick_accel_area_cb->setChecked( opt.showKickAccelArea() );
     M_pointto_cb->setChecked( opt.showPointto() );
     M_attentionto_cb->setChecked( opt.showAttentionto() );
+    M_card_cb->setChecked( opt.showCard() );
 
     M_anonymous_mode_cb->setChecked( opt.anonymousMode() );
     M_show_score_board_cb->setChecked( opt.showScoreBoard() );
-    M_show_team_logo_cb->setChecked( opt.showTeamLogo() );
+    M_show_team_graphic_cb->setChecked( opt.showTeamGraphic() );
     M_show_ball_cb->setChecked( opt.showBall() );
     M_show_players_cb->setChecked( opt.showPlayers() );
     M_show_flags_cb->setChecked( opt.showFlags() );
@@ -1234,6 +1293,10 @@ ViewConfigDialog::updateAll()
 
     M_ball_future->setValue( opt.ballFutureCycle() );
     M_player_future->setValue( opt.playerFutureCycle() );
+
+
+    M_mouse_measure_choice->setCurrentIndex( static_cast< int >( opt.mouseMeasureMode() ) );
+    M_mouse_measure_first_length->setValue( opt.mouseMeasureFirstLength() );
 }
 
 /*-------------------------------------------------------------------*/
@@ -1287,9 +1350,9 @@ ViewConfigDialog::zoomOut()
 
  */
 void
-ViewConfigDialog::unzoom()
+ViewConfigDialog::fitToScreen()
 {
-    Options::instance().unzoom();
+    Options::instance().fitToScreen();
     updateFieldScale();
     M_focus_fix_rb->setChecked( true );
 
@@ -1796,6 +1859,33 @@ ViewConfigDialog::toggleShowAttentionto()
     emit configured();
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+void
+ViewConfigDialog::clickShowCard( bool checked )
+{
+    if ( Options::instance().showCard() != checked )
+    {
+        Options::instance().toggleShowCard();
+        emit configured();
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+void
+ViewConfigDialog::toggleShowCard()
+{
+    Options::instance().toggleShowCard();
+    M_card_cb->setChecked( Options::instance().showCard() );
+
+    emit configured();
+}
+
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -1855,11 +1945,11 @@ ViewConfigDialog::toggleShowScoreBoard()
 
  */
 void
-ViewConfigDialog::clickShowTeamLogo( bool checked )
+ViewConfigDialog::clickShowTeamGraphic( bool checked )
 {
-    if ( Options::instance().showTeamLogo() != checked )
+    if ( Options::instance().showTeamGraphic() != checked )
     {
-        Options::instance().toggleShowTeamLogo();
+        Options::instance().toggleShowTeamGraphic();
         emit configured();
     }
 }
@@ -1869,10 +1959,10 @@ ViewConfigDialog::clickShowTeamLogo( bool checked )
 
  */
 void
-ViewConfigDialog::toggleShowTeamLogo()
+ViewConfigDialog::toggleShowTeamGraphic()
 {
-    Options::instance().toggleShowTeamLogo();
-    M_show_team_logo_cb->setChecked( Options::instance().showTeamLogo() );
+    Options::instance().toggleShowTeamGraphic();
+    M_show_team_graphic_cb->setChecked( Options::instance().showTeamGraphic() );
 
     emit configured();
 }
@@ -2895,3 +2985,32 @@ ViewConfigDialog::changePlayerFutureCycle( int value )
 
     emit configured();
 }
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+void
+ViewConfigDialog::selectMouseMeasureMode( int index )
+{
+    Options::MouseMeasureMode mode = static_cast< Options::MouseMeasureMode >( index );
+    if ( Options::instance().mouseMeasureMode() != mode )
+    {
+        Options::instance().setMouseMeasureMode( mode );
+        emit configured();
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+void
+ViewConfigDialog::changeMouseMeasureFirstLength( double value )
+{
+    if ( std::fabs( Options::instance().mouseMeasureFirstLength() - value ) > 1.0e-5 )
+    {
+        Options::instance().setMouseMeasureFirstLength( value );
+        emit configured();
+    }
+}
