@@ -34,10 +34,10 @@
 
 #include <QDialog>
 
-class QPushButton;
 class QCheckBox;
 class QCloseEvent;
 class QComboBox;
+class QDoubleSpinBox;
 class QLabel;
 class QLineEdit;
 class QPushButton;
@@ -94,11 +94,12 @@ private:
     QCheckBox * M_kick_accel_area_cb;
     QCheckBox * M_pointto_cb;
     QCheckBox * M_attentionto_cb;
+    QCheckBox * M_card_cb;
 
     // show/hide control
     QCheckBox * M_anonymous_mode_cb;
     QCheckBox * M_show_score_board_cb;
-    QCheckBox * M_show_team_logo_cb;
+    QCheckBox * M_show_team_graphic_cb;
     QCheckBox * M_show_ball_cb;
     QCheckBox * M_show_players_cb;
     QCheckBox * M_show_flags_cb;
@@ -154,6 +155,10 @@ private:
     QSpinBox * M_ball_future;
     QSpinBox * M_player_future;
 
+    // mouse measure
+    QComboBox * M_mouse_measure_choice;
+    QDoubleSpinBox * M_mouse_measure_first_length;
+
 public:
 
     ViewConfigDialog( QWidget * parent,
@@ -177,6 +182,7 @@ private:
     QWidget * createPlayerSelectionControls();
     QWidget * createTraceControls();
     QWidget * createInertiaMoveControls();
+    QWidget * createMouseMeasureControls();
 
 
 protected:
@@ -212,10 +218,11 @@ private slots:
     void clickShowKickAccelArea( bool checked );
     void clickShowPointto( bool checked );
     void clickShowAttentionto( bool checked );
+    void clickShowCard( bool checked );
 
     void clickAnonymousMode( bool checked );
     void clickShowScoreBoard( bool checked );
-    void clickShowTeamLogo( bool checked );
+    void clickShowTeamGraphic( bool checked );
     void clickShowBall( bool checked );
     void clickShowPlayers( bool checked );
     void clickShowFlags( bool checked );
@@ -269,6 +276,9 @@ private slots:
     void changeBallFutureCycle( int value );
     void changePlayerFutureCycle( int value );
 
+    void selectMouseMeasureMode( int index );
+    void changeMouseMeasureFirstLength( double value );
+
 public slots:
 
     // just update this dialog contents
@@ -279,7 +289,7 @@ public slots:
     // emit configured()
     void zoomIn();
     void zoomOut();
-    void unzoom();
+    void fitToScreen();
 
     void toggleEnlarge();
     void editBallSize( const QString & text );
@@ -301,10 +311,11 @@ public slots:
     void toggleShowKickAccelArea();
     void toggleShowPointto();
     void toggleShowAttentionto();
+    void toggleShowCard();
 
     void toggleAnonymousMode();
     void toggleShowScoreBoard();
-    void toggleShowTeamLogo();
+    void toggleShowTeamGraphic();
     void toggleShowPlayers();
     void toggleShowBall();
     void toggleShowFlags();
